--[[----------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2015 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

	-- WF.core.reactive
local KSReactive = require "KSReactive"

	-- WF.core.utils
local KSTableUtils = require 'KSTableUtils'
local WFUnknown = require 'WFUnknown'

	-- WF trash
local WFCatTrash = require 'WFCatTrash'

	-- Lrmobile_models
local LriAllPhotosQueryBuilder = require 'LriAllPhotosQueryBuilder'

--------------------------------------------------------------------------------

local LriAllPhotosAlbumModel = {}
local Model = {}

--------------------------------------------------------------------------------

function Model:assetListModel( sortCriteria, ascending, assetFilter, assetTypeFilter, keyword, edited, ratingFilter, ratingComparisonOperator, isSearch )

	local activeFilter = {}
	
	for i = 1, #assetFilter do
		local filter = assetFilter[ i ]
		activeFilter[ filter ] = true
	end
	
	for i = 1, #assetTypeFilter do
		local assetType = assetTypeFilter[ i ]
		activeFilter[ assetType ] = true
	end
	
	for i = 1, #edited do
		activeFilter.edited = edited[ i ]
	end
	
	
	self.catalog:docStore():applyOverlayToDocumentP( {
		docType	= 'gen-album',
		docId 	= self.albumId,
		overlay	= 
		{
			_localOnly = 
			{
				filter = 
				{
					_replace 					= true,
					image						= activeFilter.image,
					video						= activeFilter.video,
					pick 						= activeFilter.pick,
					reject 						= activeFilter.reject,
					unflagged					= activeFilter.unflagged,
					favorite 					= activeFilter.favorite,
					comment 					= activeFilter.comment,
					keyword						= keyword,
					edited						= activeFilter.edited,
					rating 						= ratingFilter,
					ratingComparisonOperator	= ratingComparisonOperator,
					sortCriteria 				= sortCriteria,
					ascending 					= ascending
				},
			},
		}
	} ):runUntilDone()
	
	local sql, queryParams	= LriAllPhotosQueryBuilder.buildCoreInfoQuery (self.albumId, ascending, assetFilter, assetTypeFilter, keyword, activeFilter.edited, sortCriteria, ratingFilter, ratingComparisonOperator, isSearch)
		
	local encoder 			= 'LrmOzAssetCoreInfo.encodeSql'
											
    local result 			= self.catalog:indexStore():query
	{ 
		sql 	= sql, 
		encoder	= encoder,
		calm 	= .125,
		unpack( queryParams )
	}

	return 
	{
		value = result,
	}
	
end

--------------------------------------------------------------------------------

function Model:trashAssetListModel()

	local isSearch = false
	local isTrash = true
	
	local fieldsToReturn = LriAllPhotosQueryBuilder.getCollectionInfoFields( isSearch, isTrash )
	local encoder = 'LrmOzAssetCoreInfo.encodeSql'
	
	
	local getMostImmediatePurgeDateFunction = function( assetCoreInfoList )
	
		local count = assetCoreInfoList:getCount()
		local mostImmediatePurgeDate
		
		if count and count > 0 then
			
			local asset = assetCoreInfoList:getAsset( 1 )
			mostImmediatePurgeDate = asset.purgeDate
			
		end
		
		return mostImmediatePurgeDate
		
	end
			
	local value = WFCatTrash.trashAssetList( self.catalog, fieldsToReturn, getMostImmediatePurgeDateFunction, encoder )

	return {

		value = value
			:map( function( rows )
			
				return rows
				
			end )
		
	}

end

--------------------------------------------------------------------------------

function Model:trashAssetCountModel()

	local value = WFCatTrash.countTrashAssets( self.catalog )

	return {

		value = value
			:map( function( rows )
			
				return rows
				
			end )
		
	}

end

--------------------------------------------------------------------------------

function Model:trashAssetsFileSizeModel( assetIds )
	
	local value = WFCatTrash.trashAssetsFileSize( self.catalog, assetIds )
	
	return {

		value = value
			:map( function( rows )
			
				return rows
				
			end )
		
	}
	
end

--------------------------------------------------------------------------------

function Model:trashAssetsTotalFileSizeModel()

	local value = WFCatTrash.trashAssetsTotalFileSize( self.catalog )
	
	return {

		value = value
			:map( function( rows )
			
				return rows
				
			end )
		
	}

end

--------------------------------------------------------------------------------

local function makeFlagStatus( self, rows )

	local flags = {
		unflagged = 0,
		pick = 0,
		reject = 0,
	}
	
	for i=1, #rows do
	
		local row = rows[i]
	
		if row[1] == 'unflagged' then	
		
			flags.unflagged = row[2]
		
	  	elseif row[1] == 'pick' then
		
			flags.pick = row[2]
			
	  	elseif row[1] == 'reject' then	
			
			flags.reject = row[2]
			
	  	end
		
	end
	
	return flags
	
end

--------------------------------------------------------------------------------

local function flagsB( self, catalog, albumId )

	local sql 		= [[SELECT flag, COUNT(docId) FROM coreInfo GROUP BY flag]]

	return catalog:ozUserIdP()
		:untilResolvedB( 'data', WFUnknown.value() )
		:closingMapSwitchB( function( userId )
		
			return catalog:indexStore()
	
				:queryB (sql, userId)
			
				:closingMapB ( 
					function( rows )
						
						if rows == nil or WFUnknown.isUnknown( rows ) then
							return WFUnknown.value()
						end
			
						return makeFlagStatus( self, rows )
		
					end )
			
		end )
		
	
end

--------------------------------------------------------------------------------

function Model:assetListWithSearch()
		
	local sql, queryParams	= LriAllPhotosQueryBuilder.buildCoreInfoQueryWithSearch ()
		
	local encoder 			= 'LrmOzAssetCoreInfo.encodeSql'
											
    local resultB 			= self.catalog:indexStore():queryB 
	{ 
		sql 	= sql, 
		nArgs 	= #queryParams,
		encoder	= encoder,
		unpack( queryParams )
	}

	return 
	{
		valueB = resultB,
	}
	
end

--------------------------------------------------------------------------------

function Model:flagStatusCountModel()

	return {
		valueB = flagsB( self, self.catalog, self.albumId )
	}
end

--------------------------------------------------------------------------------
function Model:coverImageModel()

	local sql = [[
		SELECT coreInfo.docId 
		FROM coreInfo 
		ORDER BY captureDate DESC 
		LIMIT 1]]

		
	local resultB = self.catalog:indexStore()

	:queryB( { 
		sql = sql,
		calm = 30
	} )
		
	:closingMapB( function ( rows )

		if WFUnknown.isUnknown( rows ) then
			return rows
		end

		local coverAssetId = rows and rows[ 1 ] and rows[ 1 ][ 1 ]
		
		return coverAssetId
		
	end )

	return {
		valueB = resultB
	}
end

--------------------------------------------------------------------------------
function Model:alternativeCoverAssetId()

	local sql = [[SELECT coreInfo.docId FROM coreInfo WHERE coreInfo.docId NOT IN (SELECT assetId FROM ozUploadStatus WHERE docType == 'asset') ORDER BY captureDate DESC LIMIT 1]]
					
	local coverImageAssetIdB = self.catalog:indexStore()
	
		:queryB ( sql )

		:closingMapB( 
			function ( rows )
		
				local coverAssetId = rows and rows[ 1 ] and rows[ 1 ][ 1 ]

				return coverAssetId
			
			end )
			
		:closingFilterB( 
			function( val ) 
				return val ~= nil 
			end, 
			WFUnknown.value() )

	return {
		valueB = coverImageAssetIdB,
	}
	
end

--------------------------------------------------------------------------------

function LriAllPhotosAlbumModel.makeModel( catalog, albumId )
	
	local resultB = KSReactive.variableB( WFUnknown.value() )
	
	catalog:docStore():docValueOnceP( 'gen-album', albumId )

	:whenResolvedDo( function( status, docValue )
	
		if status == 'error' then
			status = 'hallo'
		end
		
		if status == 'data' or #albumId ~=32 then
		
			local filter = KSTableUtils.derefOpt( docValue, "_localOnly", "filter" )
			
			if filter then
				
				local deref = KSTableUtils.derefOpt
				
				local activeFilter = {
					image = deref( filter, "image" ),
					video = deref( filter, "video" ),
					pick = deref( filter, "pick" ),
					reject = deref( filter, "reject" ),
					unflagged = deref( filter, "unflagged" ),
					favorite = deref( filter, "favorite" ),
					comment = deref( filter, "comment" ),
					rating = deref( filter, "rating" ),
					ratingComparisonOperator = deref( filter, "ratingComparisonOperator" ),
					sortCriteria = deref( filter, "sortCriteria" ),
					ascending = deref( filter, "ascending" ),
				}
				
				resultB:set( activeFilter )
			else
				resultB:set( nil )
			end
		end
		
	end ):runUntilDone()
	
	return {
		catalog = catalog,
		albumId = albumId,
		valueB = resultB,
		models = Model,
		
		events = {
		}
	}
end

--------------------------------------------------------------------------------

return LriAllPhotosAlbumModel
