--[[----------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2019 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

local Rx = require "Rx"

local OzFilePaths = require "OzFilePaths"
local KSPathUtils = require "KSPathUtils"

local WFRxHttpServerUtils = require "WFRxHttpServerUtils"

--------------------------------------------------------------------------------

local typeToPrefix = {
	master = '',
	preview = 'pv_',
	thumbnail2x= 'th_',
	proxy = 'proxy_'
}

local contentTypeMap = {
	jpg = 'image/jpeg',
	jpeg = 'image/jpeg',
	png = 'image/png',
	tiff = 'image/tiff',
	gif = 'image/gif',
	mpeg = 'video/mpeg',
	mpg = 'video/mpeg',
	mp4 = 'video/mp4',
	qt = 'video/quicktime',
	mov = 'video/quicktime',
	avi = 'video/x-msvideo',
	ogg = 'video/ogg',
	ogv = 'video/ogg'
}

--------------------------------------------------------------------------------

local handlers = {
	{ '^/assets/(%x+)/(%x+)/(%x+)/(%w+)\.(%w+)$', { 'GET' }, function( catalog, path, info )
				
		if #info.matches ~= 5 then
			return Rx.Observable.just{ statusCode = 400, data = 'Bad Request' }
		end

		for i = 1, 2 do
			if #info.matches[ i ] ~= 32 then
				return Rx.Observable.just{ statusCode = 400, data = 'Bad Request' }
			end
		end

		if #info.matches[ 3 ] ~= 32 and #info.matches[ 3 ] ~= 64 then
			return Rx.Observable.just{ statusCode = 400, data = 'Bad Request' }
		end

		local catalogId, assetId, contentId, renditionName, ext = unpack( info.matches )

		if catalogId ~= catalog:ozCatalogSpec():catalogId() then
			return Rx.Observable.just{ statusCode = 400, data = 'Bad Request' }
		end
		
		local contentType = 'application/octet-stream'

		if ext then

			contentType = contentTypeMap[ ext:lower() ]
			if not contentType then
				contentType = 'application/octet-stream'
			end

		end

		local basePath = OzFilePaths.getCatalogBasePath( catalogId )
		local filePath


		filePath = OzFilePaths.getRelativeBlobPath( assetId, catalogId )
		filePath = KSPathUtils.child( basePath, filePath )

		local prefix = typeToPrefix[ renditionName ]

		if not prefix then
			return Rx.Observable.just{ statusCode = 400, data = 'Bad Request' }
		end

		filePath = KSPathUtils.child( filePath, prefix .. contentId .. '.' .. ext )

		return Rx.Observable.just { contentType = contentType, filePath = filePath }

	end },
}

local LriP2PHttpServerHandlers = {}

--------------------------------------------------------------------------------

function LriP2PHttpServerHandlers.startRequest( catalog, uri, query, remoteAddr, method, postData )

	return { 
		value = WFRxHttpServerUtils.startRequest( handlers, catalog, uri, query, remoteAddr, method, postData )
	}

end

return LriP2PHttpServerHandlers