--[[----------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2013 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

local Rx = require "Rx"

	-- WF.core.utils
local KSTableUtils = require 'KSTableUtils'

	--WF.config
local LrmConfig = require 'LrmConfig'

--------------------------------------------------------------------------------

local derefOpt = KSTableUtils.derefOpt

--============================================================================--

local LriSyncStatusModel = {}
local Model = {}

--------------------------------------------------------------------------------

local function getUploadProgress( catalog )

	local numNeedingUpload = catalog:indexStore():query {

		sql = [[SELECT
			count( DISTINCT sourceAssetId )
		FROM
			needsBinaryUpload
		WHERE
			binaryType = "master"
		]],
		calm = 2,
		quiesceUntilUpToDate = true
	}

	:map( function( progressRows )

		local binaryUpload = derefOpt( progressRows, 1, 1 ) or 0

		return binaryUpload

	end )

	return numNeedingUpload

end

--------------------------------------------------------------------------------

local function getDetailedUploadProgress( catalog )

	local uploadProgressStaus = catalog:getSyncClient():getDetailedUploadProgress()
	:map( function ( progressReport ) 
		return {
			totalFileSize = progressReport.totalBytesNeedToUpload,
			bytesUploaded = progressReport.totalBytesUploadedSoFar
		}
	end )

	return uploadProgressStaus

end

--------------------------------------------------------------------------------

-- @mark Sub models

--------------------------------------------------------------------------------

function Model:makeBinaryDownloadProgressAndAlbumModel( binaryDownloadStrategy )

	binaryDownloadStrategy = binaryDownloadStrategy or LrmConfig.StrategyPreferProxy

	local masterSql =  [[SELECT albums.docId, albums.name, count( DISTINCT r.docId ) FROM
					renditionRevisions as r
				INNER JOIN
					album_asset
				ON
					album_asset.assetId = r.docId
				INNER JOIN
					albums
				ON
					albums.docId = album_asset.albumId and albums.useOffline = 1
				WHERE
					renditionName IN ( 'thumbnail2x', 'preview', 'proxyWithoutMaster', 'master', 'xmp_develop' )
				]]

	local proxySql = [[SELECT albums.docId, albums.name, count( DISTINCT r.docId ) FROM
					renditionRevisions as r
				INNER JOIN
					album_asset
				ON
					album_asset.assetId = r.docId
				INNER JOIN
					albums
				ON
					albums.docId = album_asset.albumId and albums.useOffline = 1
				WHERE
					renditionName IN ( 'thumbnail2x', 'preview', 'proxy', 'xmp_develop' )
				]]

	local contributedSql = [[SELECT albums.docId, albums.name, count( DISTINCT r.docId ) FROM
					renditionRevisions as r
				INNER JOIN
					album_asset
				ON
					album_asset.assetId = r.docId
				INNER JOIN
					albums
				ON
					albums.docId = album_asset.albumId and albums.useOffline = 1
				WHERE
					albums.spaceId IS NOT NULL AND renditionName IN ( 'thumbnail2x', 'preview', 'proxy', 'xmp_develop' )
				]]

	local sql

	if binaryDownloadStrategy == LrmConfig.StrategyPreferMaster then
		sql = masterSql
	else
		sql = proxySql
	end

	local progress = Rx.Observable.combineLatest( self._catalog:indexStore():query( { sql = sql } ),
												  self._catalog:indexStore():query( { sql = contributedSql } ) )

		:map( function( normalRows, contributedRows )

			local downloadingAlbums = {}

			for _, rows in ipairs { normalRows, contributedRows } do

				for _, row in ipairs( rows ) do

					if row[ 1 ] then
						-- map from album id to album name and count
						downloadingAlbums[ row[ 1 ] ] = {
							albumName = row[ 2 ],
							count = row[ 3 ]
						}
					end
				end
			end
			
			return downloadingAlbums

		end )

	return {
		value = progress,
	}

end

--------------------------------------------------------------------------------

function LriSyncStatusModel.makeModel( catalog )

	assert( catalog )

	local uploadingToOzProgress = getUploadProgress( catalog )

	return {
		_catalog = catalog,

		value = {
			isUploading = catalog:getSyncClient():isPushingMetadata(),
			isDownloading = catalog:getSyncClient():isPullingMetadata(),
			pendingUploadsToOz = uploadingToOzProgress,
			progressBinaryUploads = getDetailedUploadProgress( catalog ),
		},

		models = Model,

		events = {
		},

		close = function( self )


		end
	}

end

--------------------------------------------------------------------------------

return LriSyncStatusModel
