--[[----------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2013 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

	-- WF.core.utils
local WFCrypto = require 'WFCrypto'
local WFUnknown = require 'WFUnknown'
local KSTableUtils = require 'KSTableUtils'

	-- WF.core.reactive.	
local KSReactive = require "KSReactive"

	-- WF.oz_client.catalog
local WFOzContentTypes = require 'WFOzContentTypes'

	-- Lrmobile_models
local LrmAlbumUtils = require 'LrmAlbumUtils'
local LrmAlbumQueryBuilder = require 'LrmAlbumQueryBuilder'
local WFPlatformInfo = require 'WFPlatformInfo'
--==============================================================================

local LrmAlbumModel = {}
local Model = {}

--------------------------------------------------------------------------------

local function makeCollectionInfo( name, originalPhotoRows, eventName, startDate, endDate, eventNameFirst )

	if not originalPhotoRows or #originalPhotoRows == 0 then
		local photosUnknown = originalPhotoRows == nil or WFUnknown.isUnknown( originalPhotoRows ) or nil
		return { name = name, photos = { }, eventName = eventName, startDate = startDate, endDate = endDate, assetCount = 0,
				 photosUnknown = photosUnknown }
	end
				
	local nPhotos = #originalPhotoRows
	
	local collectionInfo = {
		name = name,
		photos = {},
		eventName = eventName,
		startDate = startDate,
		endDate = endDate,
		eventNameFirst = eventNameFirst,
		assetCount = nPhotos,
	}
	
	local photos = collectionInfo.photos
	
	local countCollectionInfoFields = LrmAlbumQueryBuilder.kCountCollectionInfoFields
	
	for i = 1, nPhotos do
		
		local originalPhotoRow = originalPhotoRows[ i ]

		if WFPlatformInfo.isAndroid then
		-- Android needs only limited set to save memory and improve performance
		photos[ i ] = {
			id = originalPhotoRow[ 1 ], -- coreInfo.docId
			croppedWidth = originalPhotoRow[ 2 ],
			croppedHeight = originalPhotoRow[ 3 ],
			captureDate = originalPhotoRow[ 4 ], --captureDate
			fileName = originalPhotoRow[ 5 ],
			modifiedDate = originalPhotoRow[ 6 ], -- modifiedDate
			thumbnailMD5 = originalPhotoRow[ 7 ], -- thumbnailMD5
			importDate = originalPhotoRow[ 8 ], -- importDate
			hasThumbnail = originalPhotoRow[ 9 ],
			isVideo = originalPhotoRow[ 10 ],
			duration = originalPhotoRow[ 11 ],
			isPopulated = true,
		}
		else
		photos[ i ] = {
			id = originalPhotoRow[ 1 ], -- coreInfo.docId
			captureDate = originalPhotoRow[ 2 ], --captureDate
			modifiedDate = originalPhotoRow[ 18 ], -- modifiedDate
			contentType = WFOzContentTypes.contentTypeForContentTypeNumber( originalPhotoRow[ 3 ], true ),
			flag = originalPhotoRow[ 6 ],
			croppedWidth = originalPhotoRow[ 7 ],
			croppedHeight = originalPhotoRow[ 8 ],
			fileName = originalPhotoRow[ 9 ],
			width = originalPhotoRow[ 10 ],
			height = originalPhotoRow[ 11 ],
			apertureValue = originalPhotoRow[ 12 ],
			isoSpeedRatings = originalPhotoRow[ 13 ],
			exposureTime = originalPhotoRow[ 14 ],
			cameraMaker = originalPhotoRow[ 15 ],
			cameraModel = originalPhotoRow[ 16 ],
			rating = originalPhotoRow[ 17 ],
			commentCount = originalPhotoRow[ countCollectionInfoFields + 1 ],
			favoriteCount = originalPhotoRow[ countCollectionInfoFields + 2 ],
			isVideo = originalPhotoRow[ 19 ],
			localUrl = originalPhotoRow[ 20 ],
			duration = originalPhotoRow[ 4 ],
			fnumber = originalPhotoRow[ 21 ],
			isMergedHDR = originalPhotoRow[ 22 ],
			hasThumbnail = originalPhotoRow[ 23 ]
		}
		end

	end

	return collectionInfo

end

--------------------------------------------------------------------------------

local function calculateAssetListHash( photoList )
	
	if not photoList then
		return nil
	end
	
	local n = #photoList

	if n == 0 then
		return nil
	end
	
	local digest = WFCrypto.MD5.init()
	
	for i = 1, n do
		digest:update( photoList[ i ].id )
	end
	
	local result = digest:digest()
	return result
end

-- Check to see whether the lists disagree on a field we actually care about

local function photoListsCloseEnough( aList, bList )

	if aList == bList then
		return true
	end

	if aList == nil and bList ~= nil then
		return false
	end
	
	if aList ~= nil and bList == nil then
		return false
	end

	local a_n = #aList
	local b_n = #bList
	
	if a_n ~= b_n then
		return false
	end
	
	for i = 1, a_n do
		local a_item = aList[ i ]
		local b_item = bList[ i ]
		if a_item.id ~= b_item.id then
			return false
		end
		if a_item.flag ~= b_item.flag then
			return false
		end
		
		if a_item.rating ~= b_item.rating then
			return false
		end
		
		if a_item.commentCount ~= b_item.commentCount then
			return false
		end
		
		if a_item.favoriteCount ~= b_item.favoriteCount then
			return false
		end
		
		if a_item.developUserUpdated ~= b_item.developUserUpdated then
			return false
		end
		
		if a_item.croppedWidth ~= b_item.croppedWidth then
			return false
		end
		
		if a_item.croppedHeight ~= b_item.croppedHeight then
			return false
		end
	end
	
	return true

end

--------------------------------------------------------------------------------

local function globalGridCollectionInfoB( catalog, activeAlbum, ascending, assetFilter, assetTypeFilter, keyword, edited, sortCriteria, ratingFilter, ratingComparisonOperator, isSearch )

	local sql, queryParams = LrmAlbumQueryBuilder.buildCoreInfoQuery( activeAlbum, ascending, 
												assetFilter, assetTypeFilter, keyword, edited, sortCriteria, 
												ratingFilter, ratingComparisonOperator,
												isSearch
											)
	
    local resultB = catalog:indexStore():queryB { 
		sql = sql, 
		calm = .125,
		nArgs = #queryParams, 
		unpack( queryParams )
	}

	return resultB:closingMapB( function( originalPhotoRows )
        
		return makeCollectionInfo(
				'grid', -- collection id for grid mode, not used by UI
				originalPhotoRows,
				true )

	end )
							
end

--------------------------------------------------------------------------------

local function bestPhotosCollectionInfoB( catalog, albumId, bestPhotosClusterId )

	local sql, queryParams = LrmAlbumQueryBuilder.buildBestPhotosCoreInfoQuery(albumId, bestPhotosClusterId)

	local resultB = catalog:indexStore():queryB {
		sql = sql,
		calm = .125,
		nArgs = #queryParams,
		unpack( queryParams )
	}

	return resultB:closingMapB( function( originalPhotoRows )

		return makeCollectionInfo(
			'grid', -- collection id for grid mode, not used by UI
			originalPhotoRows,
			true )

	end )

end
--------------------------------------------------------------------------------

local function oldAssetListModelB( collectionInfoB )
	local prevCollectionInfo = {}
	prevCollectionInfo.photos = {}
	return collectionInfoB:closingMapB( function( collectionInfo )
	
		if WFUnknown.isUnknown( collectionInfo ) then
			return collectionInfo
		end
		
		if type( collectionInfo ) == 'userdata' then
			return collectionInfo, false
		end
		
		local newPhotos = collectionInfo and collectionInfo.photos

		if type( newPhotos ) == 'userdata' then
			return newPhotos, false
		end
		
		if collectionInfo ~= nil then
			newPhotos = collectionInfo and collectionInfo.photos
		
			if collectionInfo and collectionInfo.photosUnknown then
				collectionInfo = nil
				newPhotos = nil
			end
			
			local prevPhotos = nil
			if prevCollectionInfo and prevCollectionInfo.photos then
				prevPhotos = prevCollectionInfo.photos
			end
			
			local closeEnough = photoListsCloseEnough( newPhotos, prevPhotos )
			
			if collectionInfo then
				collectionInfo.closeEnough = closeEnough
			end
			
			if closeEnough then
				return prevCollectionInfo
			else
				prevCollectionInfo = collectionInfo
				return prevCollectionInfo
			end
		end
	end )
	:closingMapB( function( collectionInfo )
	
		if WFUnknown.isUnknown( collectionInfo ) then
			return collectionInfo
		end
		
		local albumInfo = nil
		local photos = nil
		if collectionInfo ~= nil then
			albumInfo = {}
			albumInfo.photos = collectionInfo.photos
			albumInfo.assetCount = collectionInfo.assetCount
			albumInfo.closeEnough = collectionInfo.closeEnough
			photos = collectionInfo.photos
		end
		return {
			albumInfo = albumInfo,
			assetListHash = calculateAssetListHash( photos )
		}
	end )
end

--------------------------------------------------------------------------------

local function newAssetListModelB( encoder, catalog, activeAlbum, ascending, assetFilter, assetTypeFilter, keyword, edited, sortCriteria, ratingFilter, ratingComparisonOperator, isSearch )

	local sql, queryParams = LrmAlbumQueryBuilder.buildCoreInfoQuery( activeAlbum, ascending, 
												assetFilter, assetTypeFilter, keyword, edited, sortCriteria, 
												ratingFilter, ratingComparisonOperator,
												isSearch
											)
	
    local result = catalog:indexStore():query { 
		sql = sql, 
		encoder = encoder,
		calm = .125,
		unpack( queryParams )
	}
	
	return result
								
end


--------------------------------------------------------------------------------

function Model:cacheSizeModel()

	local cacheSizeB = self.catalog:indexStore()
		:queryB( {
			sql = [[select sum(cr.size) from cacheReferences as cr inner join
					album_asset on album_asset.assetId = cr.docId where album_asset.albumId == ? and cr.isTrash == 0 ]],
			calm = 3,
			nArgs = 1,
			self.albumId,
		} )
		:closingMapB( function( rows )
		
			if rows == WFUnknown.value() then
				return WFUnknown.value()
			end
		
			local cacheSize = 0
			
			if rows and rows[ 1 ] and rows[ 1 ][ 1 ] then
				cacheSize = rows[1][1]
			end
			
			return cacheSize
		end )
		
	return {
		valueB = cacheSizeB,
	}
end

--------------------------------------------------------------------------------

function Model:isSharedModel()

	return {
		valueB = LrmAlbumUtils.isAlbumSharedB( self.catalog:indexStore(), self.albumId )
	}
	
end

--------------------------------------------------------------------------------

function Model:unsynchronizedAssets()

	local trickerSql = [[SELECT count( distinct r.docId ) FROM renditionRevisions as r INNER JOIN album_asset ON album_asset.assetId = r.docId WHERE album_asset.albumId == ? LIMIT 1]]
	local trickerB = self.catalog:indexStore():queryB( {
		sql = trickerSql,
		self.albumId
	 } )
	:closingMapB( function( rows )
		if rows and #rows > 0 then
			return rows[1][1]
		end
		return nil
	end )	
	return {
		valueB = trickerB,
	}
end

--------------------------------------------------------------------------------

function Model:bestPhotosListModel(bestPhotosClusterId)

	local bestPhotosListB = oldAssetListModelB( bestPhotosCollectionInfoB( self.catalog, self.albumId,  bestPhotosClusterId))
	return {
		valueB = bestPhotosListB,
	}

end
--------------------------------------------------------------------------------

function Model:assetListModel( sortCriteria, ascending, assetFilter, assetTypeFilter, keyword, edited, ratingFilter, ratingComparisonOperator, isSearch, useEncoder )

	useEncoder = useEncoder or false
	
	local activeFilter = {}
	
	for i = 1, #assetFilter do
		local filter = assetFilter[ i ]
		activeFilter[ filter ] = true
	end
	
	for i = 1, #assetTypeFilter do
		local assetType = assetTypeFilter[ i ]
		activeFilter[ assetType ] = true
	end
	
	if edited then
		for i = 1, #edited do
			activeFilter.edited = edited[ i ]
		end
	end
	
	self.catalog:docStore():applyOverlayToDocumentP( {
		docType = 'album',
		docId = self.albumId,
		overlay = {
			_localOnly = {
				filter = {
					_replace = true,
					image = activeFilter.image,
					video = activeFilter.video,
					pick = activeFilter.pick,
					reject = activeFilter.reject,
					unflagged = activeFilter.unflagged,
					favorite = activeFilter.favorite,
					comment = activeFilter.comment,
					keyword = keyword,
					edited = activeFilter.edited,
					rating = ratingFilter,
					ratingComparisonOperator = ratingComparisonOperator,
					sortCriteria = sortCriteria,
					ascending = ascending
				},
			},
		}
	} ):runUntilDone()

	local assetListModelB
	
	if useEncoder then
	
		local encoder = 'LrmOzAssetCoreInfo.encodeSql'
		assetListModelB = newAssetListModelB( encoder, self.catalog, 
						self.albumId, ascending, assetFilter, assetTypeFilter, keyword, activeFilter.edited, sortCriteria, 
						ratingFilter, ratingComparisonOperator, isSearch
					)
	else
	
		assetListModelB = oldAssetListModelB( globalGridCollectionInfoB( self.catalog, 
						self.albumId, ascending, assetFilter, assetTypeFilter, keyword, activeFilter.edited, sortCriteria, 
						ratingFilter, ratingComparisonOperator, isSearch ) 
					)
		
	end
	
	return {
		value = assetListModelB,
	}
	
end

--------------------------------------------------------------------------------

local function makeFlagStatus( self, rows )

	local flags = {
		unflagged = 0,
		pick = 0,
		reject = 0,
	}
	
	for i=1, #rows do
	
		local row = rows[i]
	
		if row[1] == 'unflagged' then	
		
			flags.unflagged = row[2]
		
	  	elseif row[1] == 'pick' then
		
			flags.pick = row[2]
			
	  	elseif row[1] == 'reject' then	
			
			flags.reject = row[2]
			
	  	end
		
	end
	
	return flags
	
end

--------------------------------------------------------------------------------

local function getFlagStatusQuery()

	-- Using sub query here instead of "inner join" because there are cases where album_asset table can contain duplicate entries
	-- for same asset (in delete & restore [CT] scenarios) and adding a distinct keyword will be comparitively non-performant to sub queries
	return [[SELECT flag, count( coreInfo.docId ) FROM coreInfo WHERE docId in (select assetId from album_asset WHERE albumId == ?) GROUP BY coreInfo.flag]]

end
--------------------------------------------------------------------------------

local function flagsB( self, catalog, albumId )

	return catalog:ozUserIdP()
		:untilResolvedB( 'data', WFUnknown.value() )
		:closingKnownMapSwitchB( function (userId)

			return catalog:indexStore():queryB( getFlagStatusQuery(), albumId, userId )
				:closingKnownMapB( function( rows )
						
					return makeFlagStatus( self, rows )
					
				end )

		end )
end

--------------------------------------------------------------------------------

function Model:flagStatusCountModel()

	return {
		valueB = flagsB( self, self.catalog, self.albumId )
	}
end

--------------------------------------------------------------------------------

function Model:alternativeCoverAssetId()

		local coverImageAssetIdB = self.catalog:indexStore():queryB( [[SELECT coreInfo.docId FROM coreInfo
				INNER JOIN album_asset ON album_asset.assetId == coreInfo.docId
				WHERE albumId == ? and coreInfo.docId NOT IN ( SELECT assetId FROM ozUploadStatus WHERE docType == 'asset' )
					ORDER BY captureDate ASC LIMIT 1]], self.albumId )
	
			:closingMapB( function ( rows )
			
				local coverAssetId = rows and rows[ 1 ] and rows[ 1 ][ 1 ]
	
				return coverAssetId
				
			end )
			:closingFilterB( function( val ) return val ~= nil end, WFUnknown.value() )
			
		
	return {
		valueB = coverImageAssetIdB,
	}
	
end

--------------------------------------------------------------------------------

function Model:coverImageModel()

	local explicitCoverFieldB = self.catalog:docStore():fieldValueB( 'album', self.albumId,
							'cover/id' )

	local resultB = explicitCoverFieldB:closingKnownMapSwitchB( function( coverId )
	
		coverId = coverId or ""
		
		local queryB = self.catalog:indexStore():queryB( [[
				SELECT coreInfo.docId
				FROM coreInfo
				INNER JOIN album_asset ON album_asset.assetId == coreInfo.docId
				WHERE
					albumId == ?
				AND
					coreInfo.docId = ?
			UNION ALL
				SELECT ( SELECT coreInfo.docId
				FROM coreInfo
				INNER JOIN album_asset ON album_asset.assetId == coreInfo.docId
				WHERE
					albumId == ?
				ORDER BY
					case when album_asset.sortOrder is null then 1 else 0 end,
					album_asset.sortOrder, captureDate ASC )
			LIMIT 1]]
				, self.albumId, coverId, self.albumId )
		
			-- GRUNGE [dtull 2014-05-27]: This where clause is effectively duplicated
			-- in OzCatCollections.makeAlbumPredicate...
		
		local calmQueryB = queryB
			:closingCalmB( 'later' )
		
		return calmQueryB
		:closingMapB( function ( rows )

			if WFUnknown.isUnknown( rows ) then
				return rows
			end

			local coverAssetId = rows and rows[ 1 ] and rows[ 1 ][ 1 ]
			
			return coverAssetId
			
		end )
	end )

	return {
		value = resultB
	}
	
end

--------------------------------------------------------------------------------

function LrmAlbumModel.albumModel( catalog, albumId )
	
	local resultB = KSReactive.variableB( WFUnknown.value() )
	
	catalog:docStore():docValueOnceP( 'album', albumId )
	:whenResolvedDo( function( status, docValue )
	
		if status == 'data' then
		
			local filter = KSTableUtils.derefOpt( docValue, "_localOnly", "filter" )
			
			if filter then
				
				local deref = KSTableUtils.derefOpt
				
				local activeFilter = {
					image = deref( filter, "image" ),
					video = deref( filter, "video" ),
					pick = deref( filter, "pick" ),
					reject = deref( filter, "reject" ),
					unflagged = deref( filter, "unflagged" ),
					favorite = deref( filter, "favorite" ),
					comment = deref( filter, "comment" ),
					rating = deref( filter, "rating" ),
					ratingComparisonOperator = deref( filter, "ratingComparisonOperator" ),
					sortCriteria = deref( filter, "sortCriteria" ),
					ascending = deref( filter, "ascending" ),
				}
				
				resultB:set( activeFilter )
			else
				resultB:set( nil )
			end
		end
		
	end ):runUntilDone()
	
	return {
		catalog = catalog,
		albumId = albumId,
		valueB = resultB,
		models = Model,
		
		events = {
		}
	}
end




return LrmAlbumModel
