--[[----------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2014 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

local WFUnknown = require 'WFUnknown'

--------------------------------------------------------------------------------

local LrmAlbumUtils = {}

--==============================================================================

function LrmAlbumUtils.isAlbumSharedB( indexStore, albumId )

	local sql =       {
		                sql =
		                [[SELECT
						      sa.docId, sa.spaceId FROM space_album sa
					          INNER JOIN space s ON sa.spaceId == s.docId
					          WHERE
						      sa.albumId == ? AND
						      ((s.private == 0)
						      OR
						      (s.private == 1 AND s.allowInviteRequests == 1)
						      OR
                              EXISTS
								  (SELECT spaceId from space_invite where space_invite.spaceId = sa.spaceId
									   AND
									   ((space_invite.subtype = 'request' and space_invite.status == 'open')
									   OR
									   (space_invite.subtype = 'single_use' and space_invite.status == 'open'))
								   UNION
								   SELECT spaceId from space_user where space_user.spaceId = sa.spaceId AND space_user.status = 'accepted'))
                              ]],
		                   calm = 5,
							albumId,
	                 }

					
	local isSharedB = indexStore:queryB( sql )
		:closingMapB( function( idRow )
		
			if WFUnknown.isUnknown( idRow ) then
				return idRow
			end
			
			if idRow and idRow[ 1 ] and idRow[ 1 ][ 1 ] and idRow[ 1 ][ 2 ] then
				return true
			else
				return false
			end
			
		end )
		
		
	return isSharedB
	
end

--------------------------------------------------------------------------------

function LrmAlbumUtils.hasAnySharedAlbumsB( indexStore )

	return indexStore:queryB( [[SELECT
									space_album.docId
								FROM
									space_album
								INNER JOIN
									space
								ON
									space_album.spaceId == space.docId
								WHERE
									space.private == 0
								LIMIT 1]] )
		
		:closingMapB( function( rows )
		
			if WFUnknown.isUnknown( rows ) then
				return WFUnknown.value()
			end
			
			if rows and rows[ 1 ] and rows[ 1 ][ 1 ] then
				return true
			end
			
			return false
			
		end )
		
end

--------------------------------------------------------------------------------

function LrmAlbumUtils.getDocTypeForAlbumId (albumId)

	if #albumId ~= 32 or albumId == "10002000300040005000600070008000" then
	
		return 'gen-album'
	
	end
	
	return 'album'
	
end

--------------------------------------------------------------------------------

return LrmAlbumUtils
