--[[----------------------------------------------------------------------------

LrmCatXmpDevelop.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2016 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

-- Note: LrmCatXmpDevelop is not used on iOS.
	
	-- WF.core.reactive
local Rx = require 'Rx'

	-- WF.core.rxlua
local WFRxPromiseBridge = require 'WFRxPromiseBridge'

local WFArguments = require "WFArguments"
local KSTableUtils = require "KSTableUtils"
local KSData = require "KSData"
local KSPathUtils = require "KSPathUtils"
local OzFilePaths = require "OzFilePaths"
local WFFileUtils = require "WFFileUtils"

local check = WFArguments.check

local asKSPromise = WFRxPromiseBridge

local derefOpt = KSTableUtils.derefOpt

--------------------------------------------------------------------------------

local LrmCatXmpDevelop = {}

--------------------------------------------------------------------------------

local function getExternalXmpDevelop( catalog, docValue, sha256 )

	-- resolve external xmpCameraRaw
	
	local hrefXmpDevelop = derefOpt(docValue, '_serverProvidedMetadata', 'links', '/rels/xmp/develop', 'href' )
	local xmpFiles = derefOpt( docValue, '_localOnly', 'files', 'xmp_develop' )
	local documents = KSPathUtils.getStandardFilePath( 'carouselDocuments' )

	sha256 = sha256 or derefOpt( docValue, 'develop', 'xmpCameraRaw', 'sha256' )
	
	if xmpFiles and xmpFiles.path then
					
		local xmpAbsPath = KSPathUtils.child( documents, xmpFiles.path )
	
		if xmpFiles.sha256 == sha256 and WFFileUtils.exists( xmpAbsPath ) then

			-- xmpCameraRaw is already present					
			return Rx.Observable.just( KSData.memoryMappedContentsOfFile( xmpAbsPath ), xmpFiles.path )

		end

		-- remove the stale file
		WFFileUtils.delete( xmpAbsPath )

	end
		
	if not sha256 or not hrefXmpDevelop then
		
		return Rx.Observable.throw( 'No xmpCameraRaw available, not even external' )

	end

	return catalog:getCatUser()
		:flatMap( function( ozUser )
	
			return Rx.Observable.combineLatest( function( xmpCameraRaw, wfCatalog )

				local catalogId = wfCatalog:ozCatalogSpec():catalogId()

				local xmpRelPath = OzFilePaths.getRelativeXmpDevelopPath( catalogId, sha256, true )

				KSData.saveToFile( xmpCameraRaw, KSPathUtils.child( documents, xmpRelPath ) )
				
				return xmpCameraRaw, xmpRelPath
			
			end,
			ozUser:getBinary( hrefXmpDevelop ),
			catalog._wfCatalogStream ) -- TODO [immisch 2016-09-28]: this is totally hacky, but how to do this?
		end)

		:flatMap( function( xmpCameraRaw, xmpRelPath )

			-- MEH [immisch 2016-09-28]: WFRxDocStore has _id, but the compatibility layer gives us _docId
			local docId = docValue._id or docValue._docId

			-- update the document
			return catalog:docStore():applyOverlay {
					_id = docId,
					_localOnly = {
						files = {
							xmp_develop = {
								sha256 = sha256,
								path = xmpRelPath,
								uploadStatus = { _delete = true }
							}
						}
					}
				}
				:map( function ( )
				
					return xmpCameraRaw, xmpRelPath
				
				end )
				
		end )

end


--[[!
	\brief retrieve and store the external xmpCameraRaw develop settings
	
	\param catalog the catalog service
	\param docIdOrValue the document id or the docValue
	\param sha256 the sha256 of the external develop settings (optional)
	\return (KSData, string) xmpCameraRaw and the relative path to the file
--]]
function LrmCatXmpDevelop.getExternalXmpDevelop( catalog, docIdOrValue, sha256 )

	check( 'OzCatalogService', 'table|string', '?string')

	if type(docIdOrValue) == 'table' then
		return getExternalXmpDevelop( catalog, docIdOrValue, sha256 )
	end
	
	return catalog:docStore():docValue( docIdOrValue )
		
		:flatMap( function( docValue )
			
			return getExternalXmpDevelop( catalog, docValue, sha256 )
		
		end )

end

--------------------------------------------------------------------------------

function LrmCatXmpDevelop.getExternalXmpDevelopP( catalog, docIdOrValue, sha256 )

	return LrmCatXmpDevelop.getExternalXmpDevelop( catalog, docIdOrValue, sha256 )
		:call( asKSPromise )

end


return LrmCatXmpDevelop