--[[----------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2015 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

	-- WF.core.rxlua
local _ = require 'WFRxPromiseBridge'

	-- WF.core.reactive
local KSReactive = require 'KSReactive'
local KSPromises = require 'KSPromises'

	-- WF.core.utils
local WFCloseSupport = require 'WFCloseSupport'
local KSLuaUtils = require 'KSLuaUtils'
local KSTableUtils = require 'KSTableUtils'
local WFArguments = require 'WFArguments'
local WFUnknown = require "WFUnknown"

	-- WF.core.os
local WFFileUtils = require 'WFFileUtils'
local WFPlatformInfo = require 'WFPlatformInfo'

	-- WF.core.lua
local KSData = require 'KSData'

	-- WF.lrmobile_models
local LrmOzDevelopSession = require 'LrmOzDevelopSession'
local LrmAssetUtils = require 'LrmAssetUtils'
local LrmWFOzProfileUtils = require 'LrmWFOzProfileUtils'
local LrmConfig = require 'LrmConfig'
local LrmCatXmpDevelop = require "LrmCatXmpDevelop"

	-- WF.oz_client.utils
local OzCatalogUploadStatus = require 'OzCatalogUploadStatus'

	-- WF.imaging.camera_roll
local WFCameraRollService = require 'WFCameraRollService'

	-- WF.core.rxlua
local WFRxPromiseBridge = require 'WFRxPromiseBridge'

	-- WF.web.http_client
local WFRxHttpClient = require 'WFRxHttpClient'

local WFAnalytics = require "WFAnalytics"

local OzFilePaths = require "OzFilePaths"

--------------------------------------------------------------------------------

local derefOpt = assert( KSTableUtils.derefOpt )

local asKSPromise = WFRxPromiseBridge

--==============================================================================

local LrmDevelopModel = {}

--------------------------------------------------------------------------------

local Model = {}

--------------------------------------------------------------------------------
WFArguments.unregisterChecker( 'binaryPreference' )

WFArguments.registerChecker( 'binaryPreference', function( strategy )
	return type( strategy ) == 'string' and
		strategy == LrmConfig.StrategyPreferMaster or
		strategy == LrmConfig.StrategyPreferProxy or
		strategy == LrmConfig.StrategyProxy or
		strategy == LrmConfig.StrategyMaster or
		strategy == LrmConfig.StrategyProxyAndMaster
end )

--------------------------------------------------------------------------------

function Model:developChanges()
	
	local devChangesB = self.catalog:photoInfoB( self.assetId )
	
		:closingMapSwitchB( function( docValue )
		
			if WFUnknown.isUnknown( docValue ) then
				return KSReactive.constantB( docValue )
			end
		
			local develop = docValue.develop or {}
			
			local hasExternalXmpDevelop = LrmAssetUtils.hasExternalXmpDevelop( docValue )
			local hasExternalXmpDevelopLocally = LrmAssetUtils.hasExternalXmpDevelopLocally( docValue )

			-- Android: xmpCameraRawFiles might be in the docValue but is missing (user accessible storage)
			local externalXmpDevelopMissing
			if WFPlatformInfo.isAndroid then
				externalXmpDevelopMissing = hasExternalXmpDevelop and not ( hasExternalXmpDevelopLocally
					and WFFileUtils.exists( OzFilePaths.getAbsoluteXmpDevelopPath( docValue ) ) )
			else
				externalXmpDevelopMissing = hasExternalXmpDevelop and not hasExternalXmpDevelopLocally
			end

			if externalXmpDevelopMissing then

				local getExternalXmpP = LrmCatXmpDevelop.getExternalXmpDevelopP( self.catalog, docValue )
					:mapDataP_new( function( _, xmpRelPath )
						
						develop.localXmpDevelopPath = xmpRelPath
						
						return develop
					
					end )
					
					return getExternalXmpP:untilResolvedB( 'data', WFUnknown.value() )
			
			end
			
			develop.localXmpDevelopPath = derefOpt( docValue, '_localOnly', 'files', 'xmp_develop', 'path' )
			
			return KSReactive.constantB( develop )
			
		end )
	
	return {
		valueB = devChangesB
	}
	
end

--------------------------------------------------------------------------------

function Model:reset()

	return {
		valueB = self.resetB,
	}
	
end

--------------------------------------------------------------------------------

function Model:master()

	return {
		valueB = self.masterB,
	}

end

--------------------------------------------------------------------------------

function Model:binaryFromCameraRoll( localUrl )

	local obs = WFCameraRollService.getAssetData( localUrl )
		
		:map( function( assetInfo )
		
			if KSData.isData( assetInfo.data ) then
			
				return assetInfo.data
				
			end
			
			return nil
			
		end )
		
	return {
		value = obs,
	}

end

--------------------------------------------------------------------------------

function LrmDevelopModel.requestBinaryP( catalog, photoId, binaryStrategy, developInfo, priority)

	WFArguments.check( 'OzCatalogService', 'string', 'binaryStrategy', 'table' )
	
	priority = priority or WFRxHttpClient.requestPriorityHigh

	if developInfo.useBinaryFromCameraRoll then
		return nil
	end
	
	if binaryStrategy == LrmConfig.StrategyProxyAndMaster then
		return KSPromises.resolvedPromiseP( 'error', "Wrong usage of requestBinaryP" )
	end
	
	if ( not developInfo.hasProxy and not developInfo.hasMaster ) then
		-- always a serious error
		return KSPromises.resolvedPromiseP( 'error', "No proxy and master available on Oz" )
	end

	if ( not developInfo.hasProxy and binaryStrategy == LrmConfig.StrategyProxy ) then
		-- might be an ignorable error - if both proxy and master got requested - or not
		return KSPromises.resolvedPromiseP( 'error', "No proxy available on Oz" )
	end

	if ( not developInfo.hasMaster and binaryStrategy == LrmConfig.StrategyMaster ) then
		-- might be an ignorable error - if both proxy and master got requested - or not
		return KSPromises.resolvedPromiseP( 'error', "No master available on Oz" )
	end

	if ( binaryStrategy == LrmConfig.StrategyPreferProxy and developInfo.hasProxy ) or
		( binaryStrategy == LrmConfig.StrategyPreferMaster and not developInfo.hasMaster ) or
		( binaryStrategy == LrmConfig.StrategyProxy ) then

        if not developInfo.proxyPath then
            return catalog:proxyDataPathP( photoId, false, 2560, priority )
					:mapP( function( status, data )
                            return status, data, false
                    end )
            else
                return KSPromises.resolvedPromiseP( 'data', developInfo.proxyPath, false )
            end

    else

        if not developInfo.originalPath then
            return catalog:originalDataPathP( photoId, false, priority )
                        :mapP( function( status, data )
                                return status, data, true
                        end )
        else
            return KSPromises.resolvedPromiseP( 'data', developInfo.originalPath, true )
        end

	end

end

--------------------------------------------------------------------------------

function LrmDevelopModel.makeModel( catalog, photoId, binaryStrategy, priority )

	assert( catalog )
	assert( photoId )
	
	binaryStrategy = binaryStrategy or LrmConfig.StrategyPreferProxy
	priority = priority or WFRxHttpClient.requestPriorityHigh
	
	local developSession = LrmOzDevelopSession( catalog, photoId )
	local developInfoB = KSReactive.variableB( nil )
	local masterB = KSReactive.variableB( nil )
	local resetB = KSReactive.variableB( nil )
	
	local function cancelSession()

		if developSession then
			developSession:close()
			developSession = nil
		end
	end
	
	local function applyDevelopChanges( self, changes )
	
		KSLuaUtils.dpcall( 'applyDevelopChanges', function()
			if changes then
			
				developSession:saveAndCloseP( changes )
					:whenResolvedDo( function( status, message )
					
						debug.print( "developSession:saveAndCloseP resolved with status ",
										status, " - message ", message )
						
						if status == 'data' then
						
							-- This is probably the most appropriate point to tear down
							-- the develop UI. At this point, the local Oz database has
							-- confirmed that it has stored the new develop settings.
						
						elseif status == 'error' then
						
							-- FIXME [aksherry < scouten 2011-07-23]: We failed to record
							-- develop settings (unlikely, but we should cover the case).
							-- We should at least warn user. Not sure if there's a viable
							-- recovery path at this point.
						
						end
						
						if not WFCloseSupport.hasBeenOrWillBeClosed( self.resetB ) then
							self.resetB:set( true )
						end
						
					end ):runUntilDone()

			else
			
				debug.print( "WARNING: Got called with settings = nil. Treating this as a cancel." )
				
				cancelSession()
			
			end
		
		end )
		
	end

	local developInfo = {
		proxyPath = nil,
		originalPath = nil,
		settings = nil,
		hasProxy = false,
		hasMaster = false,
		hasOriginalInCameraRoll = false
	}
	
	local updateAcrProfileCache = false
	
	local developInfoP = developSession
	
		:docValueP()
		
		:mapP( function( status, docValue )
		
			local settings = docValue.develop or {}
			
			if status ~= 'data' or not settings then
				-- FIXME [aksherry < scouten 2011-07-23]: We failed to get develop settings
				-- or to acquire the lock on this photo. Signal an error to user and bounce
				-- out of develop.
				return KSPromises.resolvedPromiseP( 'error', 'No develop settings locally available' )
			end
			
			developInfo.settings = settings
			
			developInfo.hasProxy = LrmAssetUtils.hasProxy( docValue )
			developInfo.hasMaster = LrmAssetUtils.hasMaster( docValue )
			developInfo.docValue = docValue
			developInfo.width = derefOpt( docValue, 'importSource', 'originalWidth' )
			developInfo.height = derefOpt( docValue, 'importSource', 'originalHeight' )
			
			if type( settings.xmpCameraRaw )  == 'string' then

				-- xmpCameraRaw is inline
				return KSPromises.resolvedPromiseP( 'data', settings.xmpCameraRaw )

			end

			local sha256 = derefOpt( docValue, 'develop', 'xmpCameraRaw', 'sha256' )
			
			-- assets imported from device will not have xmpCameraRaw attached
			if not sha256 then
				return KSPromises.resolvedPromiseP( 'data', "" )
			end

			return LrmCatXmpDevelop.getExternalXmpDevelopP( catalog, docValue, sha256 )
				:mapDataP_new( function( xmpCameraRaw, xmpRelPath )
				
					-- store the relative filename in our in-memory docValue for later
					-- in the reactive chain
					
					if not docValue._localOnly then
						docValue._localOnly = {}
					end
					
					if not docValue._localOnly.files then
						docValue._localOnly.files = {}
					end
					
					docValue._localOnly.files.xmp_develop = { 
						path =  xmpRelPath,
						sha256 = sha256
					}

					return xmpCameraRaw
				
				end )

		end )
		
		:mapDataP_new( function( xmpCameraRaw )

			return LrmWFOzProfileUtils.ensureProfilesExistLocallyP( catalog, photoId )
		
		end)
		
		:mapDataP_new( function( info )
		
			if type( info ) == 'table' then
				
				if info.updateAcrProfileCache then
					updateAcrProfileCache = true
				end
				
			end

			return catalog:proxyDataPathP( photoId, true, 2560, priority )
			
		end )
		
		:mapDataP_new( function( localProxyDataPath )
		
			if localProxyDataPath then
				developInfo.proxyPath = localProxyDataPath
			end
			
			return catalog:originalDataPathP( photoId, true, priority )
			
		end )
		
		:mapDataP_new( function( localOriginalDataPath )
		
			if localOriginalDataPath then
				developInfo.originalPath = localOriginalDataPath
			end
			
			return developInfo
			
		end )
		
		:mapDataP_new( function( developInfo )
		
			local localUrl = derefOpt( developInfo, 'docValue', '_localOnly', 'localUrl' )
			
			if localUrl and WFPlatformInfo.isiOS then
			
				return WFCameraRollService.getAssetInfo( localUrl )
					
					:call( asKSPromise )
					:mapP( function( status, assetInfo )
					
						developInfo.hasOriginalInCameraRoll = status == 'data'
							
						return KSPromises.resolvedPromiseP( 'data', developInfo )
						
					end )
				
			end
			
			return developInfo
			
		end )
		
		:mapDataP_new( function( developInfo )
		
			if WFPlatformInfo.isiOS then
			
				local localUrl = derefOpt( developInfo, 'docValue', '_localOnly', 'localUrl' )
				local hasProxyLocally = LrmAssetUtils.hasProxyLocally( developInfo.docValue )
				local hasMasterLocally = LrmAssetUtils.hasMasterLocally( developInfo.docValue )
				
				local hasLocalBinary = hasProxyLocally or hasMasterLocally
				
				local useBinaryFromCameraRoll = localUrl and not hasLocalBinary and developInfo.hasOriginalInCameraRoll

				if useBinaryFromCameraRoll then

					local docId = derefOpt( developInfo, 'docValue', '_id' ) or derefOpt( developInfo, 'docValue', '_docId' )

					WFAnalytics.trackEvent {
						['event.type'] = 'useBinaryFromCameraRoll',
						['event.workflow'] = 'DEVELOP',
						['content.id'] = docId,
						['content.url'] = localUrl,
					}
				end

				developInfo.useBinaryFromCameraRoll = useBinaryFromCameraRoll

			end

			return developInfo
			
		end )

	developInfoP
		
		:mapDataP_new( function( developInfo )
			 
			local strategy = binaryStrategy
			if binaryStrategy == LrmConfig.StrategyProxyAndMaster then
				strategy = LrmConfig.StrategyProxy
			end

			return LrmDevelopModel.requestBinaryP( catalog, photoId, strategy, developInfo, priority)
			
		end )
		
		:whenResolvedDo( function( status, binaryPath, isOriginal )
			
			if status == 'data' then
			
				local localOnly = derefOpt( developInfo, 'docValue', '_localOnly' )
				
				local localProxyPath = OzCatalogUploadStatus.checkAndCorrectLocalPath( derefOpt( localOnly, 'proxy', 'filePath' ) )
				local localOriginalPath = OzCatalogUploadStatus.checkAndCorrectLocalPath( derefOpt( localOnly, 'originalFilePath' ) )
				
				if isOriginal then
					localOriginalPath = binaryPath
				else
					localProxyPath = binaryPath
				end
				
				-- this path is relative, so we don't need to correct it
				local localXmpDevelopPath = derefOpt( localOnly, 'files', 'xmp_develop', 'path' )
				
				if WFPlatformInfo.isAndroid then
					localProxyPath = binaryPath
				end
				developInfoB:set( {
					state = true,
					settings = developInfo.settings,
					proxyPath = localProxyPath,
					updateAcrProfileCache = updateAcrProfileCache,
					useBinaryFromCameraRoll = developInfo.useBinaryFromCameraRoll,
					localUrl = derefOpt( developInfo, 'docValue', '_localOnly', 'localUrl' ),
					localProxyPath = localProxyPath,
					localOriginalPath = localOriginalPath,
					localXmpDevelopPath = localXmpDevelopPath
				} )
				
			else
			
				developInfoB:set( {
					state = false,
					error = binaryPath
				} )
				
			end
			
		end )
		
		:attachTo( developInfoB )

	if binaryStrategy == LrmConfig.StrategyProxyAndMaster then
		-- request a second binary, this will always be the master. Fire subModel 'master' when ready

		developInfoP

			:mapDataP_new( function( developInfo )

				return LrmDevelopModel.requestBinaryP( catalog, photoId, LrmConfig.StrategyMaster, developInfo, priority)

			end )

			:whenResolvedDo( function( status, binaryPath, isOriginal )

				if status == 'data' then

					local localOnly = derefOpt( developInfo, 'docValue', '_localOnly' )

					local localProxyPath = OzCatalogUploadStatus.checkAndCorrectLocalPath( derefOpt( localOnly, 'proxy', 'filePath' ) )
					local localOriginalPath = OzCatalogUploadStatus.checkAndCorrectLocalPath( derefOpt( localOnly, 'originalFilePath' ) )

					if isOriginal then
						localOriginalPath = binaryPath
					else
						localProxyPath = binaryPath
					end

					-- this path is relative, so we don't need to correct it
					local localXmpDevelopPath = derefOpt( localOnly, 'files', 'xmp_develop', 'path' )

					masterB:set( {
						state = true,
						settings = developInfo.settings,
						proxyPath = localProxyPath,
						updateAcrProfileCache = updateAcrProfileCache,
						useBinaryFromCameraRoll = developInfo.useBinaryFromCameraRoll,
						localUrl = derefOpt( developInfo, 'docValue', '_localOnly', 'localUrl' ),
						localProxyPath = localProxyPath,
						localOriginalPath = localOriginalPath,
						localXmpDevelopPath = localXmpDevelopPath
					} )

				else

					masterB:set( {
						state = false,
						error = binaryPath
					} )

				end

			end )

			:attachTo( masterB )

		end

	return {
		assetId = photoId,
		catalog = catalog,
		resetB = resetB,
		masterB = masterB,
		valueB = developInfoB,
		models = Model,
		
		events = {
			cancel = cancelSession,
			applyChanges = function( self, dict ) --, thumbnail, preview )
			
				local changes = {}
				changes.xmpCameraRaw = dict.xmpCameraRaw
				changes.userOrientation = dict.userOrientation
				changes.thumbnail = dict.thumbnail
				changes.preview = dict.preview
				changes.croppedWidth = dict.croppedWidth
				changes.croppedHeight = dict.croppedHeight
				changes.localXmpDevelopPath = dict.localXmpDevelopPath
				changes.crsVersion = dict.crsVersion
				changes.cameraProfileFileName = dict.cameraProfileFileName
				changes.cameraProfileLink = dict.cameraProfileLink
				changes.appliedCameraProfileDigest = dict.appliedCameraProfileDigest or nil
				changes.cameraModelName = dict.cameraModelName or nil

				applyDevelopChanges( self, changes )
				
			end,
			
			lockSettings = function( self )
				developSession:loadSettings()
			end,
			
			resetSession = function( self )
				self.resetB:set( false )
			end,
		},
		
		close = function( self )
			cancelSession()
		end,
	}

end

return LrmDevelopModel
