--[[----------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2015 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

	-- WF.core.reactive
local KSReactive = require 'KSReactive'

	-- WF.core.utils
local WFUnknown = require 'WFUnknown'

--==============================================================================

local LrmLibraryAlbumListModelImp = {}

--==============================================================================

local function albumsDateSortedB( indexStore, ascending )

	local sorting = "DESC"
	if ascending then
		sorting = "ASC"
	end
	
	return indexStore:queryB(
		[[SELECT DISTINCT a.docId as id, a.name as name, a.useOffline as useOffline, MAX(
			CASE WHEN a.updateTimeStamp IS NULL THEN 0 ELSE a.updateTimeStamp END,
			CASE WHEN a.localTimeStamp IS NULL THEN 0 ELSE a.localTimeStamp END,
			CASE WHEN b.maxUpdateTimeStamp IS NULL THEN 0 ELSE b.maxUpdateTimeStamp END,
			CASE WHEN b.maxUpdateTimeStamp IS NULL THEN 0 ELSE b.maxUpdateTimeStamp END) as timeStamp, a.coverId as coverAssetId FROM
				(SELECT docId, name, updateTimeStamp, localTimeStamp, useOffline, coverId FROM albums WHERE subtype == 'collection') a
				LEFT JOIN
				(SELECT albumId, MAX(updateTimeStamp) as maxUpdateTimeStamp FROM album_asset GROUP BY albumId) b
					ON a.docId == b.albumId ORDER BY timeStamp ]] .. sorting )
end

--------------------------------------------------------------------------------

local function albumsNameSortedB( indexStore, ascending )
	
	local sorting = "DESC"
	if ascending then
		sorting = "ASC"
	end
	
	return indexStore:queryB( [[SELECT docId as id, name, useOffline, updateTimeStamp, coverId as coverAssetId FROM albums WHERE subtype == 'collection' ORDER BY nameLC COLLATE i18n_collate ]] .. sorting )
end

--------------------------------------------------------------------------------

local function albumsCountSortedB( indexStore, ascending )
	
	local sorting = "DESC"
	if ascending then
		sorting = "ASC"
	end
	
	return indexStore:queryB([[SELECT a.docId as id, a.name, a.useOffline, a.updateTimeStamp, a.coverId as coverAssetId FROM albums as a
	                        left outer join (select albumId, count(albumId) as albumAssetCount from album_asset group by albumId) as b
							on a.docId == b.albumId WHERE a.subtype == 'collection' order by ifnull(albumAssetCount, 0) ]] .. sorting )
end

--------------------------------------------------------------------------------

local function albumsStatusSortedB( indexStore, ascending )

-- Watson #3681078 downloaded albums come first, so reverse the ordering

	local sorting = "DESC"
	if not ascending then
		sorting = "ASC"
	end
	
	return indexStore:queryB( [[SELECT docId as id, name, useOffline, updateTimeStamp, coverId as coverAssetId FROM albums WHERE subtype == 'collection' ORDER BY useOffline ]] .. sorting )
end

--------------------------------------------------------------------------------

local function albumsSizeSortedB( indexStore, ascending )

	local sorting = "DESC"
	if ascending then
		sorting = "ASC"
	end
	
	return indexStore:queryB( [[SELECT docId as id, name, useOffline, updateTimeStamp, coverId as coverAssetId FROM albums WHERE subtype == 'collection' left outer join ( select albumId, sum(cr.size) as size from
							cacheReferences as cr inner join album_asset on album_asset.assetId = cr.docId where cr.isTrash == 0
							group by album_asset.albumId ) sq on sq.albumId == id order by ifnull(size,0) ]] .. sorting )
end

--------------------------------------------------------------------------------

local function makeAlbumListB( collectionB )

	return collectionB:closingMapB( function( data )
		
		if data == nil or WFUnknown.isUnknown( data ) then
			return WFUnknown.value()
		end
	
		local albums = {}
		for i = 1, #data do
			albums[ i ] = {}
			albums[ i ].id = data[ i ][ 1 ]
			albums[ i ].name = data[ i ][ 2 ]
			albums[ i ].useOffline = data[ i ][ 3 ] == 1
			albums[ i ].updateTimeStamp = data[ i ][ 4 ]
			albums[ i ].coverAssetId = data[ i ][ 5 ]
		end
		return albums
	end )
end

--------------------------------------------------------------------------------

function LrmLibraryAlbumListModelImp:albumsModel( sortCriteria, ascending )
	
	local albumsB = nil
	
	if sortCriteria == "modifiedDate" then
		albumsB = makeAlbumListB( albumsDateSortedB( self.catalog:indexStore(), ascending ) )
	elseif sortCriteria == "name" then
		albumsB = makeAlbumListB( albumsNameSortedB( self.catalog:indexStore(), ascending ) )
	elseif sortCriteria == "count" then
		albumsB = makeAlbumListB( albumsCountSortedB( self.catalog:indexStore(), ascending ) )
	elseif sortCriteria == "status" then
		albumsB = makeAlbumListB( albumsStatusSortedB( self.catalog:indexStore(), ascending ) )
	elseif sortCriteria == "size" then
		albumsB = makeAlbumListB( albumsSizeSortedB( self.catalog:indexStore(), ascending ) )
	end
	
	return {
		valueB = KSReactive.tableB {
			albums = albumsB,
		}
	}

end

--------------------------------------------------------------------------------

return LrmLibraryAlbumListModelImp
