--[[----------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2015 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

	-- Rx
local Rx = require "Rx"
local Observable = Rx.Observable

	-- WF.core.reactive
local KSReactive = require 'KSReactive'
local KSPromises = require 'KSPromises'

	-- WF.core.utils
local WFUnknown = require 'WFUnknown'
local KSStringUtils = require 'KSStringUtils'

	-- WF.lrmobile
local LrmAlbumUtils = require 'LrmAlbumUtils'
local LrmCatAlbum = require 'LrmCatAlbum'

--------------------------------------------------------------------------------

local KSStringUtils_trim = assert( KSStringUtils.trim )

local LrmLibraryAlbumModelImp = {}

--==============================================================================

local function createOrRenameModel( albumTitle, promiseFn )

	local resultB = KSReactive.variableB( WFUnknown.value() )
	
	local success, _ = pcall( function()
		albumTitle = KSStringUtils_trim( albumTitle )
	end )
	
	if not success or not ( #albumTitle > 0 ) then
		resultB:set( { state = false, error = "InvalidName" } )
	else
	
		promiseFn( albumTitle ):whenResolvedDo( function( status, data )
		
			if status == 'data' then
				resultB:set( { state = true, albumId = data } )
			else
				resultB:set( { state = false, error = "NameExists" } )
			end
		
		end ):runUntilDone()
	end

	return {
		valueB = resultB,
	}
	
end

--------------------------------------------------------------------------------

function LrmLibraryAlbumModelImp:createAlbum( albumTitle, parentId, isFolder, hideInorganize )

	local function promiseFn( trimmedAlbumTitle )
		return LrmCatAlbum.createAlbumP( self.catalog, trimmedAlbumTitle, parentId, isFolder, hideInorganize )
	end

	return createOrRenameModel( albumTitle, promiseFn )

end

function LrmLibraryAlbumModelImp:convertAlbumToAdhoc(albumId)
	return LrmCatAlbum.convertAlbumToAdhoc(self.catalog, albumId)
end

function LrmLibraryAlbumModelImp:convertAdhocToAlbum(albumId)
	return LrmCatAlbum.convertAdhocToAlbum(self.catalog, albumId)
end

function LrmLibraryAlbumModelImp:shareGroupAlbum( albumId , attributes)

	local result = self._wfCatalogStream
	:flatMap( function( wfCatalog )
		return LrmCatAlbum.shareGroupAlbum( wfCatalog, albumId , attributes)
	end )

	return {
		value = result
	}


end

--------------------------------------------------------------------------------

function LrmLibraryAlbumModelImp:reparentAlbum( albumId, newParentId )

	local resultB = KSReactive.variableB( WFUnknown.value() )
	local catalog = self.catalog

	LrmCatAlbum.isValidAlbumP( self.catalog, albumId )
		:mapP( function( status, data )

			if status == 'data' and data == true then
				return LrmCatAlbum.reparentAlbumP( catalog, albumId, newParentId )
			else
				return KSPromises.resolvedPromiseP( 'error', "Album doesn't exist" )
			end

		end )
		:whenResolvedDo( function( status, data )
			if status == 'data' then
				resultB:set( { state = true, albumId = albumId } )
			else
				resultB:set( { state = false, error = data, albumId = albumId } )
			end
		end )
		:runUntilDone()

	return {
		valueB = resultB,
	}

end

--------------------------------------------------------------------------------

function LrmLibraryAlbumModelImp:renameAlbum( albumId, albumTitle, oldName )

	local resultB = KSReactive.variableB( WFUnknown.value() )
	
	LrmCatAlbum.isValidAlbumP( self.catalog, albumId )
	:mapP( function( status, data )
		if status == 'data' and data == true then
			return LrmCatAlbum.renameAlbumP( self.catalog, albumId, albumTitle, false )
		else
			return KSPromises.resolvedPromiseP( 'error', "Album doesn't exist" )
		end
	end )
	:whenResolvedDo( function( status, data )
		if status == 'data' then
			resultB:set( { state = true, albumId = albumId } )
		else
			resultB:set( { state = false, albumId = albumId, error = data, oldName = oldName, newName = albumTitle } )
		end
	end ):runUntilDone()

	return {
		valueB = resultB,
	}

end

--------------------------------------------------------------------------------

function LrmLibraryAlbumModelImp:deleteAlbum( albumId )

	local resultB = KSReactive.variableB( WFUnknown.value() )
	local catalog = self.catalog
	
	LrmCatAlbum.isValidAlbumP( self.catalog, albumId )
		:mapP( function( status, data )
		
			if status == 'data' and data == true then
				return LrmCatAlbum.deleteAlbumP( catalog, albumId )
			else
				return KSPromises.resolvedPromiseP( 'error', "Album doesn't exist" )
			end
			
		end )
		:whenResolvedDo( function( status, data )
			if status == 'data' then
				resultB:set( { state = true, albumId = albumId } )
			else
				resultB:set( { state = false, error = data, albumId = albumId } )
			end
		end )
		:runUntilDone()

	return {
		valueB = resultB,
	}

end

--------------------------------------------------------------------------------

function LrmLibraryAlbumModelImp:setAlbumForOfflineUse( albumId, useOffline )

	local resultB = KSReactive.variableB( WFUnknown.value() )
	local resultP = self.catalog:docStore():applyOverlayToDocumentP {
					docType	= LrmAlbumUtils.getDocTypeForAlbumId (albumId),
					docId 	= albumId,
					overlay	= {
						_localOnly = {
							useOffline = useOffline,
						},
					},
				}

	resultP:whenResolvedDo( function( status, data )
		if status == 'data' then
			resultB:set( { state = true, albumId = albumId } )
		else
			resultB:set( { state = false, albumId = albumId, error = data } )
		end
	end ):runUntilDone()

	return {
		valueB = resultB,
	}
end

--------------------------------------------------------------------------------

function LrmLibraryAlbumModelImp:setAlbumCover( albumId, assetId )
	local resultB 		= KSReactive.variableB( WFUnknown.value() )

	local resultP 		= self.catalog:docStore():applyOverlayToDocumentP {
							docType 	= LrmAlbumUtils.getDocTypeForAlbumId (albumId),
							docId 		= albumId,
							overlay 	= {
								cover = {
									id = assetId
								}
							},
						}

	resultP:whenResolvedDo( function( status, data )
		if status == 'data' then
			resultB:set( { state = true, albumId = albumId } )
		else
			resultB:set( { state = false, albumId = albumId, error = data } )
		end
	end ):runUntilDone()

	return {
		valueB = resultB
	}
end


------------------------------------------------------------------------------

function LrmLibraryAlbumModelImp:getSharedAlbumAttributes( albumId )

	local result = self._wfCatalogStream
		:flatMap( function( wfCatalog )
				return LrmCatAlbum.getSharedAlbumAttributes( wfCatalog, albumId)
		end )

	return {
        value = result
    }


end


------------------------------------------------------------------------------

function LrmLibraryAlbumModelImp:setSharedAlbumAttributes( albumId , attributes)

	local result = self._wfCatalogStream
		:flatMap( function( wfCatalog )
				return LrmCatAlbum.setSharedAlbumAttributes( wfCatalog, albumId , attributes )
		end )

	return {
		value = result
	}


end

--------------------------------------------------------------------------------


function LrmLibraryAlbumModelImp:stopSharing( albumId )

	local result = self._wfCatalogStream
		:flatMap( function( wfCatalog )
				return LrmCatAlbum.stopSharing( wfCatalog, albumId )
		end )

	return {
		value = result
	}


end


--------------------------------------------------------------------------------


function LrmLibraryAlbumModelImp:getAccessRequests( albumId )

	local result = self._wfCatalogStream
		:flatMap( function( wfCatalog )
				return LrmCatAlbum.getAccessRequests( wfCatalog, albumId )
		end )

	return {
		value = result
	}


end

--------------------------------------------------------------------------------


function LrmLibraryAlbumModelImp:getCollaborators( albumId )


	local result = self._wfCatalogStream
		:flatMap( function( wfCatalog )
				return LrmCatAlbum.getCollaborators( wfCatalog, albumId )
		end )

	return {
		value = result
	}


end

--------------------------------------------------------------------------------


function LrmLibraryAlbumModelImp:getInvites( albumId )

	local result = self._wfCatalogStream
		:flatMap( function( wfCatalog )
				return LrmCatAlbum.getInvites( wfCatalog, albumId )
		end )

	return {
		value = result
	}


end

-------------------------------------------------------------------------------

local function applyUpdate(self,update)

	local result = self._wfCatalogStream
	:flatMap( function( wfCatalog )
		return LrmCatAlbum.updateCollaborator( wfCatalog, update )
	end )

	return {
		value = result
	}


end


--------------------------------------------------------------------------------


function LrmLibraryAlbumModelImp:updateCollaborator( update )

	return applyUpdate(self,update)


end

--------------------------------------------------------------------------------


function LrmLibraryAlbumModelImp:acceptAccessRequest( update )

	return applyUpdate(self,update)

end


--------------------------------------------------------------------------------

function LrmLibraryAlbumModelImp:rejectAccessRequest( update )

	return applyUpdate(self,update)

end

--------------------------------------------------------------------------------

function LrmLibraryAlbumModelImp:inviteByEmail( albumId, email,optAccessLevel )

	local result = self._wfCatalogStream
	:flatMap( function( wfCatalog )
		return LrmCatAlbum.inviteByEmail( wfCatalog, albumId , email ,optAccessLevel)
	end )

	return {
		value = result
	}

end

--------------------------------------------------------------------------------

function LrmLibraryAlbumModelImp:getAllCollaborators()

	local result = self._wfCatalogStream
	:flatMap( function( wfCatalog )
		return LrmCatAlbum.getAllCollaborators( wfCatalog )
	end )

	return {
		value = result
	}

end

--------------------------------------------------------------------------------

function LrmLibraryAlbumModelImp:getAlbumsList( includeAssetCount )

	local result = self._wfCatalogStream
	:flatMap( function( wfCatalog )
		return LrmCatAlbum.getAlbumsList( wfCatalog, includeAssetCount )
	end )

	return {
		value = result
	}

end

--------------------------------------------------------------------------------

function LrmLibraryAlbumModelImp:canDeleteAsset( docValue )

	return {
		value = Observable.just( LrmCatAlbum.canDeleteAsset( docValue ) )
	}

end

--------------------------------------------------------------------------------

function LrmLibraryAlbumModelImp:canDeleteAssetIds( arrayOfDocIds )

	local result = self._wfCatalogStream
		:flatMap( function( wfCatalog )
			return LrmCatAlbum.canDeleteAssetIds( wfCatalog, arrayOfDocIds )
		end )

	return {
		value = result
	}

end

--------------------------------------------------------------------------------

function LrmLibraryAlbumModelImp:hasSharedAlbumsWithDisplayFilters()

	local result = self._wfCatalogStream
		:flatMap( function( wfCatalog )
			return LrmCatAlbum.hasSharedAlbumsWithDisplayFilters( wfCatalog )
		end )

	return {
		value = result
	}

end

--------------------------------------------------------------------------------

function LrmLibraryAlbumModelImp:getMigratedToUnfilteredAlbumId( albumId )

	local result = self._wfCatalogStream
		:flatMap( function( wfCatalog )
			return LrmCatAlbum.getMigratedToUnfilteredAlbumId( wfCatalog, albumId )
		end )

	return {
		value = result
	}

end

--------------------------------------------------------------------------------

function LrmLibraryAlbumModelImp:deleteDisplayFilters( albumId )

	local result = self._wfCatalogStream
	:flatMap( function( wfCatalog )
		return LrmCatAlbum.deleteDisplayFilters( wfCatalog, albumId )
	end )

	return {
		value = result
	}

end

--------------------------------------------------------------------------------

function LrmLibraryAlbumModelImp:leaveSpace( spaceId )

	local result = self._wfCatalogStream
		:flatMap( function( wfCatalog )
			return LrmCatAlbum.leaveSpace( wfCatalog, spaceId )
		end )

	return {
		value = result
	}

end

--------------------------------------------------------------------------------

function LrmLibraryAlbumModelImp:reportAbuse( spaceId, albumId )

	local result = self._wfCatalogStream
		:flatMap( function( wfCatalog )
			return LrmCatAlbum.reportAbuse( wfCatalog, spaceId, albumId )
		end )

	return {
		value = result
	}

end

--------------------------------------------------------------------------------

function LrmLibraryAlbumModelImp:deleteAssets( spaceId, assetIds )

	local result = self._wfCatalogStream
		:flatMap( function( wfCatalog )
			return LrmCatAlbum.deleteAssets( wfCatalog, spaceId, assetIds )
		end )

	return {
		value = result
	}

end

--------------------------------------------------------------------------------

return LrmLibraryAlbumModelImp
