--[[----------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2016 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

	-- WF.lrmobile
local LrmConfig = require 'LrmConfig'

	-- WF.core.utils
local WFArguments = require 'WFArguments'

--------------------------------------------------------------------------------

local check = assert( WFArguments.check )

--==============================================================================

-- @mark LrmLibraryAlbumStatusModelImp namespace

--------------------------------------------------------------------------------

local LrmLibraryAlbumStatusModelImp = {}

--------------------------------------------------------------------------------

LrmConfig.registerBinaryStrategyCheck()

--------------------------------------------------------------------------------

function LrmLibraryAlbumStatusModelImp:unsynchronizedAssetsForAlbum( albumId, binaryDownloadStrategy )

	check( 'table', 'string', 'binaryStrategy' )
	
	local masterSql = [[
		SELECT 
			COUNT( distinct r.docId ) 
		FROM 
			renditionRevisions as r 
		INNER JOIN 
			album_asset 
		ON 
			album_asset.assetId = r.docId 
		WHERE 
			album_asset.albumId == ? 	
		AND
			renditionName IN ( 'thumbnail2x', 'preview', 'proxyWithoutMaster', 'master', 'xmp_develop' )
	]]

	local proxySql = [[
		SELECT 
			COUNT( distinct r.docId ) 
		FROM 
			renditionRevisions as r 
		INNER JOIN 
			album_asset 
		ON 
			album_asset.assetId = r.docId 
		WHERE 
			album_asset.albumId == ? 	
		AND
			renditionName IN ( 'thumbnail2x', 'preview', 'proxy', 'xmp_develop' )
	]]
	
	local query = self.catalog:indexStore():query( { sql = 'SELECT spaceId FROM albums WHERE albums.docId = ?', albumId } )
		:flatMap( function( row )
	
			local sql
	
			-- If the album has a non-null spaceId, implicitly use the proxy workflow
			if row and row[ 1 ] and row[ 1 ][ 1 ] then
				sql = proxySql
			elseif binaryDownloadStrategy == LrmConfig.StrategyPreferMaster then
				sql = masterSql
			else
				sql = proxySql
			end

			return self.catalog:indexStore():query( { sql = sql, calm = 1, albumId } )

		end )
		:map( function( rows )
			if rows and #rows > 0 then
				return rows[1][1]
			end
			
			return nil
		end )

	return {
		value = query,
	}
	
end

--------------------------------------------------------------------------------

function LrmLibraryAlbumStatusModelImp:missingCacheSizeForAlbum( albumId, binaryDownloadStrategy )

	check( 'table', 'string', 'binaryStrategy' )
	
	local masterSql = [[
		SELECT 
			SUM(rr.size) 
		FROM
			renditionRevisions as rr 
		INNER JOIN
			album_asset 
		ON
			album_asset.assetId = rr.docId 
		WHERE
			renditionName IN ( 'thumbnail2x', 'preview', 'proxyWithoutMaster', 'master', 'xmp_develop' ) 		
		AND
			album_asset.albumId == ?
	]]

	local proxySql = [[
		SELECT 
			SUM(rr.size) 
		FROM
			renditionRevisions as rr 
		INNER JOIN
			album_asset 
		ON
			album_asset.assetId = rr.docId 
		WHERE
			renditionName IN ( 'thumbnail2x', 'preview', 'proxy', 'xmp_develop' )		
		AND
			album_asset.albumId == ?
	]]
		
	local query = self.catalog:indexStore():query( { sql = 'SELECT spaceId FROM albums WHERE albums.docId = ?', albumId } )
		:flatMap( function( row )
	
			local sql
	
			-- If the album has a non-null spaceId, implicitly use the proxy workflow
			if row and row[ 1 ] and row[ 1 ][ 1 ] then
				sql = proxySql
			elseif binaryDownloadStrategy == LrmConfig.StrategyPreferMaster then
				sql = masterSql
			else
				sql = proxySql
			end

			return self.catalog:indexStore():query( { sql = sql, calm = 1, albumId } )

		end )
		:map( function( row )

			return row and row[ 1 ] and row[ 1 ][ 1 ] or 0

		end )

	return {
		value = query
	}
	
end

--------------------------------------------------------------------------------

return LrmLibraryAlbumStatusModelImp
