--[[----------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2015 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--
	
	-- WF.core.reactive
local KSReactive = require 'KSReactive'

	-- WF.core.utils
local WFUnknown = require 'WFUnknown'

	-- WF.lrmobile
local LrmAlbumUtils = require 'LrmAlbumUtils'
local LrmCatCollaboration = require 'LrmCatCollaboration'

--------------------------------------------------------------------------------

local LrmLibrarySocialModelImp = {}

--==============================================================================

function LrmLibrarySocialModelImp:addComment( assetId, comment, albumId )

	local resultB = KSReactive.variableB( WFUnknown.value() )
	
	LrmCatCollaboration.addCommentP( self.catalog, albumId, assetId, comment )
	:whenResolvedDo( function( status, data )
	
		if status == 'data' then
			resultB:set( { state = true } )
		else
			resultB:set( { state = false, error = data } )
		end
		
	end ):runUntilDone()
	
	return {
		valueB = resultB,
	}
	
end

--------------------------------------------------------------------------------

function LrmLibrarySocialModelImp:deleteComment( commentId )

	local resultB = KSReactive.variableB( WFUnknown.value() )
	
	LrmCatCollaboration.deleteCommentP( self.catalog, commentId )
		:whenResolvedDo( function( status, data )
		
			if status == 'data' then
				resultB:set( { state = true } )
			else
				resultB:set( { state = false, error = data } )
			end
			
		end ):runUntilDone()
	
	return {
		valueB = resultB,
	}

end

--------------------------------------------------------------------------------

function LrmLibrarySocialModelImp:deleteSpaceFavorite( spacefavoriteId )

	local resultB = KSReactive.variableB( WFUnknown.value() )
	
	LrmCatCollaboration.deleteSpaceFavoriteP( self.catalog, spacefavoriteId )
		:whenResolvedDo( function( status, data )
		
			if status == 'data' then
				resultB:set( { state = true } )
			else
				resultB:set( { state = false, error = data } )
			end
		
		end )
		:runUntilDone()
		
	return {
		value = resultB,
	}
	
end

--------------------------------------------------------------------------------

function LrmLibrarySocialModelImp:toggleMyFavorite( assetId, albumId )

	local resultB = KSReactive.variableB( WFUnknown.value() )
	
	LrmCatCollaboration.toggleMySpaceFavoriteP( self.catalog, assetId, albumId )
		:whenResolvedDo( function( status, data )
		
			if status == 'data' then
				resultB:set( { state = true } )
			else
				resultB:set( { state = false, error = data } )
			end
			
		end )
		:runUntilDone()
		
	return {
		value = resultB
	}
	
end

--------------------------------------------------------------------------------

function LrmLibrarySocialModelImp:shareAlbum( albumId, share )

	local resultB = KSReactive.variableB( WFUnknown.value() )
	
	local changeSharingStateP = nil
	
	if share then
		changeSharingStateP = LrmCatCollaboration.shareAlbumP( self.catalog, albumId )
	else
		changeSharingStateP = LrmCatCollaboration.unshareAlbumP( self.catalog, albumId )
	end
	
	changeSharingStateP:whenResolvedDo( function( status, data )
	
		if status == 'data' then
			resultB:set( { state = true } )
		else
			resultB:set( { state = false, error = data } )
		end
	end ):runUntilDone()
	
	return {
		valueB = resultB,
	}
end

--------------------------------------------------------------------------------

function LrmLibrarySocialModelImp:resolveAlbumFromSpace( spaceId )

	local resultB = KSReactive.variableB( WFUnknown.value() )
	
	LrmCatCollaboration.resolveAlbumFromSpaceP( self.catalog, spaceId )
		:whenResolvedDo( function( status, data )
		
			if status == 'data' and data ~= nil then
				resultB:set( { state = true, albumId = data } )
			else
				resultB:set( { state = false, error = data } )
			end

		end )
		:attachTo( resultB )
		
	return {
		value = resultB
	}
	
end

--------------------------------------------------------------------------------

function LrmLibrarySocialModelImp:getSharedAlbumUrl( albumId )

	local resultB = KSReactive.variableB( WFUnknown.value() )
	
	LrmCatCollaboration.sharedAlbumUrlP( self.catalog, albumId, true )
	:whenResolvedDo( function( status, data )
	
		if status == 'data' then
			
			local shareUrl = data
			resultB:set( { state = true, sharedAlbumUrl = shareUrl } )
		else
			resultB:set( { state = false, error = data } )
		end
	end ):runUntilDone()
	
	return {
		valueB = resultB,
	}
end

--------------------------------------------------------------------------------

return LrmLibrarySocialModelImp
