--[[----------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2017 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

local Rx = require 'Rx'
local WFRxRunUntilDone = require 'WFRxRunUntilDone'

--==============================================================================
local runUntilDone = WFRxRunUntilDone

local LrmSearchModelImp = {}

--------------------------------------------------------------------------------

function LrmSearchModelImp:searchModel()

	return {
	
		value = Rx.ReplaySubject( 1 ),
		
		events = {
			
			updateSearchTable = function( _, startIndex, assetIds )
	
				local value = {}
				for i = 1, #assetIds do
					value[i] = {
						assetIds[i],
						startIndex + i,
					}
				end
				
				return self.catalog:indexStore()
					:bulkInsertManualTable( 'TEMP_search', value )
					:call( runUntilDone )
			end,
			
			
			clearSearchTable = function()
			
				return self.catalog:indexStore()
					:clearManualTable( 'TEMP_search' )
					:call( runUntilDone )
			
			end,
			
			updateSearchFilterCountTable = function( _, filterType, count )
				local value = { 
					{ filterType, count },
				}
				
				return self.catalog:indexStore()
					:bulkInsertManualTable( 'TEMP_searchFilterCount', value )
					:call( runUntilDone )
			end,
			
			clearSearchFilterCountTable = function()
			
				return self.catalog:indexStore()
					:clearManualTable( 'TEMP_searchFilterCount' )
					:call( runUntilDone )
			end,
		}
	}
end

--------------------------------------------------------------------------------

return LrmSearchModelImp