--[[----------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2014 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--


	-- WF.core.reactive
local KSReactive = require 'KSReactive'

	-- WF.core.utils
local WFUnknown = require 'WFUnknown'

	-- WF.lrmobile
local LrmCatCollaboration = require 'LrmCatCollaboration'

--==============================================================================

local LrmSocialActivityFeedModel = {}

local Submodels = {}

--------------------------------------------------------------------------------

local function mapCommentsAndFavorites( comments, favorites, userId )

	if comments == WFUnknown.value() or favorites == WFUnknown.value() 
		or userId == WFUnknown.value() then
		
		return WFUnknown.value()
		
	end
	
	local result = {
		numComments = #comments,
		numFavorites = #favorites,
	}
	
	local activities = {}
	
	local favAuthor = {}
	
	for _, favorite in ipairs( favorites ) do
	
		local authorId = favorite.author.id
		favorite[ 'isMyActivity' ] = authorId == userId
		activities[ #activities + 1 ] = favorite
		
		local favKey = string.format( '%s%s%s', favorite.assetId, favorite.spaceId, authorId )
		favAuthor[ favKey ] = true
	end
	
	for _, comment in ipairs( comments ) do
	
		local authorId = comment.author.id
		
		local isFavorited = false
		local favKey = string.format( '%s%s%s', comment.assetId, comment.spaceId, authorId )
		
		if favAuthor[ favKey ] then
			isFavorited = true
		end
		
		comment[ 'IsCommentFavorited' ] = isFavorited
		comment[ 'isMyActivity' ] = authorId == userId
		activities[ #activities + 1 ] = comment
		
	end
	
	table.sort( activities, function( a, b ) return a.created > b.created end )
	
	result[ 'activities' ] = activities
	return result
	
end

--------------------------------------------------------------------------------

function Submodels:isMyFavoriteModel( assetId, albumId, userId )

	local isMyFavoriteB = LrmCatCollaboration.isMySpaceFavoriteB( self._catalog, assetId, albumId, userId )
		:closingFilterB( function( isMyFavorite ) return isMyFavorite ~= nil end, WFUnknown.value() )
		
	return {
		value = isMyFavoriteB
	}
	
end

--------------------------------------------------------------------------------

function LrmSocialActivityFeedModel.makeSocialActivityModel( catalog, activityContext,
															albumId, assetId )

	local socialFeedB
	
	if activityContext == 'catalog' then
	
		socialFeedB = KSReactive.closingDataCallB(
						mapCommentsAndFavorites,
						LrmCatCollaboration.getCommentsB( catalog ),
						LrmCatCollaboration.getSpaceFavoritesB( catalog ),
						catalog:ozUserIdB() )

	elseif activityContext == 'album' then
	
		socialFeedB = KSReactive.closingDataCallB(
						mapCommentsAndFavorites,
						LrmCatCollaboration.getCommentsForAlbumB( catalog, albumId ),
						LrmCatCollaboration.getSpaceFavoritesOnAlbumB( catalog, albumId ),
						catalog:ozUserIdB() )
	
	elseif activityContext == 'asset' then
	
		socialFeedB = KSReactive.closingDataCallB(
						mapCommentsAndFavorites,
						LrmCatCollaboration.getCommentsOnAlbumAssetB( catalog, albumId, assetId ),
						LrmCatCollaboration.getSpaceFavoritesOnAlbumAssetB( catalog, albumId, assetId ),
						catalog:ozUserIdB() )
	else
	
		socialFeedB = KSReactive.constantB( WFUnknown.value() )
		
	end
	
	return {
		_catalog = catalog,
		
		value = socialFeedB,
		
		models = Submodels,
		
	}
	
end

--------------------------------------------------------------------------------

return LrmSocialActivityFeedModel
