--[[----------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2016 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

	-- WF.core.utils
local WFCloseSupport = require 'WFCloseSupport'

	-- WF.core.lua.threading
local WFRequestDispatcher = require 'WFRequestDispatcher'

--==============================================================================

-- @mark LrmSyncClientMessageProcessor namespace

local LrmSyncClientMessageProcessor = {}

--------------------------------------------------------------------------------

local valueFnEnv = {} -- disallow all function calls

--------------------------------------------------------------------------------

local function parseOptions( optionsString )

	local valueFn = loadstring( 'return ' .. optionsString )
	assert( type( valueFn ) == 'function' )
	setfenv( valueFn, valueFnEnv ) -- disallow all function calls
	
	return valueFn()
	
end

--------------------------------------------------------------------------------

local function createSyncClientImp( options )

	local syncClientImpName = options.syncClientImpName
	local syncClientModule = require( syncClientImpName )
	
	assert( syncClientModule )
	
	local syncImp = syncClientModule( options )
	
	return syncImp
	
end

--------------------------------------------------------------------------------

function LrmSyncClientMessageProcessor.initMessageProcessor( optionsString )

	local options = parseOptions( optionsString )
	
	local syncClientImp = createSyncClientImp( options )

	LrmSyncClientMessageProcessor.attachMessageHandlers( syncClientImp )

end

--------------------------------------------------------------------------------

function LrmSyncClientMessageProcessor.attachMessageHandlers( syncImp )

	WFCloseSupport.mustCloseAtThreadTermination( syncImp )
	
	if rawget( syncImp, 'attachMessageHandlers' ) then
		syncImp:attachMessageHandlers()
	end

	WFRequestDispatcher.onMessageDo {
		message = 'startUploading',
		target = syncImp,
	}

	WFRequestDispatcher.onMessageDo {
		message = 'stopUploading',
		target = syncImp,
	}

	WFRequestDispatcher.onMessageDo {
		message = 'startSync',
		target = syncImp,
	}
	
	WFRequestDispatcher.onMessageDo {
		message = 'startSyncWithoutUploader',
		target = syncImp,
	}

	WFRequestDispatcher.onMessageDo {
		message = 'getDeleteFeedCallback',
		target = syncImp,
	}

	WFRequestDispatcher.onMessageDo {
		message = 'stopSync',
		target = syncImp,
	}

	WFRequestDispatcher.onMessageDo {
		message = 'startAccountsSync',
		target = syncImp,
	}

	WFRequestDispatcher.onMessageDo {
		message = 'stopAccountsSync',
		target = syncImp,
	}

	WFRequestDispatcher.onMessageDo {
		message = 'startSyncInResidualMode',
		target = syncImp,
	}

	WFRequestDispatcher.onMessageDo {
		message = 'stopSyncInResidualMode',
		target = syncImp,
	}
	
	WFRequestDispatcher.onMessageDo {
		message = 'startSpaceSyncWithoutOzCatalog',
		target = syncImp,
	}

	WFRequestDispatcher.onMessageDo {
		message = 'stopSpaceSyncWithoutOzCatalog',
		target = syncImp,
	}

	WFRequestDispatcher.onMessageDo {
		message = 'isPushingMetadata',
		target = syncImp,
	}
	
	WFRequestDispatcher.onMessageDo {
		message = 'isPullingMetadata',
		target = syncImp,
	}
	
	WFRequestDispatcher.onMessageDo {
		message = 'getRequiredUpdateStream',
		target = syncImp,
	}

	WFRequestDispatcher.onMessageDo {
		message = 'isOzCatalogDeleted',
		target = syncImp,
	}

	WFRequestDispatcher.onMessageDo {
		message = 'pausePusherReturningCounter',
		target = syncImp,
	}

	WFRequestDispatcher.onMessageDo {
		message = 'resumePusher',
		target = syncImp,
	}

	WFRequestDispatcher.onMessageDo {
		message = 'getDetailedUploadProgress',
		target = syncImp,
	}
	
	WFRequestDispatcher.enableDiagnosticLogForRequestStats()
	
end

--------------------------------------------------------------------------------

return LrmSyncClientMessageProcessor