--[[----------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2013 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--



	-- WF.core.utils
local KSTableUtils = require 'KSTableUtils'

	--WF.config
local LrmConfig = require 'LrmConfig'

--------------------------------------------------------------------------------

--local logger = WFLogger "LriSyncStatusModel"
--	:setLevel 'debug'


local derefOpt = KSTableUtils.derefOpt

--============================================================================--

local LrmSyncStatusModel = {}
local Model = {}

--------------------------------------------------------------------------------

local function getUploadProgress( catalog )

	local numNeedingUpload = catalog:indexStore():query {

		sql = [[SELECT
			count( DISTINCT sourceAssetId )
		FROM
			needsBinaryUpload
		WHERE
			binaryType = "master"
		]],
		calm = 2,
		quiesceUntilUpToDate = true
	}

	:map( function( progressRows )

		local binaryUpload = derefOpt( progressRows, 1, 1 ) or 0

		return binaryUpload

	end )

	return numNeedingUpload

end

--------------------------------------------------------------------------------

local function getDetailedUploadProgress( catalog )

	local uploadProgressStaus = catalog:getSyncClient():getDetailedUploadProgress()
	:map( function ( progressReport ) 
		return {
			totalFileSize = progressReport.totalBytesNeedToUpload,
			bytesUploaded = progressReport.totalBytesUploadedSoFar
		}
	end )

	return uploadProgressStaus

end

--------------------------------------------------------------------------------

-- @mark Sub models

--------------------------------------------------------------------------------

function Model:makeBinaryDownloadProgressModel( binaryDownloadStrategy )

	binaryDownloadStrategy = binaryDownloadStrategy or LrmConfig.StrategyPreferProxy

	local sql = [[SELECT
					count( DISTINCT r.docId )
				FROM
					renditionRevisions as r
				INNER JOIN
					album_asset
				ON
					album_asset.assetId = r.docId
				WHERE
					album_asset.albumId IN
					(
						SELECT
							docId
						FROM
							albums
						WHERE
							useOffline = 1
					)
				AND	
					renditionName IN %s
				]]

	if binaryDownloadStrategy == LrmConfig.StrategyPreferMaster then
		sql = string.format( sql, "( 'thumbnail2x', 'preview', 'proxyWithoutMaster', 'master' )" )
	else
		sql = string.format( sql, "( 'thumbnail2x', 'preview', 'proxy' )" )
	end

	local progress = self._catalog:indexStore():query( { sql = sql } )
		:map( function( rows )

			return rows[ 1 ][ 1 ]

		end )


	return {
		value = progress,
	}

end

--------------------------------------------------------------------------------

function Model:makeBinaryDownloadProgressAndAlbumModel( binaryDownloadStrategy )

	binaryDownloadStrategy = binaryDownloadStrategy or LrmConfig.StrategyPreferProxy

	local sql = [[SELECT albums.name, count( DISTINCT r.docId ) FROM
					renditionRevisions as r
				INNER JOIN
					album_asset
				ON
					album_asset.assetId = r.docId
				INNER JOIN 
					albums
				ON 
					albums.docId = album_asset.albumId and albums.useOffline = 1
				WHERE
					renditionName IN %s
				]]

	if binaryDownloadStrategy == LrmConfig.StrategyPreferMaster then
		sql = string.format( sql, "( 'thumbnail2x', 'preview', 'proxyWithoutMaster', 'master', 'xmp_develop' )" )
	else
		sql = string.format( sql, "( 'thumbnail2x', 'preview', 'proxy', 'xmp_develop' )" )
	end

	local progress = self._catalog:indexStore():query( { sql = sql } )
		:map( function( rows )

			return { 
				album = rows[ 1 ][ 1 ], 
				count = rows[ 1 ][ 2 ] }

		end )


	return {
		value = progress,
	}

end

--------------------------------------------------------------------------------

function LrmSyncStatusModel.makeModel( catalog )

	assert( catalog )

	local uploadingToOzProgress = getUploadProgress( catalog )

	return {
		_catalog = catalog,

		value = {
			isUploading = catalog:getSyncClient():isPushingMetadata(),
			isDownloading = catalog:getSyncClient():isPullingMetadata(),
			pendingUploadsToOz = uploadingToOzProgress,
			progressBinaryUploads = getDetailedUploadProgress( catalog ),
		},

		models = Model,

		events = {
		},

		close = function( self )


		end
	}

end

--------------------------------------------------------------------------------

return LrmSyncStatusModel
