--[[----------------------------------------------------------------------------

NmbCameraModelUsageIndexDefinition.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2018 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

------------------------------------------------------------------------------]]

local KSTableUtils = require 'KSTableUtils'
local WFArguments = require 'WFArguments'
local WFPlatformInfo = require 'WFPlatformInfo'

--------------------------------------------------------------------------------

local check = assert( WFArguments.check )

--------------------------------------------------------------------------------

local function generateIndexSchema( indexStoreSchema )

	check( 'WFRxIndexStoreSchema' )
	
	if not WFPlatformInfo.hasACR then
		return
	end
				  
	indexStoreSchema:addTable {
	
		sql = [[CREATE TABLE cameraModelsInUse( cameraModel )]],		
		useLocalId = true,
		hasOneRowPerDocument = true,
		docValueFilter = function( docValue )
			return docValue and docValue._deleted ~= true and 
				docValue.subtype == 'image' and docValue.type == 'asset'
		end,
		map = function( docValue, emit )

			local cameraModel = KSTableUtils.derefOpt( docValue, "xmp", "tiff", "Model" )
			if cameraModel then
				emit( string.lower( cameraModel ) )
			end
		end,
	}

end

--==============================================================================

return {
	generateIndexSchema = generateIndexSchema,
}
