--[[----------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2015 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remains the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and are protected by all applicable
intellectual property laws, including trade secret and copyright laws.
Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

	-- WF.core.os
local WFPlatformInfo = require 'WFPlatformInfo'

	-- WF.core.utils
local WFArguments = require 'WFArguments'

	-- WF.core.rxlua
local WFRxNativeObservable = require 'WFRxNativeObservable'

--------------------------------------------------------------------------------

local check = assert( WFArguments.check )

--==============================================================================

local WFCameraRollService = {}

	-- WF.imaging.camera_roll
local WFCameraRollServiceImp = nil 

if WFPlatformInfo.isiOS then
	WFCameraRollServiceImp = require 'WFCameraRollServiceImp'
end

--------------------------------------------------------------------------------

local function verifyCameraRollServiceImp()

	if not WFCameraRollServiceImp then
		error( 'WFCameraRollServiceImp not available on this platform' )
	end
	
end

--==============================================================================

function WFCameraRollService.authorizationStatus()

	verifyCameraRollServiceImp()
	
	return WFRxNativeObservable( WFCameraRollServiceImp.authorizationStatus )

end

--------------------------------------------------------------------------------

function WFCameraRollService.requestAccess()

	verifyCameraRollServiceImp()
	
	return WFRxNativeObservable( WFCameraRollServiceImp.requestAccess )

end

--------------------------------------------------------------------------------

function WFCameraRollService.getAlbums()

	verifyCameraRollServiceImp()
	
	return WFRxNativeObservable( WFCameraRollServiceImp.getAlbums )

end

--------------------------------------------------------------------------------

function WFCameraRollService.getDetailedAlbumInfo( albumId )

	check( '+string' )
	
	verifyCameraRollServiceImp()
	
	return WFRxNativeObservable( WFCameraRollServiceImp.getDetailedAlbumInfo, albumId )

end

--------------------------------------------------------------------------------

function WFCameraRollService.getAlbumAssets( albumId, optLimit )

	check( '+string', '?number' )
	
	verifyCameraRollServiceImp()
	
	optLimit = optLimit or 50
	
	return WFRxNativeObservable( WFCameraRollServiceImp.getAlbumAssets, albumId, optLimit )

end

--------------------------------------------------------------------------------

function WFCameraRollService.getAssetData( assetId )

	check( '+string' )
	
	verifyCameraRollServiceImp()
	
	return WFRxNativeObservable( WFCameraRollServiceImp.getAssetData, assetId )
	
end

--------------------------------------------------------------------------------

function WFCameraRollService.acquireFileDescriptorForOriginal( localIdentifier )

	check( '+string' )
	
	verifyCameraRollServiceImp()
	
	return WFRxNativeObservable( 
			WFCameraRollServiceImp.acquireFileDescriptorForOriginal, 
			localIdentifier 
		)
		
end

--------------------------------------------------------------------------------

function WFCameraRollService.getFileSizeForOriginal( localIdentifier )

	check( '+string' )
	
	verifyCameraRollServiceImp()
	
	return WFRxNativeObservable( 
			WFCameraRollServiceImp.getFileSizeForOriginal, 
			localIdentifier 
		)
		
end

--------------------------------------------------------------------------------

function WFCameraRollService.getAssetInfo( assetId )

	check( '+string' )
	
	verifyCameraRollServiceImp()
	
	return WFRxNativeObservable( WFCameraRollServiceImp.getAssetInfo, assetId )
	
end

--------------------------------------------------------------------------------

function WFCameraRollService.getCameraRollAssets()

	verifyCameraRollServiceImp()
	
	return WFRxNativeObservable( WFCameraRollServiceImp.getCameraRollAssets )

end

--------------------------------------------------------------------------------

function WFCameraRollService.addToAlbum( albumId, assetIds )

	check( '+string', 'table' )
	
	verifyCameraRollServiceImp()
	
	return WFRxNativeObservable( WFCameraRollServiceImp.addToAlbum, albumId, assetIds )

end

--==============================================================================

return WFCameraRollService
