--[[----------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2019 Adobe
All Rights Reserved.

NOTICE: All information contained herein is, and remains
the property of Adobe and its suppliers, if any. The intellectual
and technical concepts contained herein are proprietary to Adobe
and its suppliers and are protected by all applicable intellectual
property laws, including trade secret and copyright laws.
Dissemination of this information or reproduction of this material
is strictly forbidden unless prior written permission is obtained
from Adobe.

----------------------------------------------------------------------------]]--

	-- WF.core.self_test
local WFTestFileUtils = require 'WFTestFileUtils'

	--WF.core.utils
local KSData = require 'KSData'
local KSDate = require 'KSDate'
local KSIdGenerator = require 'KSIdGenerator'
local KSPathUtils = require 'KSPathUtils'
local WFFileUtils = require 'WFFileUtils'
local WFPlatformInfo = require 'WFPlatformInfo'
local WFTableMerge = require 'WFTableMerge'

--============================================================================--

local WFCatAssetTestUtils = {}

--------------------------------------------------------------------------------

local offlinePath = '/Volumes/foo/images'
local offlineVolumeGuid = KSIdGenerator.uuidInLowercaseHex()
local offlineRelativePath = '/foo/images'

if WFPlatformInfo.isWindows then
	offlinePath = 'Q:\foo\images'
	offlineRelativePath = 'foo\images'
end
	
--------------------------------------------------------------------------------

local function createXmpSidecarFile( filePath )

	local xmpDataAsStr = '<x:xmpmeta xmlns:x="adobe:ns:meta/" x:xmptk="Adobe XMP Core 5.6-c128 79.159124, 2016/03/18-14:01:55 (debug)">\
		   <rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">\
		      <rdf:Description rdf:about=""\
		            xmlns:xmp="http://ns.adobe.com/xap/1.0/"\
		            xmlns:dc="http://purl.org/dc/elements/1.1/"\
		            xmlns:exif="http://ns.adobe.com/exif/1.0/"\
		            xmlns:crs="http://ns.adobe.com/camera-raw-settings/1.0/">\
		         <xmp:ModifyDate>2020-12-12T11:10:09.876Z</xmp:ModifyDate>\
		         <xmp:Rating>1</xmp:Rating>\
		         <xmp:CreatorTool>Adobe Nimbus Test</xmp:CreatorTool>\
		         <dc:subject>\
		            <rdf:Seq>\
		               <rdf:li>mary</rdf:li>\
		               <rdf:li>had</rdf:li>\
		               <rdf:li>a little lamb</rdf:li>\
		            </rdf:Seq>\
		         </dc:subject>\
		         <exif:GPSAltitude>10000/2</exif:GPSAltitude>\
		         <exif:GPSAltitudeRef>1</exif:GPSAltitudeRef>\
		         <exif:GPSLongitude>100,5.125S</exif:GPSLongitude>\
		         <exif:GPSLatitude>20,1.5</exif:GPSLatitude>\
		         <crs:RawFileName>IMG_9876.cr2</crs:RawFileName>\
		      </rdf:Description>\
		   </rdf:RDF>\
		</x:xmpmeta>'

	local xmpData = KSData.make( xmpDataAsStr )
	KSData.saveToFile( xmpData, filePath )
	xmpData:close()
	
	return #xmpDataAsStr

end

--------------------------------------------------------------------------------

WFCatAssetTestUtils.createXmpSidecarFile = createXmpSidecarFile

--------------------------------------------------------------------------------

local function createXmpDevelopSettings( filePath )

	local xmpDataAsStr = '<x:xmpmeta xmlns:x="adobe:ns:meta/" x:xmptk="Adobe XMP Core 5.6-c128 79.159124, 2016/03/18-14:01:55 (debug)">\
		   <rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">\
		      <rdf:Description rdf:about=""\
		            xmlns:crs="http://ns.adobe.com/camera-raw-settings/1.0/">\
		         <crs:RawFileName>IMG_9876.cr2</crs:RawFileName>\
		      </rdf:Description>\
		   </rdf:RDF>\
		</x:xmpmeta>'

	local xmpData = KSData.make( xmpDataAsStr )
	KSData.saveToFile( xmpData, filePath )
	xmpData:close()
	
	return #xmpDataAsStr

end

--------------------------------------------------------------------------------

WFCatAssetTestUtils.createXmpDevelopSettings = createXmpDevelopSettings

--------------------------------------------------------------------------------

local tempFolderForTestName = {}

local function getDefaultLocalOnlySettings( rxTest )

	local mockOriginal = KSData.make( 'original_data' )
	local mockProxy = KSData.make( 'proxy_data' )
	local mockPreview = KSData.make( 'preview_data' )
	local mockThumb = KSData.make( 'thumb_data' )
	local mockPreset = KSData.make( 'preset_data' )
	
	local customId = KSIdGenerator.uuidInLowercaseHex()
	
	local testOriginalName = string.format( "original_%s.RAW", customId )
	local testProxyName = string.format( "proxy_%s.DNG", customId )
	local testSidecareName = string.format( "sidecar%s.xmp", customId )
	local testPresetName = string.format( "preset_%s.xmp", customId )
	
	local tempFolder = tempFolderForTestName[ rxTest:testName() ]
	
	if not tempFolder then
		tempFolder = WFTestFileUtils.createTestFolder( rxTest )
	end
	
	tempFolderForTestName[ rxTest:testName() ] = tempFolder
	
	local originalFilePath = KSPathUtils.child( tempFolder, testOriginalName )
	KSData.saveToFile( mockOriginal, originalFilePath )
	local originalFileSize = WFFileUtils.fileAttributes( originalFilePath ).fileSize
	local originalContentId = KSIdGenerator.uuidInLowercaseHex()
	
	local proxyFilePath = KSPathUtils.child( tempFolder, testProxyName )
	KSData.saveToFile( mockProxy, proxyFilePath )
	local proxyFileSize = WFFileUtils.fileAttributes( proxyFilePath ).fileSize
	local proxyContentId = KSIdGenerator.uuidInLowercaseHex()
	
	local xmpSidecarFilePath = KSPathUtils.child( tempFolder, testSidecareName )
	local xmpSidecarFileSize = createXmpSidecarFile( xmpSidecarFilePath )

	local developSettingsSha256 = '3bd8b936cc646bf1296e5ba4a76c410a27beb80809f525a8f5072f0115c54ccc'
	local xmpDevelopSettingsFilePath = KSPathUtils.child( tempFolder, developSettingsSha256 )
	local xmpDevelopSettingsFileSize = createXmpDevelopSettings( xmpDevelopSettingsFilePath )

	-- Note: Desktop rendition information is generated in createDesktopAsset.
	
	local previewFilePath = KSPathUtils.child( tempFolder, KSIdGenerator.uuidInLowercaseHex() )
	KSData.saveToFile( mockPreview, previewFilePath )
	local previewFileSize = WFFileUtils.fileAttributes( previewFilePath ).fileSize
	local previewData = tostring( mockPreview )
	local previewContentId = KSIdGenerator.uuidInLowercaseHex()
	
	local thumbnailFilePath = KSPathUtils.child( tempFolder, KSIdGenerator.uuidInLowercaseHex() )
	KSData.saveToFile( mockThumb, thumbnailFilePath )
	local thumbnailFileSize = WFFileUtils.fileAttributes( thumbnailFilePath ).fileSize
	local thumbnailData = tostring( mockThumb )
	local thumbnailContentId = KSIdGenerator.uuidInLowercaseHex()

	local presetFilePath = KSPathUtils.child( tempFolder, testPresetName )
	KSData.saveToFile( mockPreset, presetFilePath )
	local presetContentId = KSIdGenerator.uuidInLowercaseHex()

	mockOriginal:close()
	mockProxy:close()
	mockPreview:close()
	mockThumb:close()
	mockPreset:close()

	rxTest:finally( function()
	
		WFFileUtils.delete( originalFilePath )
		WFFileUtils.delete( xmpSidecarFilePath )
		WFFileUtils.delete( proxyFilePath )
		WFFileUtils.delete( previewFilePath )
		WFFileUtils.delete( thumbnailFilePath )
		WFFileUtils.delete( xmpDevelopSettingsFilePath )
		WFFileUtils.delete( presetFilePath )
		
	end )

	return {
	
		originalFilePath = originalFilePath,
		originalFileSize = originalFileSize,
		originalContentId = originalContentId,
		originalSha256 = originalContentId,
		
		xmpSidecarFilePath = xmpSidecarFilePath,
		xmpSidecarFileSize = xmpSidecarFileSize,
		
		proxyFilePath = proxyFilePath,
		proxyFileSize = proxyFileSize,
		proxyContentId = proxyContentId,
		
		previewFilePath = previewFilePath,
		previewFileSize = previewFileSize,
		previewData = previewData,
		previewContentId = previewContentId,

		thumbnailFilePath = thumbnailFilePath,
		thumbnailFileSize = thumbnailFileSize,
		thumbnailData = thumbnailData,
		thumbnailContentId = thumbnailContentId,
		
		xmpDevelopSettingsFilePath = xmpDevelopSettingsFilePath,
		xmpDevelopSettingsFileSize = xmpDevelopSettingsFileSize,
		developSettingsSha256 = developSettingsSha256,
		
		presetFilePath = presetFilePath,
		presetContentId = presetContentId,

	}

end

--------------------------------------------------------------------------------

local function getBaseDocValue( assetId, optOverrides, localOnlySettings )

	local docValue = {
	
		_id = assetId,
		type = 'asset',
		subtype = optOverrides.docSubtype or 'image',
		captureDate = KSDate.timeInIso8601Format(),
		importSource = {
			sha256 = localOnlySettings.originalContentId,
		},
		_serverProvidedMetadata = {
			links = {
				[ '/rels/master' ] = { href = '/href/bogus/testing' },
				[ '/rels/proxy_type/2560' ] = { href = '/href/bogus/proxy' }
			}
		}
	}
	
	if optOverrides.noMasterLink then
		docValue._serverProvidedMetadata.links['/rels/master'] = nil
		docValue._serverProvidedMetadata.links['/rels/proxy_type/2560'] = nil
	end
	
	return docValue
end

--------------------------------------------------------------------------------

local function createDesktopAsset( rxTest, wfCatalog, assetId, localOnlySettings, optOverrides )

	optOverrides = optOverrides or {}

	local originalRelativePath = WFFileUtils.removeVolumeFromPath( localOnlySettings.originalFilePath )
	local originalVolumeGuid = WFFileUtils.getVolumeGuidForPath( localOnlySettings.originalFilePath )

	if optOverrides.localOriginalIsOnDisconnectedVolume then
	
		localOnlySettings.originalFilePath = offlinePath
		originalVolumeGuid = offlineVolumeGuid
		originalRelativePath = offlineRelativePath
		
	end
	
	local proxyRelativePath = WFFileUtils.removeVolumeFromPath( localOnlySettings.proxyFilePath )
	local proxyVolumeGuid = WFFileUtils.getVolumeGuidForPath( localOnlySettings.proxyFilePath )
	
	if optOverrides.localProxyIsOnDisconnectedVolume then
	
		localOnlySettings.proxyFilePath = offlinePath
		proxyVolumeGuid = offlineVolumeGuid
		proxyRelativePath = offlineRelativePath
	
	end

	local docValue = getBaseDocValue( assetId, optOverrides, localOnlySettings )

	docValue._localOnly = { 
	
		files = {
		
			original = {
			
				master = {
					relativePath = originalRelativePath,
					volumeGuid = originalVolumeGuid,
					size = localOnlySettings.originalFileSize,
					sha256 = localOnlySettings.originalContentId,
				},
				
				xmp = {
					path = localOnlySettings.xmpSidecarFilePath,
					fileSize = localOnlySettings.xmpSidecarFileSize,
				},
				
			},
			
			proxy = {
				[ '2560' ] = {
					relativePath = proxyRelativePath,
					volumeGuid = proxyVolumeGuid,
					size = localOnlySettings.proxyFileSize,
					sha256 = localOnlySettings.proxyContentId,
				}
			},
			
			xmp_develop = {
				path = localOnlySettings.xmpDevelopSettingsFilePath,
				fileSize = localOnlySettings.xmpDevelopSettingsFileSize,
				sha256 = localOnlySettings.developSettingsSha256,
			}
			
		},
		xmpSidecarExists = true,
		xmpSidecarNeedsUpload = true,
	}
	
	if optOverrides.noLocalOriginal then
		docValue._localOnly.files.original = nil
	end

	if optOverrides.noLocalProxy then
		docValue._localOnly.files.proxy = nil
	end

	if optOverrides.docSubtype == "preset" and not optOverrides.noLocalOriginal then
		-- on desktop a preset binary is handled like an original
		localOnlySettings.presetFilePath = localOnlySettings.originalFilePath
	end

	-- Add in rendition information. Need the fully formed doc value in order to do this.
	
	local NmbLocalRenditionDigest = require 'NmbLocalRenditionDigest'

	local renditionDigest = NmbLocalRenditionDigest.maybeComputeFromDocValue( docValue )

	if renditionDigest then
	
		local mockPreview = KSData.make( 'preview_data' )
		
		local previewFilePath = wfCatalog:fileSpec():pathForPreviewDigestAndLongEdge( 
				docValue._id, renditionDigest, 2048, false )

		local parentFolder = KSPathUtils.parent( previewFilePath )
		WFFileUtils.mkdir( parentFolder )
		KSData.saveToFile( mockPreview, previewFilePath )
		
		localOnlySettings.previewFilePath = previewFilePath
		
		mockPreview:close()
		
		local mockThumb = KSData.make( 'thumb_data' )
		
		local thumbnailFilePath = wfCatalog:fileSpec():pathForPreviewDigestAndLongEdge( 
				docValue._id, renditionDigest, 320, false )

		localOnlySettings.thumbnailFilePath = thumbnailFilePath

		parentFolder = KSPathUtils.parent( thumbnailFilePath )
		WFFileUtils.mkdir( parentFolder )
		KSData.saveToFile( mockThumb, thumbnailFilePath )

		mockThumb:close()

		rxTest:finally( function()
		
			WFFileUtils.delete( previewFilePath )
			WFFileUtils.delete( thumbnailFilePath )
		
		end )
	
	end

	return docValue

end

--------------------------------------------------------------------------------

local function createiOSAsset( rxTest, assetId, localOnlySettings, optOverrides )

	optOverrides = optOverrides or {}

	local docValue = getBaseDocValue( assetId, optOverrides, localOnlySettings )

	docValue._localOnly = { 
		
		files = {
		
			master = {
				path = localOnlySettings.originalFilePath,
				contentId = localOnlySettings.originalContentId,
				fileSize = localOnlySettings.originalFileSize,
			},
		
			latestMaster = {
				filePath = localOnlySettings.originalFilePath,
			},
		
			proxy = {
				path = localOnlySettings.proxyFilePath,
				contentId = localOnlySettings.proxyContentId,
				fileSize = localOnlySettings.proxyFileSize,
			},
		
			thumbnail2x = {
				path = localOnlySettings.thumbnailFilePath,
				contentId = localOnlySettings.thumbnailContentId,
				fileSize = localOnlySettings.thumbnailFileSize,
			},
			
			preview = {
				path = localOnlySettings.previewFilePath,
				contentId = localOnlySettings.previewContentId,
				fileSize = localOnlySettings.previewFileSize,
			},
			
			xmp_develop = {
				path = localOnlySettings.xmpDevelopSettingsFilePath,
				contentId = localOnlySettings.developSettingsSha256,
				fileSize = localOnlySettings.xmpDevelopSettingsFileSize,
			}
			
		}
		
	}

	if optOverrides.noLocalOriginal then
		docValue._localOnly.files.latestMaster = nil
		docValue._localOnly.files.master = nil
	end

	if optOverrides.noLocalProxy then
		docValue._localOnly.files.proxy = nil
	end

	if ( optOverrides.docSubtype == "preset"
		or optOverrides.docSubtype == "profiles"
		or optOverrides.docSubtype == "camera_profile" ) and not optOverrides.noLocalOriginal then
		docValue._localOnly = {
			files = {
				presetProfile = {
					path = localOnlySettings.presetFilePath,
					sha256 = localOnlySettings.presetContentId,
				}
			}
		}
	end

	return docValue

end

--------------------------------------------------------------------------------

local function createAndroidAsset( rxTest, assetId, localOnlySettings, optOverrides )

	optOverrides = optOverrides or {}

	local previewData = KSData.memoryMappedContentsOfFile( localOnlySettings.previewFilePath )
	local thumbnail2xData = KSData.memoryMappedContentsOfFile( localOnlySettings.thumbnailFilePath )

	localOnlySettings.previewMd5 = previewData:digest( 'MD5' )
	localOnlySettings.thumbMd5 = thumbnail2xData:digest( 'MD5' )

	previewData:close()
	thumbnail2xData:close()
	
	local revId = KSIdGenerator.uuidInLowercaseHex()

	local docValue = getBaseDocValue( assetId, optOverrides, localOnlySettings )

	docValue._localOnly = { 
		
		thumbnail2x = {
			md5 = localOnlySettings.thumbMd5,
			referenceHeight = 960,
			referenceWidth = 1280,
			epoch = 1,
			revId = revId,
			size = localOnlySettings.thumbnailFileSize
		},
		
		preview = {
			md5 = localOnlySettings.previewMd5,
			referenceHeight = 960,
			referenceWidth = 1280,
			epoch = 1,
			revId = revId,
			size = localOnlySettings.previewFileSize
		},
		
		proxy = {
			size = localOnlySettings.proxyFileSize,
			filePath = localOnlySettings.proxyFilePath,
			epoch = 1,
		},
		
		originalFilePath = localOnlySettings.originalFilePath,
		
		latestOriginal = {
			filePath = localOnlySettings.originalFilePath,
		},
		
		original = {
			epoch = 1,
			size = localOnlySettings.originalFileSize,
		},
		
		files = {
		
			xmp_develop = {
				path = localOnlySettings.xmpDevelopSettingsFilePath,
				sha256 = localOnlySettings.developSettingsSha256,
			}
			
		}
		
	}

	if optOverrides.noLocalOriginal then
		docValue._localOnly.latestOriginal = nil
		docValue._localOnly.originalFilePath = nil
		docValue._localOnly.original = nil
	end

	if optOverrides.noLocalProxy then
		docValue._localOnly.proxy = nil
	end

	if ( optOverrides.docSubtype == "preset"
		or optOverrides.docSubtype == "profiles" ) and not optOverrides.noLocalOriginal then
		docValue._localOnly = {
			files = {
				presetProfile = {
					path = localOnlySettings.presetFilePath,
					sha256 = localOnlySettings.presetContentId,
				}
			}
		}
	end

	if optOverrides.docSubtype == "camera_profile" and not optOverrides.noLocalOriginal then
		docValue._localOnly = {
			files = {
				original = {
					master = {
						path = localOnlySettings.originalFilePath
					}
				}
			}
		}

	end

	return docValue

end

--------------------------------------------------------------------------------

function WFCatAssetTestUtils.createAssetDocument( rxTest, wfCatalog, assetId, optOverrides )

	optOverrides = optOverrides or {}

	local docValue = {}

	local localOnlySettings = getDefaultLocalOnlySettings( rxTest )

	if WFPlatformInfo.isDesktop then
	
		docValue = createDesktopAsset( rxTest, wfCatalog, assetId, localOnlySettings, optOverrides )
	
	elseif WFPlatformInfo.isiOS then
	
		docValue = createiOSAsset( rxTest, assetId, localOnlySettings, optOverrides )
	
	elseif WFPlatformInfo.isAndroid then
	
		docValue = createAndroidAsset( rxTest, assetId, localOnlySettings, optOverrides )
	
	end

	return docValue, localOnlySettings

end

--------------------------------------------------------------------------------

function WFCatAssetTestUtils.getOverlayToRemoveLocalOriginal( assetId )
	if WFPlatformInfo.isDesktop then
		
		return {
			_id = assetId,
			_localOnly = {
				files = {
					original = { _delete = true }
				}
			}
		} 
		
	elseif WFPlatformInfo.isiOS then

		return {
			_id = assetId,
			_localOnly = {
				files = {
					master = { _delete = true }
				}
			}
		} 
		
	elseif WFPlatformInfo.isAndroid then
	
		return {
			_id = assetId,
			_localOnly = {
				localUrl =  { _delete = true }, 
				originalFilePath = { _delete = true },
				latestOriginal = { _delete = true },
				original = { _delete = true },
				xmpSidecarExists = false,
			}
		} 
	
	end
	
end

--------------------------------------------------------------------------------

function WFCatAssetTestUtils.getOverlayToRemoveLocalProxy( assetId )

	if WFPlatformInfo.isDesktop then
		
		return {
			_id = assetId,
				_localOnly = {
					files = {
						proxy = { _delete = true }
					}
				}
			}
		
	elseif WFPlatformInfo.isiOS then

		return {
			_id = assetId,
			_localOnly = {
				files = {
					proxy = { _delete = true }
				}
			}
		} 
		
	elseif WFPlatformInfo.isAndroid then
	
		return {
			_id = assetId,
			_localOnly = {
				proxy = { _delete = true }
			}
		} 
	
	end
end

--------------------------------------------------------------------------------

function WFCatAssetTestUtils.getOriginalPathOverlay( assetId, sha256, originalFilePath )
	
	local originalFileSize = WFFileUtils.fileAttributes( originalFilePath ).fileSize
	
	local overlay = {}
	
	if WFPlatformInfo.isDesktop then
	
		overlay = {
			_id = assetId,
			_localOnly = {
				files = {
					original = {
						master = {
							relativePath = WFFileUtils.removeVolumeFromPath( originalFilePath ),
							volumeGuid = WFFileUtils.getVolumeGuidForPath( originalFilePath ),
							size = originalFileSize,
							sha256 = sha256,
						},
					},
				},
				xmpSidecarExists = false,
			}
		}
		
	elseif WFPlatformInfo.isiOS then
		
		overlay = {
			_id = assetId,
			_localOnly = { 
				files = {	
					master = {
						path = originalFilePath,
						contentId = sha256,
						fileSize = originalFileSize,
					},
				},
				xmpSidecarExists = false,
			}
		}

	elseif WFPlatformInfo.isAndroid then
		
		
		overlay = {
			_id = assetId,
			_localOnly = {
				-- this is a fake id (not the actual format of the field) and would apply only to local imports
				-- it's necessary for the binary uploader tests for missing originals
				localUrl = KSIdGenerator.uuidInLowercaseHex(), 
				originalFilePath = originalFilePath,
		
				latestOriginal = {
					filePath = originalFilePath,
				},
				
				original = {
					epoch = 1,
					size = originalFileSize,
				},
				xmpSidecarExists = false,
			}
		}
	end
	
	return overlay
	
end

--------------------------------------------------------------------------------

function WFCatAssetTestUtils.addOriginalPathToDocument( baseAssetDoc, sha256, originalFilePath )
	
	baseAssetDoc = WFTableMerge.applyOverlay( baseAssetDoc, WFCatAssetTestUtils.getOriginalPathOverlay( baseAssetDoc._id, sha256, originalFilePath ) )
	
	return baseAssetDoc
end

--------------------------------------------------------------------------------

function WFCatAssetTestUtils.getProxyPathOverlay( assetId, proxyFilePath )
	
	local proxyFileSize = WFFileUtils.fileAttributes( proxyFilePath ).fileSize
	
	local overlay = {}
	
	if WFPlatformInfo.isDesktop then
	
		overlay = {
			_id = assetId,
			_localOnly = {
				files = {
					proxy = {
						['2560'] = {
							relativePath = WFFileUtils.removeVolumeFromPath( proxyFilePath ),
							volumeGuid = WFFileUtils.getVolumeGuidForPath( proxyFilePath ),
							size = proxyFileSize,
						},
					},
				},
			}
		}
		
	elseif WFPlatformInfo.isiOS then
	
		overlay = {
			_id = assetId,
			_localOnly = { 
				files = {	
					proxy = {
						path = proxyFilePath,
						size = proxyFileSize,
						epoch = 1,
					},
				}
			}
		}
	
	elseif WFPlatformInfo.isAndroid then
		
		overlay = {
			_id = assetId,
			_localOnly = {
				proxy = {
					size = proxyFileSize,
					filePath = proxyFilePath,
					epoch = 1,
				},
			},
		}
		
	end
	
	return overlay
end

--------------------------------------------------------------------------------

function WFCatAssetTestUtils.addProxyPathToDocument( baseAssetDoc, proxyFilePath )
	
	baseAssetDoc = WFTableMerge.applyOverlay( baseAssetDoc, WFCatAssetTestUtils.getProxyPathOverlay( baseAssetDoc._id, proxyFilePath ) )
	
	return baseAssetDoc
end

--------------------------------------------------------------------------------

function WFCatAssetTestUtils.addBasicOzMetadataUpdate( baseAssetDoc, catalogId, assetId, revisionId )
	
	local hrefPrefaceForCreate = '/v2/catalogs/' .. catalogId .. '/assets/' .. assetId .. '/revisions/{revision_id}/'
		
	-- add the oz metadata overlay so we know where to upload it to
	local overlay = { 
		_serverProvidedMetadata = {
			links = {
				[ '/rels/master_create' ] = {
					href = hrefPrefaceForCreate .. 'master',
					templated = true,
				},
				[ '/rels/proxy_create' ] = {
					href = hrefPrefaceForCreate .. 'proxies/{proxy_id}',
					templated = true,
				},
				[ '/rels/xmp_develop_create' ] = {
					href = hrefPrefaceForCreate .. 'xmp/develop',
					templated = true,
				},
				[ '/rels/xmp_create' ] = {
					href = hrefPrefaceForCreate .. 'xmp',
					templated = true,
				},
			},
			revision_ids = {
				revisionId,
			},
		}
	}
	
	baseAssetDoc = WFTableMerge.applyOverlay( baseAssetDoc, overlay )
	
	return baseAssetDoc
end

--------------------------------------------------------------------------------

function WFCatAssetTestUtils.addBasicOzMetadataUpdateForIACAsset( baseAssetDoc, spaceId, assetId, revisionId )
	
	local hrefPrefaceForCreate = '/v2/spaces/' .. spaceId .. '/assets/' .. assetId .. '/revisions/{revision_id}/'
		
	-- add the oz metadata overlay so we know where to upload it to
	local overlay = { 
		_serverProvidedMetadata = {
			links = {
				[ '/rels/master_create' ] = {
					href = hrefPrefaceForCreate .. 'master',
					templated = true,
				},
				[ '/rels/proxy_create' ] = {
					href = hrefPrefaceForCreate .. 'proxies/{proxy_id}',
					templated = true,
				},
				[ '/rels/xmp_develop_create' ] = {
					href = hrefPrefaceForCreate .. 'xmp/develop',
					templated = true,
				},
				[ '/rels/xmp_create' ] = {
					href = hrefPrefaceForCreate .. 'xmp',
					templated = true,
				},
			},
			revision_ids = {
				revisionId,
			},
		}
	}
	
	baseAssetDoc = WFTableMerge.applyOverlay( baseAssetDoc, overlay )
	
	return baseAssetDoc
end

--------------------------------------------------------------------------------

function WFCatAssetTestUtils.createBaseAssetDocument( catalogId, assetId, revisionId, sha256, originalFilePath )
	
	local originalFileSize
	local fileName
	if originalFilePath then
		originalFileSize = WFFileUtils.fileAttributes( originalFilePath ).fileSize
		fileName = KSPathUtils.leafName( originalFilePath )
	end
	
	return {
		_id = assetId,
		_newRevId = revisionId,
		type = 'asset',
		subtype = 'image',
		captureDate = "2015-06-08T08:19:59",

		importSource = {
			originalHeight = 3840,
			originalWidth = 5760,
			importedOnDevice = "Macbook Pro",
			importTimestamp = "2017-04-15T11:03:40Z",
			contentType = "image/jpeg",
			sha256 = sha256,
			fileSize = originalFileSize,
			importedBy = "129d51066a7a3a5711fa62810b758fc8",
			fileName = fileName,
		},
	}

end

--------------------------------------------------------------------------------

return WFCatAssetTestUtils
