--[[----------------------------------------------------------------------------

WFCatCameraCanonicalMap.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2015 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remains the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and are protected by all applicable
intellectual property laws, including trade secret and copyright laws.
Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

--==============================================================================
-- Contains a mapping between the xmp make/model name and its common canonical name.
-- The same mapping is used by Oz when curating the camera name for each asset.
-- The camera name curated by Oz is used by USS for filtering and facet-search.
-- We are using the same map to ensure consistency between local and online search.
--==============================================================================

local _canonical_make_map = {}
-- make canonical names, should be consistent with: 
-- https://git.corp.adobe.com/oz/federated-index-worker/blob/master/data/camera/make/canonical.json
_canonical_make_map["CASIO COMPUTER CO.,LTD."]= "Casio"
_canonical_make_map["Creo/Leaf"]= "Leaf"
_canonical_make_map["EASTMAN KODAK COMPANY"]= "Kodak"
_canonical_make_map["Hewlett-Packard Company"]= "Hewlett-Packard"
_canonical_make_map["Konica Minolta Camera, Inc."]= "Konica Minolta"
_canonical_make_map["LEICA CAMERA AG"]= "Leica"
_canonical_make_map["Leica Camera AG"] = "Leica"
_canonical_make_map["LG Mobile"] = "LG"
_canonical_make_map["Mamiya-OP Co.,Ltd."] = "Mamiya"
_canonical_make_map["Minolta Co., Ltd."] = "Minolta"
_canonical_make_map["NIKON CORPORATION"] = "Nikon"
_canonical_make_map["OLYMPUS CORP."] = "Olympus"
_canonical_make_map["OLYMPUS CORPORATION"] = "Olympus"
_canonical_make_map["OLYMPUS IMAGING CORP."] = "Olympus"
_canonical_make_map["OLYMPUS OPTICAL CO.,LTD"] = "Olympus"
_canonical_make_map["PENTAX Corporation"] = "PENTAX"
_canonical_make_map["PENTAX RICOH IMAGING"] = "PENTAX"
_canonical_make_map["Phase One A/S"] = "Phase One"
_canonical_make_map["RICOH IMAGING COMPANY, LTD."] = "RICOH"
_canonical_make_map["SAMSUNG TECHWIN"] = "SAMSUNG"
_canonical_make_map["SAMSUNG TECHWIN Co."] = "SAMSUNG"
_canonical_make_map["SEIKO EPSON CORP."] = "SEIKO"


local _canonical_model_map = {}
-- a-mount model canonical names, should be consistent with
-- https://git.corp.adobe.com/oz/federated-index-worker/blob/master/data/camera/model/a-mount.json
_canonical_model_map["DSLR-A900"] = "Alpha α900"
_canonical_model_map["DSLR-A850"] = "Alpha α850"
_canonical_model_map["SLT-A99V"] = "Alpha α99"
_canonical_model_map["SLT-A99"] = "Alpha α99"
_canonical_model_map["HV"] = "Alpha α99"
_canonical_model_map["ILCA-99M2"] = "Alpha α99 II"
_canonical_model_map["DSLR-A100"] = "Alpha α100"
_canonical_model_map["DSLR-A100/S"] = "Alpha α100"
_canonical_model_map["DSLR-A700"] = "Alpha α700"
_canonical_model_map["DSLR-A200"] = "Alpha α200"
_canonical_model_map["DSLR-A300"] = "Alpha α300"
_canonical_model_map["DSLR-A300K/N"] = "Alpha α300"
_canonical_model_map["DSLR-A350"] = "Alpha α350"
_canonical_model_map["DSLR-A350K/N"] = "Alpha α350"
_canonical_model_map["DSLR-A230"] = "Alpha α230"
_canonical_model_map["DSLR-A330"] = "Alpha α330"
_canonical_model_map["DSLR-A330L/T"] = "Alpha α330"
_canonical_model_map["DSLR-A380"] = "Alpha α380"
_canonical_model_map["DSLR-A500"] = "Alpha α500"
_canonical_model_map["DSLR-A550"] = "Alpha α550"
_canonical_model_map["DSLR-A450"] = "Alpha α450"
_canonical_model_map["DSLR-A290"] = "Alpha α290"
_canonical_model_map["DSLR-A390"] = "Alpha α390"
_canonical_model_map["DSLR-A560"] = "Alpha α560"
_canonical_model_map["DSLR-A580"] = "Alpha α580"
_canonical_model_map["SLT-A33"] = "Alpha α33"
_canonical_model_map["SLT-A55V"] = "Alpha α55"
_canonical_model_map["SLT-A55"] = "Alpha α55"
_canonical_model_map["SLT-A35"] = "Alpha α35"
_canonical_model_map["SLT-A65V"] = "Alpha α65"
_canonical_model_map["SLT-A65"] = "Alpha α65"
_canonical_model_map["SLT-A77V"] = "Alpha α77"
_canonical_model_map["SLT-A77"] = "Alpha α77"
_canonical_model_map["SLT-A57"] = "Alpha α57"
_canonical_model_map["SLT-A37"] = "Alpha α37"
_canonical_model_map["SLT-A58"] = "Alpha α58"
_canonical_model_map["ILCA-77M2"] = "Alpha α77 II"
_canonical_model_map["ILCA-68"] = "Alpha α68"

-- e-mount model canonical names, should be consistent with
-- https://git.corp.adobe.com/oz/federated-index-worker/blob/master/data/camera/model/e-mount.json 
_canonical_model_map["ILCE-3000"] = "Alpha α3000"
_canonical_model_map["ILCE-7"] = "Alpha α7"
_canonical_model_map["ILCE-7R"] = "Alpha α7R"
_canonical_model_map["Lusso"] = "Alpha α7R"
_canonical_model_map["ILCE-5000"] = "Alpha α5000"
_canonical_model_map["ILCE-6000"] = "Alpha α6000"
_canonical_model_map["ILCE-3500"] = "Alpha α3500"
_canonical_model_map["ILCE-7S[4]"] = "Alpha α7S"
_canonical_model_map["ILCE-5100"] = "Alpha α5100"
_canonical_model_map["ILCE-7M2"] = "Alpha α7 II"
_canonical_model_map["ILCE-7RM2"] = "Alpha α7R II"
_canonical_model_map["ILCE-7SM2"] = "Alpha α7S II"
_canonical_model_map["ILCE-6300"] = "Alpha α6300"
_canonical_model_map["ILCE-6500"] = "Alpha α6500"
_canonical_model_map["ILCE-9"] = "Alpha α9"
_canonical_model_map["ILCE-7RM3"] = "Alpha α7R III"
_canonical_model_map["ILCE-7M3"] = "Alpha α7 III"
_canonical_model_map["ILCE-QX1"] = "Alpha αQX1"
_canonical_model_map["NEX-VG10E"] = "NEX-VG10 NEX-VG10E"
_canonical_model_map["NEX-VG10A"] = "NEX-VG10 NEX-VG10A"
_canonical_model_map["NEX-VG10J"] = "NEX-VG10 NEX-VG10J"
_canonical_model_map["NEX-VG20E"] = "NEX-VG20 NEX-VG20E"
_canonical_model_map["NEX-VG20A"] = "NEX-VG20 NEX-VG20A"
_canonical_model_map["NEX-VG20J"] = "NEX-VG20 NEX-VG20J"
_canonical_model_map["NEX-VG30E"] = "NEX-VG30"
_canonical_model_map["NEX-VG900E"] = "NEX-VG900"
_canonical_model_map["NEX-FS100E"] = "NEX-FS100 NEX-FS100E"
_canonical_model_map["NEX-FS100U"] = "NEX-FS100 NEX-FS100U"
_canonical_model_map["NEX-FS100J"] = "NEX-FS100 NEX-FS100J"
_canonical_model_map["NEX-FS700E"] = "NEX-FS700 NEX-FS700E"
_canonical_model_map["NEX-FS700U"] = "NEX-FS700 NEX-FS700U"
_canonical_model_map["NEX-FS700J"] = "NEX-FS700 NEX-FS700J"
_canonical_model_map["NEX-FS700R"] = "NEX-FS700 NEX-FS700R"
_canonical_model_map["NEX-EA50E"] = "NEX-EA50 NEX-EA50E"
_canonical_model_map["NEX-EA50U"] = "NEX-EA50 NEX-EA50U"
_canonical_model_map["PXW-FS7"] = "PXW-FS7 II"
_canonical_model_map["PXW-FS5"] = "PXW-FS5"


local WFCatCameraCanonicalMap = {}

WFCatCameraCanonicalMap["ModelMap"] = _canonical_model_map
WFCatCameraCanonicalMap["MakeMap"] = _canonical_make_map

return WFCatCameraCanonicalMap
