--[[----------------------------------------------------------------------------

WFCatSearchByTextConstants.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2015 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remains the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and are protected by all applicable
intellectual property laws, including trade secret and copyright laws.
Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--
local WFZString = require 'WFZString'
local ZSTR = assert( WFZString.fromZString )

	-- WF.core
local WFConfiguration = require 'WFConfiguration'

--==============================================================================

local WFCatSearchByTextConstants = {}

-- this has to be in sync with import "AgConfiguration".Features.enableCategorySearch
--local ENABLE_CATEGORY_SEARCH = WFConfiguration.Search.enableCategorySearch
-- Search With String below to get more configurable search features
-- [ Configurable Search Features ] currently we don't need configuration, as this is not a dynanmic decisionn
WFCatSearchByTextConstants.ENABLE_CATEGORY_SEARCH = false

-- This has to be in sync with import "AgConfiguration".Features.enableSearchCompletion
-- local ENABLE_SEARCH_COMPLETIONS = WFConfiguration.Search.enableSearchCompletion
-- Search With String below to get more configurable search features
-- [ Configurable Search Features ] currently we don't need configuration, as this is not a dynanmic decision
WFCatSearchByTextConstants.ENABLE_SEARCH_COMPLETIONS = false


-- this has to match with AgConfiguration.searchConfiguration.isSearchOnline
local isSearchOnline = WFConfiguration.searchConfiguration.isSearchOnline
if isSearchOnline  == nil then
	isSearchOnline = true -- making the default as online search , as(13.07.2017)
end

WFCatSearchByTextConstants.ENABLE_LOCAL_TEXT_SEARCH  =  not isSearchOnline

local tableVersionNumber = "_v1"
local text_search = "text_search_"
local textTerm = 'text'
--------------------------------------------------------------------------------
function WFCatSearchByTextConstants.getTableName( tableNameCore )

	local tableNameWithVersion = string.format("%s%s", tableNameCore, tableVersionNumber )
	return tableNameWithVersion
end

--------------------------------------------------------------------------------
function WFCatSearchByTextConstants.getCategoryTableName( categoryName )

	local tableNameWithVersion = string.format("%s%s%s", text_search, categoryName, tableVersionNumber )
	return tableNameWithVersion
end

--------------------------------------------------------------------------------
function WFCatSearchByTextConstants.getTermName( categoryName )
	return string.format("%s_%s", textTerm, categoryName )
end


--------------------------------------------------------------------------------

WFCatSearchByTextConstants.NoExtension = ZSTR "$$$/WFCatSearch/Extension/NoExtension=NoExtension"

WFCatSearchByTextConstants.extraWordSeparator = "WF&&EXTRA&&WF"

WFCatSearchByTextConstants.pickValue = "pick"
WFCatSearchByTextConstants.rejectValue = "reject"
WFCatSearchByTextConstants.FlaggedString = ZSTR "$$$/WFCatSearch/Flagged=Flagged"
WFCatSearchByTextConstants.UnFlaggedString = ZSTR "$$$/WFCatSearch/Unflagged=Unflagged"
WFCatSearchByTextConstants.PickedString = ZSTR "$$$/WFCatSearch/Picked=Picked"
WFCatSearchByTextConstants.RejectedString = ZSTR "$$$/WFCatSearch/Rejected=Rejected"
WFCatSearchByTextConstants.ISOString = ZSTR "$$$/WFCatSearch/ISO=ISO"
WFCatSearchByTextConstants.FStopString = ZSTR "$$$/WFCatSearch/FStopSymbol=f/"
WFCatSearchByTextConstants.FocalLengthString = ZSTR "$$$/WFCatSearch/FocalLengthInMM=mm"
WFCatSearchByTextConstants.ExposureUnitString = ZSTR "$$$/WFCatSearch/ExposureUnit=sec"
WFCatSearchByTextConstants.ExposureSeparator = "/"


-- not space for RatingStringSingular, RatingStringSingular, as it will be made as seperate term for search
-- other terms the additioanl strig will be removed for search.
-- for rating even if the other strings are removed since the 
-- number can be present in other metadata, it will make the search meaningless

WFCatSearchByTextConstants.RatingStringSingular = ZSTR "$$$/WFCatSearch/StarRatingSingular=-Star"
WFCatSearchByTextConstants.RatingStringPlural = ZSTR "$$$/WFCatSearch/StarRatingPlural=-Stars"

-- the additional string, which will be removed
WFCatSearchByTextConstants.CameraString = ZSTR "$$$/WFCatSearch/Camera=Camera "
WFCatSearchByTextConstants.LensString = ZSTR "$$$/WFCatSearch/Lens=Lens "
WFCatSearchByTextConstants.YearString = ZSTR "$$$/WFCatSearch/Year=Year "
WFCatSearchByTextConstants.MonthString = ZSTR "$$$/WFCatSearch/Month=Month "
WFCatSearchByTextConstants.DateString = ZSTR "$$$/WFCatSearch/Date=Date "
WFCatSearchByTextConstants.LocationString = ZSTR "$$$/WFCatSearch/Location=Location "
WFCatSearchByTextConstants.RatingString1 = ZSTR "$$$/WFCatSearch/Rating=Rating "
WFCatSearchByTextConstants.RatingString2 = ZSTR "$$$/WFCatSearch/StarRating=Star "
WFCatSearchByTextConstants.TitleString = ZSTR "$$$/WFCatSearch/Title=Title "
WFCatSearchByTextConstants.Captiontring = ZSTR "$$$/WFCatSearch/Caption=Caption "
WFCatSearchByTextConstants.FlagString = ZSTR "$$$/WFCatSearch/Flag=Flag "
WFCatSearchByTextConstants.AssetTypeString1 = ZSTR "$$$/WFCatSearch/AssetType1=AssetType "
WFCatSearchByTextConstants.AssetTypeString2 = ZSTR "$$$/WFCatSearch/AssetType2=Type "
WFCatSearchByTextConstants.AssetTypeImageString1 = ZSTR "$$$/WFCatSearch/AssetTypeImage1=Photos "
WFCatSearchByTextConstants.AssetTypeImageString2 = ZSTR "$$$/WFCatSearch/AssetTypeImage2=Images "
WFCatSearchByTextConstants.AssetTypeVideoString = ZSTR "$$$/WFCatSearch/AssetTypeVideo=Videos "
WFCatSearchByTextConstants.ExtensionString1 = ZSTR "$$$/WFCatSearch/Extension1=Extension "
WFCatSearchByTextConstants.ExtensionString2 = ZSTR "$$$/WFCatSearch/Extension2=File "
WFCatSearchByTextConstants.FStopAdditionalStart1 = ZSTR "$$$/WFCatSearch/FNumber=FNumber "
WFCatSearchByTextConstants.FStopAdditionalStart2 = ZSTR "$$$/WFCatSearch/FStop=FStop "
WFCatSearchByTextConstants.FocalLengthAdditionalStart = ZSTR "$$$/WFCatSearch/FocalLength=FocalLength "
WFCatSearchByTextConstants.ExposureAdditionalStart1 = ZSTR "$$$/WFCatSearch/Exposure=Exposure "
WFCatSearchByTextConstants.ExposureAdditionalStart2 = ZSTR "$$$/WFCatSearch/ShutterSpeed=ShutterSpeed "
WFCatSearchByTextConstants.KeywordStart = ZSTR "$$$/WFCatSearch/Keyword=Keyword "



WFCatSearchByTextConstants.Months = {
	ZSTR "$$$/WFCatSearch/January=January",
	ZSTR "$$$/WFCatSearch/February=February",
	ZSTR "$$$/WFCatSearch/March=March",
	ZSTR "$$$/WFCatSearch/April=April",
	ZSTR "$$$/WFCatSearch/May=May",
	ZSTR "$$$/WFCatSearch/June=June",
	ZSTR "$$$/WFCatSearch/July=July",
	ZSTR "$$$/WFCatSearch/August=August",
	ZSTR "$$$/WFCatSearch/September=September",
	ZSTR "$$$/WFCatSearch/October=October",
	ZSTR "$$$/WFCatSearch/November=November",
	ZSTR "$$$/WFCatSearch/December=December",
	}

WFCatSearchByTextConstants.category_locationKeyString		= 'location'
WFCatSearchByTextConstants.category_keywordsKeyString		= 'keywords'
WFCatSearchByTextConstants.category_autoTagsKeyString		= 'autoTags'
WFCatSearchByTextConstants.category_captionKeyString		= 'caption'
WFCatSearchByTextConstants.category_titleKeyString		= 'title'
WFCatSearchByTextConstants.category_cameraKeyString		= 'camera'
WFCatSearchByTextConstants.category_lensKeyString			= 'lens'
WFCatSearchByTextConstants.category_yearKeyString			= 'year'
WFCatSearchByTextConstants.category_monthKeyString		= 'month'
WFCatSearchByTextConstants.category_dateKeyString			= 'date'
WFCatSearchByTextConstants.category_completeDateKeyString = 'completedate'
WFCatSearchByTextConstants.category_ratingKeyString		= 'rating'
WFCatSearchByTextConstants.category_flagKeyString			= 'flag'
WFCatSearchByTextConstants.category_assetTypeKeyString	= 'assettype'
WFCatSearchByTextConstants.category_fileExtensionKeyString= 'extension'
WFCatSearchByTextConstants.category_isoKeyString			= 'iso'
WFCatSearchByTextConstants.category_fStopKeyString		= 'fstop'
WFCatSearchByTextConstants.category_focalLengthKeyString	= 'focallength'
WFCatSearchByTextConstants.category_exposureKeyString		= 'exposure'


WFCatSearchByTextConstants.categoryBasedSearchNotNeeded = {}
WFCatSearchByTextConstants.categoryBasedSearchNotNeeded[ WFCatSearchByTextConstants.category_autoTagsKeyString ] = true


WFCatSearchByTextConstants.categories = { }
WFCatSearchByTextConstants.categories[ WFCatSearchByTextConstants.category_locationKeyString ] 	
		= ZSTR "$$$/WFCatSearch/category/location=location"
WFCatSearchByTextConstants.categories[ WFCatSearchByTextConstants.category_keywordsKeyString ] 	
		= ZSTR "$$$/WFCatSearch/category/keyword=keyword"
WFCatSearchByTextConstants.categories[ WFCatSearchByTextConstants.category_autoTagsKeyString ] 	
		= ZSTR "$$$/WFCatSearch/category/autoTag=autoTag"
WFCatSearchByTextConstants.categories[ WFCatSearchByTextConstants.category_captionKeyString ] 		
		= ZSTR "$$$/WFCatSearch/category/caption=caption"
WFCatSearchByTextConstants.categories[ WFCatSearchByTextConstants.category_titleKeyString ] 		
		= ZSTR "$$$/WFCatSearch/category/title=title"
WFCatSearchByTextConstants.categories[ WFCatSearchByTextConstants.category_cameraKeyString ] 	
		= ZSTR "$$$/WFCatSearch/category/camera=camera"
WFCatSearchByTextConstants.categories[ WFCatSearchByTextConstants.category_lensKeyString ] 		
		= ZSTR "$$$/WFCatSearch/category/lens=lens"
WFCatSearchByTextConstants.categories[ WFCatSearchByTextConstants.category_yearKeyString ] 		
		= ZSTR "$$$/WFCatSearch/category/year=year"
WFCatSearchByTextConstants.categories[ WFCatSearchByTextConstants.category_monthKeyString ] 		
		= ZSTR "$$$/WFCatSearch/category/month=month"
WFCatSearchByTextConstants.categories[ WFCatSearchByTextConstants.category_dateKeyString ] 	 	
		= ZSTR "$$$/WFCatSearch/category/date=date"
WFCatSearchByTextConstants.categories[ WFCatSearchByTextConstants.category_completeDateKeyString ] 	 	
		= ZSTR "$$$/WFCatSearch/category/completedate=complete date"
WFCatSearchByTextConstants.categories[ WFCatSearchByTextConstants.category_ratingKeyString ] 	
		= ZSTR "$$$/WFCatSearch/category/rating=rating"
WFCatSearchByTextConstants.categories[ WFCatSearchByTextConstants.category_flagKeyString ] 	
		= ZSTR "$$$/WFCatSearch/category/flag=flag"
WFCatSearchByTextConstants.categories[ WFCatSearchByTextConstants.category_assetTypeKeyString ] 	
		= ZSTR "$$$/WFCatSearch/category/assettype=asset type"
WFCatSearchByTextConstants.categories[ WFCatSearchByTextConstants.category_fileExtensionKeyString ] 	
		= ZSTR "$$$/WFCatSearch/category/fileExtension=file extension"
WFCatSearchByTextConstants.categories[ WFCatSearchByTextConstants.category_isoKeyString ] 	
		= ZSTR "$$$/WFCatSearch/category/ISO=iso"
WFCatSearchByTextConstants.categories[ WFCatSearchByTextConstants.category_fStopKeyString ] 	
		= ZSTR "$$$/WFCatSearch/category/FStop=fstop"
WFCatSearchByTextConstants.categories[ WFCatSearchByTextConstants.category_focalLengthKeyString ] 	
		= ZSTR "$$$/WFCatSearch/category/Focallength=focal length"
WFCatSearchByTextConstants.categories[ WFCatSearchByTextConstants.category_exposureKeyString ] 	
		= ZSTR "$$$/WFCatSearch/category/Exposure=exposure"



WFCatSearchByTextConstants.perCategoryExtraStartMap = {}

--------------------------------------------------------------------------------
WFCatSearchByTextConstants.perCategoryExtraStartMap[ WFCatSearchByTextConstants.categories[ string.lower( WFCatSearchByTextConstants.category_cameraKeyString ) ] ] =
						{ WFCatSearchByTextConstants.CameraString }
						
WFCatSearchByTextConstants.perCategoryExtraStartMap[ WFCatSearchByTextConstants.categories[ string.lower( WFCatSearchByTextConstants.category_lensKeyString ) ] ] = 
						{ WFCatSearchByTextConstants.LensString }
						
WFCatSearchByTextConstants.perCategoryExtraStartMap[ WFCatSearchByTextConstants.categories[ string.lower( WFCatSearchByTextConstants.category_yearKeyString ) ] ] =
						{ WFCatSearchByTextConstants.YearString } 
						
WFCatSearchByTextConstants.perCategoryExtraStartMap[ WFCatSearchByTextConstants.categories[ string.lower( WFCatSearchByTextConstants.category_monthKeyString ) ] ] =
						{ WFCatSearchByTextConstants.MonthString } 	
						
WFCatSearchByTextConstants.perCategoryExtraStartMap[ WFCatSearchByTextConstants.categories[ string.lower( WFCatSearchByTextConstants.category_dateKeyString ) ] ] =
						{ WFCatSearchByTextConstants.DateString } 
						
WFCatSearchByTextConstants.perCategoryExtraStartMap[ WFCatSearchByTextConstants.categories[ string.lower( WFCatSearchByTextConstants.category_locationKeyString ) ] ] =
						{ WFCatSearchByTextConstants.LocationString } 
						
WFCatSearchByTextConstants.perCategoryExtraStartMap[ WFCatSearchByTextConstants.categories[ string.lower( WFCatSearchByTextConstants.category_ratingKeyString ) ] ] =
						{ WFCatSearchByTextConstants.RatingString1, WFCatSearchByTextConstants.RatingString2 } 
						
WFCatSearchByTextConstants.perCategoryExtraStartMap[ WFCatSearchByTextConstants.categories[ string.lower( WFCatSearchByTextConstants.category_captionKeyString ) ] ] =
						{ WFCatSearchByTextConstants.Captiontring } 
						
WFCatSearchByTextConstants.perCategoryExtraStartMap[ WFCatSearchByTextConstants.categories[ string.lower( WFCatSearchByTextConstants.category_titleKeyString ) ] ] =
						{ WFCatSearchByTextConstants.TitleString } 
						
WFCatSearchByTextConstants.perCategoryExtraStartMap[ WFCatSearchByTextConstants.categories[ string.lower( WFCatSearchByTextConstants.category_flagKeyString ) ] ] =
						{ WFCatSearchByTextConstants.FlagString } 
						
WFCatSearchByTextConstants.perCategoryExtraStartMap[ WFCatSearchByTextConstants.categories[ string.lower( WFCatSearchByTextConstants.category_assetTypeKeyString ) ] ] =
						{ WFCatSearchByTextConstants.AssetTypeString1,  WFCatSearchByTextConstants.AssetTypeString2,    
							WFCatSearchByTextConstants.AssetTypeImageString1, WFCatSearchByTextConstants.AssetTypeImageString2, 
							WFCatSearchByTextConstants.AssetTypeVideoString } 
						
WFCatSearchByTextConstants.perCategoryExtraStartMap[ WFCatSearchByTextConstants.categories[  string.lower( WFCatSearchByTextConstants.category_fileExtensionKeyString ) ] ] =
						{ WFCatSearchByTextConstants.ExtensionString1, WFCatSearchByTextConstants.ExtensionString2, 
						WFCatSearchByTextConstants.AssetTypeImageString1,  WFCatSearchByTextConstants.AssetTypeImageString2,
						WFCatSearchByTextConstants.AssetTypeVideoString } 
						
WFCatSearchByTextConstants.perCategoryExtraStartMap[ WFCatSearchByTextConstants.categories[ string.lower( WFCatSearchByTextConstants.category_fStopKeyString ) ] ] =
						{ WFCatSearchByTextConstants.FStopAdditionalStart1, WFCatSearchByTextConstants.FStopAdditionalStart2 } 
						
WFCatSearchByTextConstants.perCategoryExtraStartMap[ WFCatSearchByTextConstants.categories[ string.lower( WFCatSearchByTextConstants.category_focalLengthKeyString ) ] ] =
						{ WFCatSearchByTextConstants.FocalLengthAdditionalStart } 
						
WFCatSearchByTextConstants.perCategoryExtraStartMap[ WFCatSearchByTextConstants.categories[ string.lower( WFCatSearchByTextConstants.category_exposureKeyString ) ] ] =
						{ WFCatSearchByTextConstants.ExposureAdditionalStart1, WFCatSearchByTextConstants.ExposureAdditionalStart2 } 
						
WFCatSearchByTextConstants.perCategoryExtraStartMap[ WFCatSearchByTextConstants.categories[ string.lower( WFCatSearchByTextConstants.category_keywordsKeyString ) ] ] =
						{ WFCatSearchByTextConstants.KeywordStart } 
						
--------------------------------------------------------------------------------
WFCatSearchByTextConstants.perCategoryExtraWordUsageMap= {}

WFCatSearchByTextConstants.perCategoryExtraWordUsageMap[ WFCatSearchByTextConstants.categories
				[ WFCatSearchByTextConstants.category_cameraKeyString ] ] = true
						
--------------------------------------------------------------------------------	
-- for used enterd values, treat words independant of case
WFCatSearchByTextConstants.perCaetgoryWordCaseIgnore = {}
WFCatSearchByTextConstants.perCaetgoryWordCaseIgnore[ WFCatSearchByTextConstants.categories[ string.lower( WFCatSearchByTextConstants.category_captionKeyString ) ] ] =
						true
						
WFCatSearchByTextConstants.perCaetgoryWordCaseIgnore[ WFCatSearchByTextConstants.categories[ string.lower( WFCatSearchByTextConstants.category_titleKeyString ) ] ] =
						true
						
WFCatSearchByTextConstants.perCaetgoryWordCaseIgnore[ WFCatSearchByTextConstants.categories[ string.lower( WFCatSearchByTextConstants.category_locationKeyString ) ] ] =
						true
						
WFCatSearchByTextConstants.perCaetgoryWordCaseIgnore[ WFCatSearchByTextConstants.categories[ string.lower( WFCatSearchByTextConstants.category_keywordsKeyString ) ] ] =
						true
						
--==============================================================================

return WFCatSearchByTextConstants
