--[[----------------------------------------------------------------------------

WFCatSearchByTextDataProcessors.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2015 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remains the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and are protected by all applicable
intellectual property laws, including trade secret and copyright laws.
Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--
local WFCatSearchByTextDataProcessors = {}

	-- WF.core.string
local KSStringUtils = require 'KSStringUtils'
local WFOzContentTypes = require 'WFOzContentTypes'

-------------------------------------------------------------------------------
local noDataString = "NO DATA"
-------------------------------------------------------------------------------
local WFCatSearchByTextConstants = require 'WFCatSearchByTextConstants'
local perCategoryExtraStartMap = WFCatSearchByTextConstants.perCategoryExtraStartMap
local Months = WFCatSearchByTextConstants.Months
local perCategoryExtraWordUsageMap =  WFCatSearchByTextConstants.perCategoryExtraWordUsageMap
local extraWordSeparator = WFCatSearchByTextConstants.extraWordSeparator

-------------------------------------------------------------------------------
-- requirement on asset type is slighlty different.
-- it should be searchable with below (localized contnet )
	-- AssetType - [ WFCatSearchByTextConstants.AssetTypeString1 ]
	-- Type  -[ WFCatSearchByTextConstants.AssetTypeString2 ]
	-- Images  - [ WFCatSearchByTextConstants.AssetTypeImageString2 ] :  ( photos and its sub type )
	-- Photos - [ WFCatSearchByTextConstants.AssetTypeImageString1 ] :  ( photos and its sub type )
	-- Videos - [ WFCatSearchByTextConstants.AssetTypeVideoString  ] : videos and its sub type
	-- by non localized extension
	
local imageMimeTypeStartString = WFOzContentTypes.kImageContentTypeStartString -- non localized mime type starts
local videoMimeTypeStartString = WFOzContentTypes.kVideoContentTypeStartString -- non localized mime type starts

-------------------------------------------------------------------------------
-- we have to keep this in sync with WFCatSearchByTextConstants.category_assetTypeKeyString additionaal string in 
--WFCatSearchByTextConstants
local imageBasedAdditionalWords = {  WFCatSearchByTextConstants.AssetTypeString1, WFCatSearchByTextConstants.AssetTypeString2 , 
	 WFCatSearchByTextConstants.AssetTypeImageString1,  WFCatSearchByTextConstants.AssetTypeImageString2 }

local videoBasedAdditionalWords = {  WFCatSearchByTextConstants.AssetTypeString1, WFCatSearchByTextConstants.AssetTypeString2 , 
	 WFCatSearchByTextConstants.AssetTypeVideoString }

-------------------------------------------------------------------------------
local imageBasedFileExtensionAdditionalWords = {  WFCatSearchByTextConstants.ExtensionString1, WFCatSearchByTextConstants.ExtensionString2 , 
	 WFCatSearchByTextConstants.AssetTypeImageString1,  WFCatSearchByTextConstants.AssetTypeImageString2 }

local videoBasedFileExtensionAdditionalWords = {  WFCatSearchByTextConstants.ExtensionString1, WFCatSearchByTextConstants.ExtensionString2 , 
	 WFCatSearchByTextConstants.AssetTypeVideoString }

-------------------------------------------------------------------------------	 
local assetSubTypeAdditionalWords = {  WFCatSearchByTextConstants.AssetTypeString1, WFCatSearchByTextConstants.AssetTypeString2 }
	 
-------------------------------------------------------------------------------
local function round(num, idp)
  local mult = 10^(idp or 0)
  return math.floor(num * mult + 0.5) / mult
end
--------------------------------------------------------------------------------
local function addAdditionalSearchStringsIfNeeded( additioanalDataNeeded, baseContent, categoryKey, tableToInsert, additionalStarts )
	if baseContent then
		-- moving this from client side to here. As doing here we need to do this only once
		baseContent = KSStringUtils.trimWhitespace( baseContent )
		if tableToInsert == nil then
			tableToInsert = { baseContent }
		else
			table.insert( tableToInsert, baseContent )
		end
		
		if additioanalDataNeeded then
			if additionalStarts == nil then
				local localizedCategoryKey = WFCatSearchByTextConstants.categories[ categoryKey ]
				 additionalStarts = perCategoryExtraStartMap[ localizedCategoryKey ]
			end
			
			for _, startingWord in ipairs( additionalStarts  or {} ) do
				local additioanalString  = startingWord .. ( baseContent )
				table.insert( tableToInsert, additioanalString )
			end
		end
	end
	
	return tableToInsert or {}
end
--------------------------------------------------------------------------------
local function getMainData( values )
	return values and values[ 1 ]
end

--------------------------------------------------------------------------------
local function getAdditionalData( values )
	return values and values[ 2 ]
end

-------------------------------------------------------------------------------

local function extractValuesFromMap ( map )
	local values = {}
	for _, value in pairs ( map ) do
		table.insert( values, value )
	end
	return values
end


-------------------------------------------------------------------------------
local function maybeGetCaptureDate( captureDate )	
	if captureDate and not string.find( captureDate, "^0000" ) then
		local _, _, year, month, dayOfMonth = string.find( captureDate, "^([0-9][0-9][0-9][0-9])-([0-9][0-9])-([0-9][0-9])" )
		return year, month, dayOfMonth
	end
end

-------------------------------------------------------------------------------
function WFCatSearchByTextDataProcessors.addAllItems( values, addAditionalWords, categoryKey  )
	local processedData = {}
	local items = getMainData( values )
	for data in pairs( items or {} ) do
		addAdditionalSearchStringsIfNeeded( addAditionalWords,  data, categoryKey, processedData )
		--processedData[ #processedData + 1 ] = data
	end
	
	return processedData
end

-------------------------------------------------------------------------------
function WFCatSearchByTextDataProcessors.processTextDataDefault( values, addAditionalWords, categoryKey )
	-- we support _specialCharsTobeTreatedPartOfToken  "-!()&=\\" as part of words.
	-- any other special treatment needed, category need to add separate process methods
	local stringValue = getMainData( values )
	
	-- check whether string has any valid contetnt
	local temp = stringValue
	temp = string.replace( stringValue , " ", "")
	local nonEmptyLen = string.len( temp )

	if stringValue and string.len( stringValue ) > 0  and nonEmptyLen > 0 then
		return addAdditionalSearchStringsIfNeeded( addAditionalWords,  stringValue, categoryKey )
	end
end

-------------------------------------------------------------------------------
function WFCatSearchByTextDataProcessors.processRatings( inptValues, addAditionalWords, categoryKey )	
	local ratings = getMainData( inptValues )
	if ratings then
		local values = extractValuesFromMap( ratings )
		if values then 
			-- we expect only one user
			local userData = values[ 1 ]
			if userData then
				local ratingVal = userData.rating
				if ratingVal and ratingVal > 0 then
				
					local displayString = WFCatSearchByTextConstants.RatingStringSingular
					if ratingVal > 1 then
						displayString =  WFCatSearchByTextConstants.RatingStringPlural
					end
					
					local ratingContnetString = string.format("%s%s", ratingVal, displayString )
					return addAdditionalSearchStringsIfNeeded( addAditionalWords, ratingContnetString, categoryKey )
	
				end
			end
		end
	end
end

-------------------------------------------------------------------------------
function WFCatSearchByTextDataProcessors.processReviews( inputValues, addAditionalWords, categoryKey )	
		
	local reviews = getMainData( inputValues )
	-- if no value present, it will be termed as UnFlagged 
	local exactStatusString = WFCatSearchByTextConstants.UnFlaggedString
	
	-- if reviews is nil, we will pass the default as  
	-- WFCatSearchByTextConstants.UnFlaggedString
	if reviews and reviews ~=  WFCatSearchByTextConstants.UnFlaggedString then
		local values = extractValuesFromMap( reviews )
		if values then 
			-- we expect only one user till we have collaboration with multiple users.
			local userData = values[ 1 ]
		
			if userData then
				local flagValue = userData.flag

				if flagValue == WFCatSearchByTextConstants.pickValue then
					exactStatusString = WFCatSearchByTextConstants.PickedString
					
				elseif flagValue == WFCatSearchByTextConstants.rejectValue then
					exactStatusString = WFCatSearchByTextConstants.RejectedString 
				end

			end
		end
	end
	
	-- add exact string details if present
	if exactStatusString and string.len( exactStatusString ) > 0 then
		local content =  addAdditionalSearchStringsIfNeeded( addAditionalWords, exactStatusString, categoryKey )
		return content
	end
	
	return  {}
end


-------------------------------------------------------------------------------
function WFCatSearchByTextDataProcessors.processAssetTypes( values, addAditionalWords, categoryKey )	
	local assetMimeType = getMainData( values )
	if assetMimeType then
		local assetTypeLocalized  = WFOzContentTypes.getlocalizedStringForExtention( assetMimeType )
		
		if assetTypeLocalized then
			-- we cant use the addAditionalWords here, as some are applicable for all, some are applicable for 
			-- image and some for videos
			local additionalWords 
				
			if assetMimeType == WFOzContentTypes.kRawContentType then
				additionalWords = imageBasedAdditionalWords
			else
				if string.find( assetMimeType, imageMimeTypeStartString ) == 1 then
					additionalWords = imageBasedAdditionalWords
					
				elseif string.find( assetMimeType, videoMimeTypeStartString ) == 1 then
					additionalWords = videoBasedAdditionalWords
				end
			end
			
			return addAdditionalSearchStringsIfNeeded( addAditionalWords, assetTypeLocalized, categoryKey, nil, additionalWords )
		end
		
		return {}
	end
end

-------------------------------------------------------------------------------
function WFCatSearchByTextDataProcessors.processFileNames( values, addAditionalWords, categoryKey )	
	
	local fileName = getMainData( values )
	if fileName then
		--local extensionValue = fileName:match( "%.([^%.]+)$" ):lower()
		
		--https://jira.corp.adobe.com/browse/NMB-1786 no result was due to the null
		-- indexing exception
		local extensionValue = fileName:match( "%.([^%.]+)$" )
		extensionValue = extensionValue and extensionValue:lower() 
		
		local assetMimeType
		if extensionValue then
			assetMimeType = WFOzContentTypes.contentTypeForExtention( extensionValue )
		else
			extensionValue = WFCatSearchByTextConstants.NoExtension
		end
		 
		local additionalWords 
				
		if not assetMimeType then
			-- This should be a raw file type or we assume it as image
			additionalWords = imageBasedFileExtensionAdditionalWords
		else
			if string.find( assetMimeType, imageMimeTypeStartString ) == 1 then
				additionalWords = imageBasedFileExtensionAdditionalWords
				
			elseif string.find( assetMimeType, videoMimeTypeStartString ) == 1 then
				additionalWords = videoBasedFileExtensionAdditionalWords
			end
		end
		
		local extensionValueMod = string.format(".%s", extensionValue )
		return addAdditionalSearchStringsIfNeeded( addAditionalWords, extensionValueMod, categoryKey, nil,additionalWords )
	end
end

-------------------------------------------------------------------------------
function WFCatSearchByTextDataProcessors.processAssetSubTypes( subType, addAditionalWords, categoryKey )	
	-- we need to use customized additioanl words here.
	local content =  {}
	if subType then
		if subType == WFOzContentTypes.kVideoSubTypeString then
			addAdditionalSearchStringsIfNeeded( addAditionalWords, 
					WFCatSearchByTextConstants.AssetTypeVideoString, categoryKey, content, assetSubTypeAdditionalWords )
		else
			
			addAdditionalSearchStringsIfNeeded( addAditionalWords, 
					WFCatSearchByTextConstants.AssetTypeImageString1, categoryKey, content, assetSubTypeAdditionalWords )
					
			table.insert( content, WFCatSearchByTextConstants.AssetTypeImageString2 )
		end	
	end
	
	return content or {}
end


-------------------------------------------------------------------------------
function WFCatSearchByTextDataProcessors.processISOValues( values, addAditionalWords, categoryKey )	
	local isoValue = getMainData( values )

	if isoValue and string.len( isoValue ) > 0  then
		local isoVal = string.format( "%s%s", WFCatSearchByTextConstants.ISOString, isoValue )
		return addAdditionalSearchStringsIfNeeded( addAditionalWords,  isoVal, categoryKey )
	end
	
end

-------------------------------------------------------------------------------
function WFCatSearchByTextDataProcessors.processFStop( values, addAditionalWords, categoryKey )	
	local fNumber = getMainData( values )

	--we expect a table here
	if fNumber and #fNumber == 2 then
		local val1 = fNumber[ 1 ]
		local val2 = fNumber[ 2 ]
		if val2 ~= 0 then
			local fnumber = val1 / val2
			local fnumberStr = string.format( "%s%s", WFCatSearchByTextConstants.FStopString, fnumber)
			return addAdditionalSearchStringsIfNeeded( addAditionalWords,  fnumberStr, categoryKey )
		end
	end
end
-----------------------------------------------------------------------
function WFCatSearchByTextDataProcessors.processFocalLength( values, addAditionalWords, categoryKey )	
	local focalLength = getMainData( values )

	--we expect a table here
	if focalLength and #focalLength == 2 then
		local val1 = focalLength[ 1 ]
		local val2 = focalLength[ 2 ]
		if val2 ~= 0 then
			local fL = val1 / val2
			fL = round( fL, 1 )
			local focalLengthString = string.format( "%s%s", fL, WFCatSearchByTextConstants.FocalLengthString )
			return addAdditionalSearchStringsIfNeeded( addAditionalWords,  focalLengthString, categoryKey )
		end
	end
end


-------------------------------------------------------------------------------
function WFCatSearchByTextDataProcessors.processExposure( values, addAditionalWords, categoryKey )	
	local exposureVal = getMainData( values )

	--we expect a table here
	if exposureVal and #exposureVal == 2 then
		local val1 = exposureVal[ 1 ]
		local val2 = exposureVal[ 2 ]
		
		if val2 ~= 0 then
			local exposureString = string.format( "%s%s%s%s", val1, WFCatSearchByTextConstants.ExposureSeparator, val2, WFCatSearchByTextConstants.ExposureUnitString )
			return addAdditionalSearchStringsIfNeeded( addAditionalWords,  exposureString, categoryKey )
		end
	end
end



-------------------------------------------------------------------------------
function WFCatSearchByTextDataProcessors.processCameraModel( cameraData , addAditionalWords, categoryKey )		
	-- here in camera data we will get model and make
	-- we need to add the make information to model, if model is 
	-- not starting with the first token of  the make
	-- and we need to make the core part of the make also searchable
	-- e.g if iPhone 5C is model and Apple is the manufacturer, we should be
	-- able to search with Apple and iPhone
	
	-- if Nikon D800 is the model name and Nikon corporation is the 
	-- manufacturer, it should be searchable with Nikon and with D800
	local model  = getMainData( cameraData )
	local completeModelString = model
	if not model then
		return {}
	end

	local doesModelStartWithManufacturerName = false
	local manufacturerFirstToken
	
	local additionalData = getAdditionalData( cameraData )
	if additionalData then 
		-- here we know what we requested only one data as per the definition of 
		-- additional data we added to WFCatSearchByTextFields, 
		local manufacturer = additionalData[ 1 ]
		-- get the first token
		if manufacturer and manufacturer ~= noDataString then
			local spacePos = string.find(manufacturer, " ")
			if not spacePos then
				manufacturerFirstToken = manufacturer
			else
				manufacturerFirstToken = string.sub( manufacturer, 1,  spacePos-1 )
			end
			
			if string.find( model, manufacturerFirstToken )  ~= 1 then
				completeModelString = string.format("%s %s", manufacturerFirstToken, model )
			else
				-- here we keep the name as it is for the model cpmplete name
				-- e.g Nikon D70
				doesModelStartWithManufacturerName = true
			end
		end
	end
	
	local content =  addAdditionalSearchStringsIfNeeded( addAditionalWords, completeModelString, categoryKey )
	
	if addAditionalWords then
		-- the above will make it searchable with manufactors name, but if we want to search only with the 
		if content and  perCategoryExtraWordUsageMap[ categoryKey ] then
			local modelNameShort = model
			
			-- get only D90 from Nikon D90 , as anyway Nikon D90 is searchable separately
			if doesModelStartWithManufacturerName then
				if manufacturerFirstToken then
					modelNameShort = string.replace( model, manufacturerFirstToken, "")
					modelNameShort = string.replace( modelNameShort, " ", "")
				end
			end
			
			local extrString = string.format("%s%s%s",modelNameShort,extraWordSeparator,completeModelString)
			table.insert( content, extrString )
		
		end
	end
	
	return content or {}
end

-------------------------------------------------------------------------------
function WFCatSearchByTextDataProcessors.processCaptureDate( values, addAditionalWords, categoryKey )		
	-- we need addAditionalWords only to make the hinting work. For search we should not use those
	-- as it will be present in all assets 
	-- assets without a true captureDate will have a captureDate of '0000-00-00T00:00:00'
	-- we want to ignore these
	local captureDate = getMainData( values )
	local _, _, dayOfMonth = maybeGetCaptureDate( captureDate )

	if dayOfMonth  then		
		return addAdditionalSearchStringsIfNeeded( addAditionalWords, dayOfMonth, categoryKey )
	end	
end



-------------------------------------------------------------------------------
function WFCatSearchByTextDataProcessors.processCaptureCompleteDate( values, addAditionalWords, categoryKey )		
	-- we need addAditionalWords only to make the hinting work. For search we should not use those
	-- as it will be present in all assets 
	-- assets without a true captureDate will have a captureDate of '0000-00-00T00:00:00'
	-- we want to ignore these
	local captureDate = getMainData( values )
	local year, month, dayOfMonth = maybeGetCaptureDate( captureDate )

	if dayOfMonth and month and year  then		
		local monthString = string.format ( "%02d", month ) 
		local dateString = string.format ( "%02d", dayOfMonth ) 
		local completeDateString = string.format("%s/%s/%s",monthString,dateString,year)
		return  addAdditionalSearchStringsIfNeeded( addAditionalWords, completeDateString, categoryKey )
	end	
end

-------------------------------------------------------------------------------
function WFCatSearchByTextDataProcessors.processCaptureMonth( values , addAditionalWords, categoryKey )	
	-- we need addAditionalWords only to make the hinting work. For search we should not use those
	-- as it will be present in all assets 
	-- assets without a true captureDate will have a captureDate of '0000-00-00T00:00:00'
	-- we want to ignore these
	local captureDate = getMainData( values )
	local _, month, _ = maybeGetCaptureDate( captureDate )

	if month  then
		month = tonumber( month )
		local monthVal = Months[ month ]
		if monthVal then
			return addAdditionalSearchStringsIfNeeded( addAditionalWords, monthVal, categoryKey )
		end
	end
end

-------------------------------------------------------------------------------
function WFCatSearchByTextDataProcessors.processCaptureYear( values , addAditionalWords, categoryKey)	
	-- we need addAditionalWords only to make the hinting work. For search we should not use those
	-- as it will be present in all assets 
	-- assets without a true captureDate will have a captureDate of '0000-00-00T00:00:00'
	-- we want to ignore these
	local captureDate = getMainData( values )
	local year, _, _ = maybeGetCaptureDate( captureDate )
	if year  then
		return addAdditionalSearchStringsIfNeeded( addAditionalWords, year, categoryKey )
	end
end



-- this is a relevance process method it expects to get values and it is expected to
-- do  with its main and additional data
-------------------------------------------------------------------------------
function WFCatSearchByTextDataProcessors.processTagsRelevance( values )	
	local processedData = {}
	local tags = getMainData( values )
	for tag, relevance in pairs( tags or {} ) do
		if relevance == true or ( type( relevance ) == 'number' and relevance > 50 ) then
			processedData[ tag ] = relevance
		end
	end
	return processedData
end

-------------------------------------------------------------------------------
function WFCatSearchByTextDataProcessors.processTags( values )	
	local processedData = {}
	local tags = getMainData( values )
	for tag, relevance in pairs( tags or {} ) do
		if relevance == true or ( type( relevance ) == 'number' and relevance > 50 ) then
			processedData[ #processedData + 1 ] = tag
		end
	end
	return processedData
end

-------------------------------------------------------------------------------

return WFCatSearchByTextDataProcessors
