--[[----------------------------------------------------------------------------

WFCatSearchByTextIndexedData.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2015 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remains the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and are protected by all applicable
intellectual property laws, including trade secret and copyright laws.
Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--
local WFCatSearchByTextDataProcessors = require "WFCatSearchByTextDataProcessors"
local WFCatSearchByTextConstants = require "WFCatSearchByTextConstants"

local WFCatSearchByTextIndexedData = {}


----------------------------------------------------------------------------
local function validateLastEntry( destinationTable, requriedFilelds )
	local lastEntry = destinationTable[ #destinationTable ]
	local identifier = table.concat( lastEntry.mainDocFieldPath, ":" )
	
	local processingDetails = lastEntry and lastEntry.processingDetails
	assert ( processingDetails, "processingDetails is expected for " .. identifier )
	
	for i, details in ipairs( processingDetails ) do	
		for _,requriedField in ipairs( requriedFilelds ) do
			local expectedVal = details[ requriedField ]
			if not expectedVal then
				local x = 0
			end
			assert( expectedVal, requriedField .. " is expected for " .. i .. "th data in "  .. identifier )
		end
	end
end


----------------------------------------------------------------------------
function WFCatSearchByTextIndexedData.initializeSearchIndexedFieldDetails( destinationTable )

--[[ \brief : moving all fields as a table to centre place
	having this way will enable us to take the details from the outside the class also
	i.e the application/user can dynamicaly  add the requirement
	--]]
	
	-- information provided for each of the data is where to get the data from the doc  (docFieldPath) and 
	-- what needs to be done with the data. (processMethod)
	-- if processMethod is not specified, data obtained can be directly added to the searchable text data.
	-- else the data read from the doc needs to be sent to the process method and use the return value from that 
	-- method.
	-- processMethods are expected to return the value as a table (array) eventhough only one value needs to be returned.

	table.insert( destinationTable, 
					{ 
						mainDocFieldPath = { 'xmp', 'iptcCore', 'Location' }, 
						processingDetails =
						{
							{
								category = WFCatSearchByTextConstants.category_locationKeyString,
								textProcessMethod = WFCatSearchByTextDataProcessors.processTextDataDefault, 
								defaultRelevance = 100
							}
						},
					} 
				)
	validateLastEntry( destinationTable, { "category", "textProcessMethod" } )
	
	table.insert( destinationTable, 
					{ 
						mainDocFieldPath = { 'xmp', 'photoshop', 'State' }, 
						processingDetails =
						{
							{
								category = WFCatSearchByTextConstants.category_locationKeyString,
								textProcessMethod = WFCatSearchByTextDataProcessors.processTextDataDefault, 
								defaultRelevance = 100
							}
						},
					}  
				)
	validateLastEntry( destinationTable, { "category", "textProcessMethod" } )
	
	table.insert( destinationTable, 
					{ 
						mainDocFieldPath = { 'xmp', 'photoshop', 'Country' }, 
						processingDetails =
						{
							{
								category = WFCatSearchByTextConstants.category_locationKeyString,
								textProcessMethod = WFCatSearchByTextDataProcessors.processTextDataDefault, 
								defaultRelevance = 100
							}
						}
					}  
				) 
	validateLastEntry( destinationTable, { "category", "textProcessMethod" } )
	
	table.insert( destinationTable, 
					{ 
						mainDocFieldPath = { 'xmp', 'photoshop', 'City' }, 
						processingDetails =
						{
							{
								category = WFCatSearchByTextConstants.category_locationKeyString,
								textProcessMethod = WFCatSearchByTextDataProcessors.processTextDataDefault, 
								defaultRelevance = 100
							}
						}
					}  
				) 
	validateLastEntry( destinationTable, { "category", "textProcessMethod" } )
	
	table.insert( destinationTable, 
					{ 
						mainDocFieldPath = { 'location', 'city' }, 
						processingDetails =
						{
							{
								category = WFCatSearchByTextConstants.category_locationKeyString,
								textProcessMethod = WFCatSearchByTextDataProcessors.processTextDataDefault, 
								defaultRelevance = 100
							}
						}
					}  
				) 
	validateLastEntry( destinationTable, { "category", "textProcessMethod" } )
	
	table.insert( destinationTable, 
					{ 
						mainDocFieldPath = { 'location', 'state' }, 
						processingDetails =
						{
							{
								category = WFCatSearchByTextConstants.category_locationKeyString,
								textProcessMethod = WFCatSearchByTextDataProcessors.processTextDataDefault, 
								defaultRelevance = 100
							}
						}
					}  
				) 
	validateLastEntry( destinationTable, { "category", "textProcessMethod" } )
	
	table.insert( destinationTable, 
					{ 
						mainDocFieldPath = { 'location', 'country' }, 
						processingDetails =
						{
							{
								category = WFCatSearchByTextConstants.category_locationKeyString,
								textProcessMethod = WFCatSearchByTextDataProcessors.processTextDataDefault, 
								defaultRelevance = 100
							}
						}
					}  
				) 
	validateLastEntry( destinationTable, { "category", "textProcessMethod" } )
	
	table.insert( destinationTable, 
					{ 
						mainDocFieldPath = { 'xmp', 'dc', 'subject' }, 
						processingDetails =
						{
							{
								category = WFCatSearchByTextConstants.category_keywordsKeyString,
								textProcessMethod = WFCatSearchByTextDataProcessors.addAllItems, 
								defaultRelevance = 100
							}
						}
					} 
				)
	validateLastEntry( destinationTable, { "category", "textProcessMethod" } )
	
	table.insert( destinationTable, 
					{ 
						mainDocFieldPath = { 'autoTags', 'tags' }, 
						processingDetails =
						{
							{
								category = WFCatSearchByTextConstants.category_autoTagsKeyString,
								textProcessMethod = WFCatSearchByTextDataProcessors.processTags, 
								relevancyProcessMethod = WFCatSearchByTextDataProcessors.processTagsRelevance, 
								defaultRelevance = 50
							}
						}
					} 
				)
	validateLastEntry( destinationTable, { "category", "textProcessMethod", "relevancyProcessMethod" } )
	
	table.insert( destinationTable, 
					{ 
						mainDocFieldPath = { 'xmp', 'dc', 'title' }, 
						processingDetails =
						{
							{
								category = WFCatSearchByTextConstants.category_titleKeyString,
								textProcessMethod = WFCatSearchByTextDataProcessors.processTextDataDefault, 
								defaultRelevance = 100
							}
						} 
					}
				)
	validateLastEntry( destinationTable, { "category", "textProcessMethod"  } )
	
	table.insert( destinationTable, 
					{ 
						mainDocFieldPath = { 'xmp', 'dc', 'description' }, 
						processingDetails =
						{
							{
								category = WFCatSearchByTextConstants.category_captionKeyString,
								textProcessMethod = WFCatSearchByTextDataProcessors.processTextDataDefault, 
								defaultRelevance = 100
							}
						}
					} 
				)
	validateLastEntry( destinationTable, { "category", "textProcessMethod"  } )
	
	table.insert( destinationTable, 
					{ 
						mainDocFieldPath = { 'xmp', 'tiff', 'Model' }, 
						processingDetails =
						{
							{
								additionalData = { { 'xmp', 'tiff', 'Make' } }, 
								category = WFCatSearchByTextConstants.category_cameraKeyString,
								textProcessMethod = WFCatSearchByTextDataProcessors.processCameraModel, 
								defaultRelevance = 100
							}
						} 
					}
				)
	validateLastEntry( destinationTable, { "category", "textProcessMethod"  } )
	
	table.insert( destinationTable, 
					{ 
						mainDocFieldPath = { 'xmp', 'aux', 'Lens' },
						processingDetails =
						{ 
							{
								category = WFCatSearchByTextConstants.category_lensKeyString,
								textProcessMethod = WFCatSearchByTextDataProcessors.processTextDataDefault, 
								defaultRelevance = 100
							}
						}
					}
				)
	validateLastEntry( destinationTable, { "category", "textProcessMethod"  } )
	
	table.insert( destinationTable, 
					{ 
						mainDocFieldPath = { 'captureDate' }, 
						processingDetails =
						{
							{
								category = WFCatSearchByTextConstants.category_dateKeyString,
								textProcessMethod = WFCatSearchByTextDataProcessors.processCaptureDate, 
								defaultRelevance = 100
							},
							{
								category = WFCatSearchByTextConstants.category_completeDateKeyString,
								textProcessMethod = WFCatSearchByTextDataProcessors.processCaptureCompleteDate, 
								defaultRelevance = 100
							},
							{ 
								category = WFCatSearchByTextConstants.category_monthKeyString,
								textProcessMethod = WFCatSearchByTextDataProcessors.processCaptureMonth, 
								defaultRelevance = 100
							},
							{ 
								category = WFCatSearchByTextConstants.category_yearKeyString,
								textProcessMethod = WFCatSearchByTextDataProcessors.processCaptureYear, 
								defaultRelevance = 100
							},
						}
					
					} 
				)
	validateLastEntry( destinationTable, { "category", "textProcessMethod"  } )

	
	table.insert( destinationTable, 
					{ 
						mainDocFieldPath = { 'ratings' },
						processingDetails =
						{  
							{
								category = WFCatSearchByTextConstants.category_ratingKeyString,
								textProcessMethod = WFCatSearchByTextDataProcessors.processRatings, 
								defaultRelevance = 100
							}
						} 
					}
				)
	validateLastEntry( destinationTable, { "category", "textProcessMethod"  } )
	
	table.insert( destinationTable, 
					{ 
						mainDocFieldPath = { 'reviews' },
						processingDetails =
						{   
							{
								category = WFCatSearchByTextConstants.category_flagKeyString,
								textProcessMethod = WFCatSearchByTextDataProcessors.processReviews, 
								defaultRelevance = 100,
								defaultValue = WFCatSearchByTextConstants.UnFlaggedString
							}
						} 
					}
				)
	validateLastEntry( destinationTable, { "category", "textProcessMethod", "defaultValue"  } )
	
	table.insert( destinationTable, 
					{ 
						mainDocFieldPath = { 'importSource', 'contentType' }, 
						processingDetails =
						{   
							{
								category = WFCatSearchByTextConstants.category_assetTypeKeyString,
								textProcessMethod = WFCatSearchByTextDataProcessors.processAssetTypes, 
								defaultRelevance = 100
							}
						} 
					}
				)
	validateLastEntry( destinationTable, { "category", "textProcessMethod"  } )
	
	table.insert( destinationTable, 
					{ 
						mainDocFieldPath = { 'importSource', 'fileName' }, 
						processingDetails =
						{   
							{
								category = WFCatSearchByTextConstants.category_fileExtensionKeyString,
								textProcessMethod = WFCatSearchByTextDataProcessors.processFileNames, 
								defaultRelevance = 100
							}
						} 
					}
				)
	validateLastEntry( destinationTable, { "category", "textProcessMethod"  } )
	
	table.insert( destinationTable, 
					{ 
						mainDocFieldPath = { 'subtype' }, 
						processingDetails =
						{    
							{
								category = WFCatSearchByTextConstants.category_assetTypeKeyString,
								textProcessMethod = WFCatSearchByTextDataProcessors.processAssetSubTypes, 
								defaultRelevance = 100
							}
						}
					}
				)
	validateLastEntry( destinationTable, { "category", "textProcessMethod"  } )
	
	table.insert( destinationTable, 
					{ 
						mainDocFieldPath = { 'xmp', 'exif', 'ISOSpeedRatings' },
						processingDetails =
						{    
							{
								category = WFCatSearchByTextConstants.category_isoKeyString,
								textProcessMethod = WFCatSearchByTextDataProcessors.processISOValues, 
								defaultRelevance = 100
							}
						}
					}
				)
	validateLastEntry( destinationTable, { "category", "textProcessMethod"  } )
	
	table.insert( destinationTable, 
					{ 
						mainDocFieldPath = { 'xmp', 'exif', 'FNumber' },
						processingDetails =
						{    
							{
								category = WFCatSearchByTextConstants.category_fStopKeyString,
								textProcessMethod = WFCatSearchByTextDataProcessors.processFStop, 
								defaultRelevance = 100
							}
						}
					}
				)
	validateLastEntry( destinationTable, { "category", "textProcessMethod"  } )
	
	table.insert( destinationTable, 
					{ 
						mainDocFieldPath = { 'xmp', 'exif', 'FocalLength' },
						processingDetails =
						{    
							{
								category = WFCatSearchByTextConstants.category_focalLengthKeyString,
								textProcessMethod = WFCatSearchByTextDataProcessors.processFocalLength, 
								defaultRelevance = 100
							}
						}
					}
				)
	validateLastEntry( destinationTable, { "category", "textProcessMethod"  } )
	
	table.insert( destinationTable, 
					{ 
						mainDocFieldPath = { 'xmp', 'exif', 'ExposureTime' },
						processingDetails =
						{    
							{
								category = WFCatSearchByTextConstants.category_exposureKeyString,
								textProcessMethod = WFCatSearchByTextDataProcessors.processExposure, 
								defaultRelevance = 100
							}
						}
					}
				)
	validateLastEntry( destinationTable, { "category", "textProcessMethod"  } )
	
	
	
end


return WFCatSearchByTextIndexedData
