--[[----------------------------------------------------------------------------

WFCatSortByRelevance.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2015 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remains the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and are protected by all applicable
intellectual property laws, including trade secret and copyright laws.
Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

--==============================================================================

--------------------------------------------------------------------------------

--local keywordSeparator = ":::"
local WFCatSortByRelevance = {}
local KSStringUtils = require 'KSStringUtils' 
local relevancySumFieldPos = 4
local idPos = 2
local relevanceResultIdentifier = 'RELEVANCE'
local WFCatSearchByTextConstants = require 'WFCatSearchByTextConstants'

--------------------------------------------------------------------------------

function WFCatSortByRelevance.executeSort( wfCatalog, assetIds, order, sortFieldInfo )

	local tableName = WFCatSearchByTextConstants.getTableName( 'text_search_with_relevance' )
	
	local terms = sortFieldInfo.value
	assert( terms, "no sort value. this sorting should not have been called")

	local  whereClause =  ""
	
	if terms then
		local whereConditionBase = string.format( "%s.search_text ==  ",tableName ) 
		whereClause = ' WHERE '
			
		for i , term in ipairs ( terms ) do
			term = KSStringUtils.trim( term )
			--https://jira.corp.adobe.com/browse/NMB-1518
			term = string.replace( term, "'", "''" )
			if i > 1 then
				whereClause = whereClause .. " OR " 
			end
			whereClause = whereClause .. whereConditionBase .. '\''.. term .. '\''
		end
	end
	
	if order ~= "ASC" then
		order = "DESC"
	end
	
	local contentString = string.format( 
			[[SELECT  '%s',  %s.docId, %s.search_text, SUM(%s.relevance) 
				 FROM  TEMP_ids  LEFT JOIN  %s  ON   %s.docId = TEMP_ids.docId %s 
				 GROUP BY %s.docId 
				 ORDER BY %s %s ]], 
			 relevanceResultIdentifier, 
			 tableName, tableName, tableName, tableName, tableName,
			 whereClause, 
			 tableName,
			 relevancySumFieldPos,
			 order )
	
	return wfCatalog:indexStore():query( {
			sql = contentString,
			tempIds = assetIds,
			calm = 1,
		} )
	
end



--------------------------------------------------------------------------------

function WFCatSortByRelevance.isSortingNeeded( sortInfo )
	-- based on the data , sort handlers can actually decide whether sorting is needed
	-- e.g relevancy sort order is used, but no data specified. So we don't need to execute 
	-- sort in that case
	return sortInfo.sortFieldInfo and sortInfo.sortFieldInfo.value   or false
end

--------------------------------------------------------------------------------
function WFCatSortByRelevance.isMultipleDataExpectedFromAsset(  )
	return true
end

--------------------------------------------------------------------------------
function WFCatSortByRelevance.getAdditionalDataForAsset( resultData )
	if resultData [1] == relevanceResultIdentifier then
		return relevanceResultIdentifier .. " : " .. tostring( resultData[ relevancySumFieldPos ] )
	end
end

--------------------------------------------------------------------------------
function WFCatSortByRelevance.getAssetIdPos(  )
	return idPos
end


--==============================================================================

return WFCatSortByRelevance
