--[[----------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2016 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

	-- RxLua
local Rx = require 'Rx'

	-- WF.core.utils
local WFCloseSupport = require 'WFCloseSupport'
local WFArguments = require 'WFArguments'

	-- WF.oz_client.managed_catalog
local WFCatIdleDownloader = require 'WFCatIdleDownloader'
local WFCatSyncClientBase = require 'WFCatSyncClientBase'

--------------------------------------------------------------------------------

local check = assert( WFArguments.check )

--==============================================================================

-- @mark WFCatSyncClientImp namespace

--------------------------------------------------------------------------------

local syncClientImpMethods = {

	_syncClientBase = false,
		-- (WFCatSyncClientBase)
		
	WFCloseSupport.addCloseSupportMethods,
	
}

--------------------------------------------------------------------------------

local function albumAssetsForFilteredAssetsQueryFn( catalog, assetIds )
	
	return catalog:indexStore():queryOnce {
				sql = [[SELECT album_asset_v2.docId, album_asset_v2.assetId FROM TEMP_ids JOIN album_asset_v2 ON 
							TEMP_ids.docId = album_asset_v2.assetId]],
				tempIds = assetIds,
			}
end

--------------------------------------------------------------------------------

function syncClientImpMethods:__init( options )

	check( 'WFCatSyncClientImp', 'table' )
	
	options.albumAssetsForFilteredAssetsQueryFn = albumAssetsForFilteredAssetsQueryFn
	
	local syncClientBase = WFCatSyncClientBase( options )
	
	self:setOwned {
		_syncClientBase = syncClientBase
	}
	
	-- Setup custom asset sync handlers
	
	local catalogPuller = syncClientBase:getCatalogPuller()
	
	catalogPuller:addAssetTypeHandler( 'asset', WFCatIdleDownloader.addWaitingForProxyAnnotation )
	
end

--------------------------------------------------------------------------------

function syncClientImpMethods:close()

	check( 'WFCatSyncClientImp' )

end

--------------------------------------------------------------------------------

function syncClientImpMethods:startSync( ... )

	check( 'WFCatSyncClientImp' )
	
	return self._syncClientBase:startSync( ... )
	
end

------------------------------------------------------------------------

function syncClientImpMethods:stopSync()

	check( 'WFCatSyncClientImp' )
	
	return self._syncClientBase:stopSync()
	
end

------------------------------------------------------------------------

function syncClientImpMethods:startSyncInResidualMode()

	check( 'WFCatSyncClientImp' )
	
	return self._syncClientBase:startSyncInResidualMode()
	
end

------------------------------------------------------------------------

function syncClientImpMethods:stopSyncInResidualMode()

	check( 'WFCatSyncClientImp' )
	
	return self._syncClientBase:stopSyncInResidualMode()

end

------------------------------------------------------------------------

function syncClientImpMethods:startMetadataSyncInFreemiumMode()

	check( 'WFCatSyncClientImp' )
	
	return self._syncClientBase:startMetadataSyncInFreemiumMode()
	
end

------------------------------------------------------------------------

function syncClientImpMethods:stopMetadataSyncInFreemiumMode()

	check( 'WFCatSyncClientImp' )
	
	return self._syncClientBase:stopMetadataSyncInFreemiumMode()

end

--------------------------------------------------------------------------------

function syncClientImpMethods:pausePusherReturningCounter()

	check( 'WFCatSyncClientImp' )

	return self._syncClientBase:pausePusherReturningCounter()
	
end

------------------------------------------------------------------------

function syncClientImpMethods:resumePusher()

	check( 'WFCatSyncClientImp' )

	return self._syncClientBase:resumePusher()

end

--------------------------------------------------------------------------------

-- For Lr CC debug panel.

function syncClientImpMethods:_DEBUG_startMetadataPusher()

	check( 'WFCatSyncClientImp' )
	
	return self._syncClientBase:_DEBUG_startMetadataPusher()
	
end

------------------------------------------------------------------------

-- For Lr CC debug panel.

function syncClientImpMethods:_DEBUG_stopMetadataPusher()

	check( 'WFCatSyncClientImp' )
	
	return self._syncClientBase:_DEBUG_stopMetadataPusher()

end

--------------------------------------------------------------------------------

function syncClientImpMethods:setUserIsOverQuota( ... )

	check( 'WFCatSyncClientImp' )
	
	return self._syncClientBase:setUserIsOverQuota( ... )
	
end

--------------------------------------------------------------------------------

function syncClientImpMethods:setUserIsExpired( ... )

	check( 'WFCatSyncClientImp' )
	
	return self._syncClientBase:setUserIsExpired( ... )
	
end

--------------------------------------------------------------------------------

function syncClientImpMethods:getLatestDownloadedAssetIdsStream()

	check( 'WFCatSyncClientImp' )
	
	return self._syncClientBase:getLatestDownloadedAssetIdsStream()
	
end

--------------------------------------------------------------------------------

function syncClientImpMethods:isOzCatalogDeleted()

	check( 'WFCatSyncClientImp' )
	
	return self._syncClientBase:isOzCatalogDeleted()
	
end

--------------------------------------------------------------------------------

function syncClientImpMethods:isUploadingMetadata()

	check( 'WFCatSyncClientImp' )
	
	return self._syncClientBase:isUploadingMetadata()
	
end

--------------------------------------------------------------------------------

function syncClientImpMethods:isDownloadingMetadata()

	check( 'WFCatSyncClientImp' )
	
	return self._syncClientBase:isDownloadingMetadata()
	
end

--------------------------------------------------------------------------------

function syncClientImpMethods:getRequiredUpdateStream()

	check( 'WFCatSyncClientImp' )
	
	return self._syncClientBase:getRequiredUpdateStream()
	
end

--------------------------------------------------------------------------------

return Rx.Class( 'WFCatSyncClientImp', syncClientImpMethods )
