--[[----------------------------------------------------------------------------

WFCrInfo.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2018 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

------------------------------------------------------------------------------]]

	-- WF.acr
local WFCrInfoImp = require 'WFCrInfoImp'

	-- WF.core
local WFArguments = require 'WFArguments'
local WFConfiguration = require 'WFConfiguration'
local WFLogger = require 'WFLogger'
local WFPlatformInfo = require 'WFPlatformInfo'

--------------------------------------------------------------------------------

local DISABLE_HEIC_SUPPORT = WFConfiguration.Shared.disableHeicSupport or false

assert( WFPlatformInfo.hasACR, "ACR integration not enabled" )

local check = WFArguments.check
local logger = WFLogger 'WFCrInfo'


--============================================================================--

-- @mark WFCrInfo namespace

local WFCrInfo = {
	_heicEntitlement = nil,
	_heicSupported = nil,
}

--------------------------------------------------------------------------------

--[[!

	\brief Get the Adobe Camera Raw version.
		
	\returns major, minor, dot, version
		
]]--

function WFCrInfo.getACRVersion()

	return WFCrInfoImp.getACRVersion()

end

--------------------------------------------------------------------------------

--[[!

	\brief Set whether or not HEIC files should be supported. Can be decided by the
		client based on a user's license
		
]]--

function WFCrInfo.setHeicEntitlement( heicEntitlement )
	check( 'boolean' )
	
	logger:infof( 'Setting heicEntitlement: %s', heicEntitlement )
	
	WFCrInfo._heicEntitlement = heicEntitlement
end

--------------------------------------------------------------------------------

--[[!

	\brief Check if ACR supports decoding HEIC files
		
	\returns boolean
		
]]--

function WFCrInfo.supportsHeic()

	if WFCrInfo._heicSupported == nil then
		WFCrInfo._heicSupported = WFCrInfoImp.haveH265Decoder() and not DISABLE_HEIC_SUPPORT
	end
	
	if WFCrInfo._heicEntitlement == nil then
		logger:errorf( 'Not yet obtained the entitlement' )
	end
	
	return ( WFPlatformInfo.isMac or WFCrInfo._heicEntitlement ) and WFCrInfo._heicSupported
		-- Using the codec is free on Mac, so no need to check entitlement
end

--------------------------------------------------------------------------------

function WFCrInfo.getUserPresetDir()

	return WFCrInfoImp.getUserPresetDir()

end

--------------------------------------------------------------------------------

function WFCrInfo.setUserPresetDir( path )

	return WFCrInfoImp.setUserPresetDir( path )

end

--------------------------------------------------------------------------------

function WFCrInfo.setUserCameraProfilesDir( path )

	return WFCrInfoImp.setUserCameraProfilesDir( path )

end

--------------------------------------------------------------------------------

function WFCrInfo.setUserPreferencesDir( path )

	return WFCrInfoImp.setUserPreferencesDir( path )

end

--------------------------------------------------------------------------------

return WFCrInfo
