--[[----------------------------------------------------------------------------

WFOzDocStorePath.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2012 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

------------------------------------------------------------------------------]]

	-- WF.core.os
local KSPathUtils = require 'KSPathUtils'
local WFFileUtils = require 'WFFileUtils'

	-- WF.core.utils
local WFArguments = require 'WFArguments'


--------------------------------------------------------------------------------

local check = assert( WFArguments.check )


--============================================================================--
--[[!
	\class WFOzDocStorePath
	\brief This namespace provides shared code for calculating
		where on disk a local Oz catalog should be stored.
]]--

-- @mark WFOzDocStorePath namespace

local WFOzDocStorePath = {}

--------------------------------------------------------------------------------
--[[!
	\brief Finds the directory that should be used for a catalog, creating it
	if it does not already exist.
	
	\param catalogId (string) The catalog's unique ID.
	
	\return (string) The full path to the directory on the local disk.
--]]

function WFOzDocStorePath.dirPathForCatalogId( catalogId )

	check( 'string' )
	
	if catalogId:match '^:memory:' then
		return catalogId
	end

	if require 'WFCapabilities'.getCapability( 'WFDocStorePathHack' ) then
		
		local cachesDir = KSPathUtils.getStandardFilePath 'caches'
		local adobeCachesDir = KSPathUtils.child( cachesDir, 'Adobe' )
		local lrCachesDir = KSPathUtils.child( adobeCachesDir, 'Lightroom' )
		local catalogIdDir = KSPathUtils.child( lrCachesDir, catalogId )
		local ozCatDir = KSPathUtils.child( catalogIdDir, 'Catalog.ozcat' )
		
		return ozCatDir

	end
	
	local documentsDir = KSPathUtils.getStandardFilePath 'carouselDocuments'
	
	local catalogIdDir = KSPathUtils.child( documentsDir, catalogId )
	
	local ozCatDir = KSPathUtils.child( catalogIdDir, 'Catalog.ozcat' )

	if WFFileUtils.exists( ozCatDir ) == 'file' then
		debug.print( "Catalog directory " .. ozCatDir .. " exists... as a file???  Deleting" )
		WFFileUtils.delete( ozCatDir )
	end

	WFFileUtils.mkdir( ozCatDir )
	
	return ozCatDir

end

--------------------------------------------------------------------------------

return WFOzDocStorePath
