--[[----------------------------------------------------------------------------

WFSQLiteCorruptionLogger.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2018 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

------------------------------------------------------------------------------]]
	
local WFArguments = require "WFArguments"

	-- WF.core.lua.threading
local WFWorkerThread = require 'WFWorkerThread'

--------------------------------------------------------------------------------

local WFRxRunUntilDone = require "WFRxRunUntilDone"

--------------------------------------------------------------------------------

local logger = require "WFLogger"( "WFSQLiteCorruptionLogger" )

--------------------------------------------------------------------------------

local _wfSQLiteCorruptionLoggerThread

--------------------------------------------------------------------------------

local function getWFSQLiteCorruptionLoggerThread()

	_wfSQLiteCorruptionLoggerThread = 
		_wfSQLiteCorruptionLoggerThread or
		WFWorkerThread( "WFSQLiteCorruptionLoggerImp.lua" )

	return _wfSQLiteCorruptionLoggerThread

end

--------------------------------------------------------------------------------

local function logCorruptionEvent( errorType, path, msg, sql )

	WFRxRunUntilDone( 
		getWFSQLiteCorruptionLoggerThread()
			:request( "logCorruptionEvent",
						errorType, path, msg, sql ) )
			
end

--------------------------------------------------------------------------------

local function addDb( wfSQLiteDBImpInstance )

	WFArguments.check( "userdata" )

	local hadHandlers = 
			wfSQLiteDBImpInstance
				:setFatalErrorHook( logCorruptionEvent );

	if hadHandlers then

		logger:warnf( "Clobbered existing error handlers on %s", 
						wfSQLiteDBImpInstance )

	end

	return wfSQLiteDBImpInstance

end

--------------------------------------------------------------------------------

local function getCorruptionEventStream()

	return getWFSQLiteCorruptionLoggerThread()
			:request( "getCorruptionEventStream" )

end

--==============================================================================

return {

	addDb = addDb,

	getCorruptionEventStream = getCorruptionEventStream,

}
