precision highp float;

fshin vec2 vTexCoord;

uniform sampler2D uSampler;

// Lut broken ...
#define Lut 0

#if Lut
uniform sampler2D uUndoRenderCurve;
#endif

#if GLES3
out vec4 fragColor;
#else
#define fragColor gl_FragColor
#endif

vec3 evalUndoRenderCurve(vec3 col)
{
    vec2 fMinMax;
    vec2 nMinMax;
    
    float eps = 0.00001;
    
    
    fMinMax.x = min(min(col.r,col.g),col.b);
    fMinMax.y = max(max(col.r,col.g),col.b);
    
    vec2 t = pow(fMinMax,vec2(3.14453125));
    
    nMinMax = pow(fMinMax,vec2(0.8125))*0.3828125*(1.0-t) + (1.0-pow(1.0-fMinMax,vec2(0.69140625)))*t;
    
    fMinMax.y = (nMinMax.y-nMinMax.x)/(fMinMax.y-fMinMax.x+eps);
    
    return (col-fMinMax.x)*fMinMax.y + nMinMax.x;
}

vec2 urc( vec2 x )
{
    vec2 x2 = x*x;
    vec2 x3 = x2*x;
    vec2 x4 = x3*x;
    vec2 x5 = x4*x;
    vec2 x6 = x5*x;
    vec2 x7 = x6*x;
    vec2 x8 = x7*x;
    vec2 x9 = x8*x;
    vec2 x10 = x9*x;
    return 368.6 * x10  - 1629.0 * x9 + 3032.0 * x8 - 3080.0 * x7 + 1842.0 * x6 - 646.5 * x5 + 119.7 * x4 - 5.115 * x3 - 2.119 * x2 + 0.7675 * x + 0.003588;
}

void main()
{
    vec3 col = clamp( texture( uSampler, vec2( vTexCoord.s, vTexCoord.t ) ).xyz, 0.0, 1.0 );

    // RGB linear -> Prophoto
    const mat3 rgb2proPhotoMatrix = mat3(0.52932378745827813, 0.098426540365603984, 0.016845769297481432,
                                  0.33005506400369466, 0.87350872697710469, 0.11769549131351224,
                                  0.14064019169871309, 0.028111560975330636, 0.86547398998538982);

#if Lut
    const float eps = 1.0/255.0;
    // Undo render curve
    col = rgb2proPhotoMatrix * col;
    vec2 minmax0 = vec2( min( min( col.r, col.g ), col.b ), max( max( col.r, col.g ), col.b ) );
    vec2 minmax1 = vec2( texture( uUndoRenderCurve, vec2( minmax0.x, 0.0 ) ).x, texture( uUndoRenderCurve, vec2( minmax0.y, 0.0 ) ).x );
    col = (minmax1.y - minmax1.x) * (col - minmax0.x) / (minmax0.y - minmax0.x + eps ) + minmax1.x;
#else
    col = evalUndoRenderCurve( clamp( rgb2proPhotoMatrix * col, 0.0, 1.0 ) );
#endif
    
    // !!! Ignore white balance for now !!!

    // To grey
    float lum = max( dot( col, vec3( 0.30, 0.59, 0.11 ) ), 0.0 );

    const float kMaxFlareLinear_real64 = 0.000244140625;    // ~ 2^-12

    fragColor = vec4( vec3( log2( max( lum + kMaxFlareLinear_real64, 0.0 ) ) ), 1.0 );
}
