#extension GL_EXT_shader_framebuffer_fetch : require

precision highp float;

fshin vec2 vTexCoord;

uniform sampler2D uGp;              // gaussian level
uniform sampler2D uLp;              // laplacian level

uniform float uReference;           // Value in log space at which image was tone mapped
uniform float uInv_ds;              // Inverse of difference in stops between samples, "stops per sample"

#if GLES3
layout(location = 0) inout highp vec4 fragColor;
#define lastFragData fragColor
#else
#define fragColor gl_FragColor
#define lastFragData gl_LastFragData[0]
#endif

void main()
{
    float g = texture( uGp, vTexCoord ).x;   // Its grey => just care about one of the 3 identical channels
    
    // Triangle function blending between samples: ___/\__
    float line0 = uInv_ds * (g - uReference) + 1.0;   // for da/ds: da always 1
    float line1 = uInv_ds * (uReference - g) + 1.0;   // for da/ds: da always 1
    float alpha = max( min( line0, line1 ), 0.0 );
    
    fragColor = vec4( vec3( alpha * texture( uLp, vTexCoord ).x + lastFragData.x ), 1.0 );
}
